/*****************************************************************************/
/*                        WiNRADiO G39DDC EXCELSIOR                          */
/*                                SDK DEMO                                   */
/*                   Interlayer between G39DDC API and GUI                   */
/*                        (C) Copyright RADIXON 2014                         */
/*****************************************************************************/

#ifndef __U_RECEIVER_H__
#define __U_RECEIVER_H__

#include <wx/wx.h>
#include <wx/thread.h>
#include "g39ddcapi.h"
#include "u_defs.h"
#include "u_types.h"
#include "u_fft.h"


//minimam DDC1/DDC2 BW which allows FMW mode
#define MIN_FMW_DDC2_BW             100000
//number of supported demodulator modes
#define DEM_MODE_COUNT              G39DDC_MODE_COUNT

//MUTE_BITS_MANUAL - mute bit, set when manual mute (Mute button in GUI)
#define MUTE_BITS_MANUAL            0x00000001
//MUTE_BITS_INACTIVE_CHANNEL - mute bit, set when given channel is inactive (RX1/RX2 buttons in GUI)
#define MUTE_BITS_INACTIVE_CHANNEL  0x00000002
//MUTE_BITS_SQUELCH - mute bit, set when squelched
#define MUTE_BITS_SQUELCH           0x00000004


//SWP_STEP - frequency step of start and end frequency of sweeper
#define SWP_STEP                    1000000

//AGC modes
#define AGC_OFF                     0
#define AGC_SLOW                    1
#define AGC_MEDIUM                  2
#define AGC_FAST                    3


//receiver event codes
typedef enum
{
    ReceiverEvent_Power=0,              //power state changed
    ReceiverEvent_HF_UHF,               //front-end mode changed from HF to VHF/UHF or vice versa
    ReceiverEvent_Attenuator,           //HF attenuator changed
    ReceiverEvent_Preamplifier,         //VHF/UHF preamplifier state changed
    ReceiverEvent_FrontEndFrequency,    //front-end frequency changed
    ReceiverEvent_IFRBWIndex,           //IF spectrum RBW changed
    ReceiverEvent_ActiveChannel,        //active channel (RX1/RX2) changed
    ReceiverEvent_SweeperStarted,       //sweeping started
    ReceiverEvent_SweeperStopped,       //sweeping stopped
} TReceiverEvent;

//channel related event codes
typedef enum
{
    ChannelEvent_DDC1=0,                //DDC1 BW (DDC type) changed
    ChannelEvent_DDC1_RBWList,          //DDC1 spectrum RBW list changed
    ChannelEvent_DDC1_RBWIndex,         //DDC1 spectrum RBW changed
    ChannelEvent_DDC1_Frequency,        //DDC1 relative frequency changed
    ChannelEvent_DDC1_AbsFrequency,     //DDC1 absolute frequency changed

    ChannelEvent_DDC2,                  //DDC2 BW (DDC type) changed
    ChannelEvent_DDC2_List,             //DDC2 BW list (list of allowed DDC types) changed
    ChannelEvent_DDC2_RBWList,          //DDC2 spectrum RBW list changed
    ChannelEvent_DDC2_RBWIndex,         //DDC2 spectrum RBW changed
    ChannelEvent_DDC2_Frequency,        //DDC2 relative frequency changed
    ChannelEvent_DDC2_AbsFrequency,     //DDC2 absolute frequency changed

    ChannelEvent_DEM_Frequency,         //demod relative frequency changed
    ChannelEvent_DEM_AbsFrequency,      //demod absolute frequency changed
    ChannelEvent_DEM_Bandwidth,         //demod filter bandwidth changed
    ChannelEvent_DEM_Mode,              //demod mode changed
    ChannelEvent_DEM_AMSSideband,       //AMS side band changed
    ChannelEvent_DEM_DSBSideband,       //DSB side band changed
    ChannelEvent_DEM_ISBSideband,       //ISB side band changed
    ChannelEvent_DEM_CWTone,            //CW tone frequency changed
    ChannelEvent_DEM_FMWStereo,         //FMW stereo decoder enabled/disabled
    ChannelEvent_DEM_FMWDeemphasis,     //FMW de-emphasis changed

    ChannelEvent_Audio,                 //audio sample rate (ASR) changed
    ChannelEvent_Audio_RBWList,         //audio spectrum RBW list changed
    ChannelEvent_Audio_RBWIndex,        //audio spectrum RBW changed
    ChannelEvent_Audio_Volume,          //volume changed
    ChannelEvent_Audio_Mute,            //mute/unmode
    ChannelEvent_Audio_Squelch,         //squelch state changed
    ChannelEvent_Audio_Filter,          //audio filter enabled/disabled
    ChannelEvent_Audio_FilterParams,    //audio filter params changed

    ChannelEvent_AGC,                   //AGC changed
    ChannelEvent_Gain                   //gain changed

} TChannelEvent;


//callbacks, GUI implements the following methods to receive events, frequency spectrums etc. from TReceiver object
class TReceiverCallback
{
public:
    virtual void ReceiverCallback_Event(TReceiverEvent Event)=0;
    virtual void ReceiverCallback_ChannelEvent(uint32_t Channel,TChannelEvent Event)=0;
    virtual void ReceiverCallback_DDC1Spectrum(uint32_t Channel,float *Data,uint32_t Count,double Range,double Offset)=0;
    virtual void ReceiverCallback_DDC2Spectrum(uint32_t Channel,float *Data,uint32_t Count,double Range,double Offset)=0;
    virtual void ReceiverCallback_AudioSpectrum(uint32_t Channel,float *Data,uint32_t Count,uint32_t Range)=0;
    virtual void ReceiverCallback_IFSpectrum(float *Data,uint32_t Count,double Range,double Offset)=0;
    virtual void ReceiverCallback_Audio(uint32_t Channel,const float *Samples,uint32_t Count)=0;
    virtual void ReceiverCallbask_Sweeper(float *Data,uint32_t Count)=0;
    virtual bool ReceiverCallback_SweeperQueryEnd(void)=0;
};

//channel data object
class TChannel
{
public:
    uint32_t            m_DDC1_Index;           //current DDC type index of DDC1
    G39DDC_DDC_INFO     m_DDC1_Info;            //current DDC type info of DDC1
    uint32_t            m_DDC1_Count;           //number of available DDC types of DDC1
    int32_t             m_DDC1_Frequency;       //relative DDC1 center frequency
    int64_t             m_DDC1_AbsFrequency;    //absolute DDC1 center frequency
    uint32_t            m_DDC1_RBW_Index;       //index of current DDC1 spectrum RBW
    uint32_t            m_DDC1_FFT_Size;        //DDC1 FFT size
    float               m_DDC1_FFT_Buffer[MAX_FFT_SIZE*2];  //buffer used by DDC1 FFT
    float               m_DDC1_FFT_Coeffs[MAX_FFT_SIZE];    //DDC1 FFT coeffs (normalization coeffs * window coeffs)
    TFFTBuffer          m_DDC1_FFT_OverlappingBuffer;       //DDC1 FFT overlapping buffer
    float               m_DDC1_SpectrumCompensation[MAX_FFT_SIZE]; //DDC1 spectrum compensation data
    wxMutex             m_DDC1_FFT_Mutex;
    double              m_DDC1_FFT_Range;   //total frequency range of DDC1 frequency spectrum (DDC1RBW*(LastBin-FirstBin+1)) sent to GUI
    double              m_DDC1_FFT_Offset;  //offset of start of useful DDC1 band in total frequency range
    uint32_t            m_DDC1_FFT_FirstBin;//the first DDC1 FFT bin where useful band starts
    uint32_t            m_DDC1_FFT_BinCount;//the last DDC1 FFT bin where useful band ends
    uint32_t            m_DDC1_SamplesPerBuffer;//number of sample sets per DDC1 stream buffer

    uint32_t            m_DDC2_Index;       //current DDC type index of DDC2
    G39DDC_DDC_INFO     m_DDC2_Info;        //current DDC type info of DDC2
    uint32_t            m_DDC2_Count;       //number of available DDC types of DDC2
    int32_t             m_DDC2_Frequency;   //relative DDC2 center frequency
    int64_t             m_DDC2_AbsFrequency;//absolute DDC2 center frequency
    uint32_t            m_DDC2_RBW_Index;   //index of current DDC2 spectrum RBW
    uint32_t            m_DDC2_FFT_Size;    //DDC2 FFT size
    float               m_DDC2_FFT_Buffer[MAX_FFT_SIZE*2];//buffer used by DDC2 FFT
    float               m_DDC2_FFT_Coeffs[MAX_FFT_SIZE];//DDC2 FFT coeffs (normalization coeffs * window coeffs)
    TFFTBuffer          m_DDC2_FFT_OverlappingBuffer;//DDC2 FFT overlapping buffer
    float               m_DDC2_SpectrumCompensation[MAX_FFT_SIZE]; //DDC2 spectrum compensation data
    wxMutex             m_DDC2_FFT_Mutex;
    double              m_DDC2_FFT_Range;   //total frequency range of DDC2 frequency spectrum (DDC1RBW*(LastBin-FirstBin+1)) sent to GUI
    double              m_DDC2_FFT_Offset;  //offset of start of useful DDC1 band in total frequency range
    uint32_t            m_DDC2_FFT_FirstBin;//the first DDC2 FFT bin where useful band starts
    uint32_t            m_DDC2_FFT_BinCount;//the last DDC2 FFT bin where useful band ends

    int32_t             m_DEM_Frequency;    //demod relative frequency
    int64_t             m_DEM_AbsFrequency; //demod absolute frequency (displayed in frqeuency edit box, below RX1/RX2 buttons)
    uint32_t            m_DEM_Bandwidth;    //demod bandwidth
    uint32_t            m_DEM_Mode;         //current demod mode
    uint32_t            m_DEM_AMSSideband;  //AMS side band
    uint32_t            m_DEM_DSBSideband;  //DSB side band
    uint32_t            m_DEM_ISBSideband;  //ISB side band
    uint32_t            m_DEM_CWTone;       //CW tone frequency
    BOOL                m_DEM_FMWStereo;    //FMW stereo
    uint32_t            m_DEM_FMWDeemphasis;//FMW de-emphasis (micro-seconds)


    uint32_t            m_Audio_Index;      //index of current audio sample rate
    uint32_t            m_Audio_SampleRate; //current audio sample rate
    uint32_t            m_Audio_RBW_Index;  //index of current audio spectrum RBW
    wxMutex             m_Audio_FFT_Mutex;
    uint32_t            m_Audio_FFT_Size;   //audio spectrum FFT size
    float               m_Audio_FFT_Buffer[MAX_FFT_SIZE*2]; //buffer used by audio FFT
    float               m_Audio_FFT_Coeffs[MAX_FFT_SIZE]; //audio FFT coeffs (normalization coeffs * window coeffs)
    TFFTBuffer          m_Audio_FFT_OverlappingBuffer; //audio spectrum FFT overlapping buffer
    uint32_t            m_Audio_MuteBits;   //mute bits, when non-zero audio is muted
    uint32_t            m_Audio_SamplesPerBuffer; //number of sample sets be audio stream buffer

    bool                m_Audio_Filter;     //audio filter enabled/disabled
    uint32_t            m_Audio_Filter_CutOffLow;   //audio filter cut-off low frequency
    uint32_t            m_Audio_Filter_CutOffHigh;  //audio filter cut-off high frequency
    double              m_Audio_Filter_Deemphasis;  //audio filter deemphasis

    uint32_t            m_AGC;          //AGC
    int32_t             m_Gain;         //gain

    inline TChannel(void):m_DDC1_FFT_OverlappingBuffer(MAX_FFT_SIZE),m_DDC1_FFT_Mutex(wxMUTEX_RECURSIVE),
        m_DDC2_FFT_OverlappingBuffer(MAX_FFT_SIZE),m_DDC2_FFT_Mutex(wxMUTEX_RECURSIVE),m_Audio_FFT_Mutex(wxMUTEX_RECURSIVE),
        m_Audio_FFT_OverlappingBuffer(MAX_FFT_SIZE)
    {
    }
};

class TReceiver:public IG39DDCDeviceCallback
{
public:
    inline uint32_t GetDemModeCount(void) const
    {
        return DEM_MODE_COUNT; //except DRM mode
    }

    wxString GetDemModeName(uint32_t Mode);

    void SetDemMode(uint32_t Channel,uint32_t Mode);

    inline uint32_t GetDemMode(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_Mode;
    }

    inline float GetSignalLevel(uint32_t Channel)
    {
     float RMS;

        m_Device->GetSignalLevel(Channel,NULL,&RMS);

        return RMS;
    }

    inline void GetDDCInfo(uint32_t TypeIndex,G39DDC_DDC_INFO *Info)
    {
        m_Device->GetDDCInfo(TypeIndex,Info);
    }

    inline uint32_t GetDDC1Count(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC1_Count;
    }

    void SetDDC1(uint32_t Channel,uint32_t TypeIndex);

    inline uint32_t GetDDC1(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC1_Index;
    }
    inline G39DDC_DDC_INFO GetDDC1Info(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC1_Info;
    }

    void GetDDC1RBWs(uint32_t Channel,double *RBWs);

    inline uint32_t GetDDC1RBWIndex(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC1_RBW_Index;
    }
    void SetDDC1RBWIndex(uint32_t Channel,uint32_t RBWIndex);

    inline int64_t GetDDC1AbsFrequency(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC1_AbsFrequency;
    }

    inline int64_t GetDDC1Frequency(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC1_Frequency;
    }

    void SetDDC1Frequency(uint32_t Channel,int32_t Frequency);

    inline uint32_t GetDDC2Count(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC2_Count;
    }

    void SetDDC2(uint32_t Channel,uint32_t TypeIndex);

    inline uint32_t GetDDC2(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC2_Index;
    }

    inline G39DDC_DDC_INFO GetDDC2Info(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC2_Info;
    }

    void GetDDC2RBWs(uint32_t Channel,double *RBWs);

    inline uint32_t GetDDC2RBWIndex(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC2_RBW_Index;
    }

    void SetDDC2RBWIndex(uint32_t Channel,uint32_t RBWIndex);

    inline int32_t GetDDC2Frequency(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC2_Frequency;
    }

    void SetDDC2Frequency(uint32_t Channel,int32_t Frequency);

    inline int64_t GetDDC2AbsFrequency(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DDC2_AbsFrequency;
    }

    inline int32_t GetDemFrequency(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_Frequency;
    }

    void SetDemFrequency(uint32_t Channel,int32_t Frequency);

    inline int64_t GetDemAbsFrequency(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_AbsFrequency;
    }

    void SetDemAbsFrequency(uint32_t Channel,int64_t Frequency);

    inline uint32_t GetDemBandwidth(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_Bandwidth;
    }

    void SetDemBandwidth(uint32_t Channel,uint32_t Bandwidth);

    inline uint32_t GetDemAMSSideband(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_AMSSideband;
    }

    void SetDemAMSSideband(uint32_t Channel,uint32_t Sideband);

    inline uint32_t GetDemDSBSideband(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_DSBSideband;
    }

    void SetDemDSBSideband(uint32_t Channel,uint32_t Sideband);

    inline uint32_t GetDemISBSideband(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_ISBSideband;
    }

    void SetDemISBSideband(uint32_t Channel,uint32_t Sideband);

    inline uint32_t GetDemCWTone(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_CWTone;
    }

    void SetDemCWTone(uint32_t Channel,uint32_t Frequency);

    inline bool GetDemFMWStereo(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_FMWStereo!=FALSE;
    }

    void SetDemFMWStereo(uint32_t Channel,bool Stereo);

    inline uint32_t GetDemFMWDeemphasis(uint32_t Channel) const
    {
        return m_Channel[Channel].m_DEM_FMWDeemphasis;
    }

    void SetDemFMWDeemphasis(uint32_t Channel,uint32_t Deemphasis);

    inline int32_t GetDemTuneError(uint32_t Channel)
    {
     int32_t Error;

        m_Device->GetDemodulatorState(Channel,G39DDC_DEMODULATOR_STATE_TUNE_ERROR,&Error,sizeof(Error));

        return Error;
    }

    inline double GetAMDepth(uint32_t Channel)
    {
     double AMDepth;

        m_Device->GetDemodulatorState(Channel,G39DDC_DEMODULATOR_STATE_AM_DEPTH,&AMDepth,sizeof(AMDepth));

        return AMDepth;
    }

    inline uint32_t GetDeviation(uint32_t Channel)
    {
     uint32_t Deviation;

        m_Device->GetDemodulatorState(Channel,G39DDC_DEMODULATOR_STATE_FM_DEVIATION,&Deviation,sizeof(Deviation));

        return Deviation;
    }

    inline bool IsDemAMSLocked(uint32_t Channel)
    {
     BOOL Locked;

        m_Device->GetDemodulatorState(Channel,G39DDC_DEMODULATOR_STATE_AMS_LOCK,&Locked,sizeof(Locked));

        return Locked!=FALSE;
    }

    inline bool IsDemDSBLocked(uint32_t Channel)
    {
     BOOL Locked;

        m_Device->GetDemodulatorState(Channel,G39DDC_DEMODULATOR_STATE_DSB_LOCK,&Locked,sizeof(Locked));

        return Locked!=FALSE;
    }

    inline bool IsDemFMWStereo(uint32_t Channel)
    {
     BOOL Stereo;

        m_Device->GetDemodulatorState(Channel,G39DDC_DEMODULATOR_STATE_FMW_STEREO,&Stereo,sizeof(Stereo));

        return Stereo!=FALSE;
    }

    inline double GetCurrentGain(uint32_t Channel)
    {
     double Gain;

        m_Device->GetCurrentGain(Channel,&Gain);

        return Gain;
    }

    inline uint32_t GetAudioSampleRateCount(void) const
    {
        uint32_t Count;

        m_Device->GetAudioSampleRateCount(&Count);

        return Count;
    }

    inline uint32_t GetAudioSampleRate(uint32_t Index) const
    {
        uint32_t SampleRate;

        m_Device->GetAudioSampleRate(Index,&SampleRate);

        return SampleRate;
    }

    inline uint32_t GetAudio(uint32_t Channel,uint32_t *SampleRate) const
    {
        if(SampleRate)
        {
            *SampleRate=m_Channel[Channel].m_Audio_SampleRate;
        }

        return m_Channel[Channel].m_Audio_Index;
    }

    void SetAudio(uint32_t Channel,uint32_t Index);

    void GetAudioRBWs(uint32_t Channel,double *RBWs);

    inline uint32_t GetAudioRBWIndex(uint32_t Channel) const
    {
        return m_Channel[Channel].m_Audio_RBW_Index;
    }
    void SetAudioRBWIndex(uint32_t Channel,uint32_t RBWIndex);

    inline uint8_t GetVolume(uint32_t Channel) const
    {
        uint8_t Volume;

        m_Device->GetVolume(Channel,&Volume);

        return Volume;
    }

    void SetVolume(uint32_t Channel,uint8_t Volume);

    inline bool GetMute(uint32_t Channel) const
    {
        return (m_Channel[Channel].m_Audio_MuteBits & MUTE_BITS_MANUAL)!=0;
    }

    void SetMute(uint32_t Channel,bool Mute);

    inline bool GetSquelched(uint32_t Channel) const
    {
        return (m_Channel[Channel].m_Audio_MuteBits & MUTE_BITS_SQUELCH)!=0;
    }

    void SetSquelched(uint32_t Channel,bool Squelched);


    inline bool GetAudioFilter(uint32_t Channel) const
    {
        return m_Channel[Channel].m_Audio_Filter;
    }

    void SetAudioFilter(uint32_t Channel,bool Enabled);

    inline void GetAudioFilterParams(uint32_t Channel,uint32_t *Low,uint32_t *High,double *Deem)
    {
        *Low=m_Channel[Channel].m_Audio_Filter_CutOffLow;
        *High=m_Channel[Channel].m_Audio_Filter_CutOffHigh;
        *Deem=m_Channel[Channel].m_Audio_Filter_Deemphasis;
    }

    void SetAudioFilterParams(uint32_t Channel,uint32_t Low,uint32_t High,double Deem);

    inline uint32_t GetAGC(uint32_t Channel) const
    {
        return m_Channel[Channel].m_AGC;
    }

    void SetAGC(uint32_t Channel,uint32_t AGC);

    inline int32_t GetGain(uint32_t Channel) const
    {
        return m_Channel[Channel].m_Gain;
    }

    void SetGain(uint32_t Channel,int32_t Gain);

    inline uint32_t GetActiveChannel(void) const
    {
        return m_ActiveChannel;
    }
    void SetActiveChannel(uint32_t Channel);

    inline uint16_t GetADCLevel(void) const
    {
        return m_ADCLevel;
    }

    inline bool GetHF(void) const
    {
        return m_HF;
    }

    inline uint32_t GetIFRBWIndex(void) const
    {
        return m_IF_RBW_Index;
    }

    void SetIFRBWIndex(uint32_t Index);

    void GetIFRBWs(double *RBWs); //RBWs is pointer to array with RBW_COUNT elements

    inline int64_t GetFrontEndFrequency(void) const
    {
        return m_FrontEndFrequency;
    }

    inline bool GetSweeper(void) const
    {
        return m_Sweeper;
    }

    void StartSweeper(int64_t Start,int64_t End,uint32_t RBWIndex);
    void StopSweeper(void);
    void GetSweeperParams(int64_t *Start,int64_t *End,double *RBW);

    void FrontEndFrequencyUp(void);
    void FrontEndFrequencyDown(void);

    void SetPreamplifier(bool Preamp);
    inline bool GetPreamplifier(void) const
    {
        return m_Preamp;
    }

    void SetAttenuator(uint32_t Attenuator);
    inline uint32_t GetAttenuator(void) const
    {
        return m_Attenuator;
    }

    void SetPower(bool Power);
    inline bool GetPower(void) const
    {
        return m_Power;
    }

    bool GetPowerDirect(void);

    inline bool IsConnected(void)
    {
        return m_Device->IsConnected()!=FALSE;
    }

    inline void GetDeviceInfo(G39DDC_DEVICE_INFO &Info) const
    {
        Info=m_DeviceInfo;
    }

    inline const G39DDC_DEVICE_INFO& GetDeviceInfo(void) const
    {
        return m_DeviceInfo;
    }

    inline void SetCallback(TReceiverCallback *Callback)
    {
        m_Callback=Callback;
    }

    bool Open(const char *DevicePath);
    void Close(void);

    TReceiver(void);
    virtual ~TReceiver(void);
private:
    IG39DDCDevice           *m_Device;      //G39DDC device object interface
    TReceiverCallback       *m_Callback;    //callback
    int                     m_Ignore;       //when non-zero, all the set methods are ignored
    G39DDC_DEVICE_INFO      m_DeviceInfo;   //device info
    bool                    m_Power;        //power state
    int64_t                 m_FrontEndFrequency;   //current front-end frequency
    bool                    m_HF;           //if non-zero front-end is in HF mode, otherwide it is in VHF/UHF mode
    uint32_t                m_Attenuator;   //HF attenuator
    bool                    m_Preamp;       //VHF/UHF preamplifier
    TChannel                *m_Channel;     //channels data
    uint32_t                m_ActiveChannel;    //index of active channel (RX1/RX2)

    uint32_t                m_IF_RBW_Index; //index of IF spectrum RBW
    uint32_t                m_IF_FFT_Size;  //IF spectrum FFT size
    float                   m_IF_FFT_Buffer[MAX_IF_FFT_SIZE*2]; //buffer used by IF spectrum FFT
    float                   m_IF_FFT_Coeffs[MAX_IF_FFT_SIZE]; //IF spectrum FFT coeffs (normalization coeffs * window coeffs * 1/32768)
    float                   m_IF_SpectrumCompensation[MAX_IF_FFT_SIZE/2]; //IF spectrum compensation data
    wxMutex                 m_IF_FFT_Mutex;

    uint16_t                m_ADCLevel;     //the latest ADC level (0..32767), 32767 = ADC clipping

    bool                    m_Sweeper;      //sweeper state
    int64_t                 m_Sweeper_StartFrequency;   //sweeper start frequency
    int64_t                 m_Sweeper_EndFrequency;     //sweeper end frequency
    uint32_t                m_Sweeper_FFT_Size;         //sweeper FFT size
    double                  m_Sweeper_FFT_RBW;          //sweeper RBW
    float                   m_Sweeper_FFT_Buffer[MAX_IF_FFT_SIZE*2];    //buffer for sweeper FFT
    float                   m_Sweeper_FFT_Coeffs[MAX_IF_FFT_SIZE];      //sweeper coeffs (normalization coeffs * window coeffs * 1/32768)
    float                   m_Sweeper_SpectrumCompensation[MAX_IF_FFT_SIZE/2]; //sweeper IF spectrum compensation data
    int64_t                 m_Sweeper_FirstFrontEndFrequency; //the first front-end window frequency which includes sweeper start frequency
    int64_t                 m_Sweeper_LastFrontEndFrequency;  //the last front-end window frequency which includes sweeper end frequency
    int64_t                 m_Sweeper_FFT_RestBinCount; //rest number of FFT bins of total sweep
    double                  m_Sweeper_NextWindowStartFrequency; //absolute frequency of next front-end windows start


    static const wxChar    *m_ModeNames[DEM_MODE_COUNT];

    void G39DDC_CALLBACK G39DDC_DDC1StreamCallback(IG39DDCDevice *Device,uint32_t Channel,const void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample);
    BOOL G39DDC_CALLBACK G39DDC_DDC1PlaybackStreamCallback(IG39DDCDevice *Device,uint32_t Channel,void *Buffer,uint32_t NumberOfSamples,uint32_t BitsPerSample);
    void G39DDC_CALLBACK G39DDC_DDC2StreamCallback(IG39DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples);
    void G39DDC_CALLBACK G39DDC_DDC2PreprocessedStreamCallback(IG39DDCDevice *Device,uint32_t Channel,const float *Buffer,uint32_t NumberOfSamples,float SlevelPeak,float SlevelRMS);
    void G39DDC_CALLBACK G39DDC_IFCallback(IG39DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint16_t MaxADCAmplitude,uint32_t ADCSampleRate);
    BOOL G39DDC_CALLBACK G39DDC_SweepingCallback(IG39DDCDevice *Device,const int16_t *Buffer,uint32_t NumberOfSamples,uint32_t CenterFrequency,uint64_t FrontEndFrequency);
    void G39DDC_CALLBACK G39DDC_AudioStreamCallback(IG39DDCDevice *Device,uint32_t Channel,uint32_t Type,const float *Buffer,uint32_t NumberOfSamples);
    BOOL G39DDC_CALLBACK G39DDC_AudioPlaybackStreamCallback(IG39DDCDevice *Device,uint32_t Channel,float *Buffer,uint32_t NumberOfSamples);



    void InvokeEvent(TReceiverEvent Event);
    void InvokeEvent(uint32_t Channel,TChannelEvent Event);

    void OnPowerChanged(void);
    void OnAttenuatorChanged(void);
    void OnPreaplifierChanged(void);
    void OnFrontEndFrequencyChanged(void);
    void OnIFRBWChanged(void);
    void UpdateIFFFTCoeffs(void);
    void UpdateSpectrumCompensationData(void);
    void UpdateIFSpectrumCompensationData(void);

    void OnDDC1Changed(uint32_t Channel);
    void OnDDC1FrequencyChanged(uint32_t Channel);
    void OnDDC1AbsFrequencyChanged(uint32_t Channel);
    void UpdateDDC1SpectrumCompensationData(uint32_t Channel);
    void UpdateDDC1FFTCoeffs(uint32_t Channel);
    void StopDDC1(uint32_t Channel);

    void OnDDC2Changed(uint32_t Channel);
    void OnDDC2CountChanged(uint32_t Channel);
    void OnDDC2FrequencyChanged(uint32_t Channel);
    void OnDDC2AbsFrequencyChanged(uint32_t Channel);
    void UpdateDDC2SpectrumCompensationData(uint32_t Channel);
    void UpdateDDC2FFTCoeffs(uint32_t Channel);
    void StopDDC2(uint32_t Channel);

    void GetDemBandwidthMinMax(uint32_t Channel,uint32_t *Min,uint32_t *Max);
    void GetDemFrequencyMinMax(uint32_t Channel,int32_t *Min,int32_t *Max);
    void OnDemFrequencyChanged(uint32_t Channel);
    void OnDemAbsFrequencyChanged(uint32_t Channel);
    void OnDemBandwidthChanged(uint32_t Channel);
    void OnDemModeChanged(uint32_t Channel);

    void OnAudioChanged(uint32_t Channel);
    void UpdateAudioFFTCoeffs(uint32_t Channel);
    void SetAudioMuteBits(uint32_t Channel,uint32_t Bits);
    void ClearAudioMuteBits(uint32_t Channel,uint32_t Bits);
    void StopAudio(uint32_t Channel);

    void UpdateAGC(uint32_t Channel);

    void StartDDC1(uint32_t Channel);
    void StartDDC2(uint32_t Channel);
    void StartAudio(uint32_t Channel);
};

#endif
