/* ***********************************************************************/
/*                                                                       */
/*                      WiNRADiO G39DDC EXCELSIOR                        */
/*                 G39DDC device driver (PCIe and USB)                   */
/*                     (C) Copyright 2015 RADIXON.                       */
/*                                                                       */
/* ***********************************************************************/

#include "g39ddc.h"

static void g39_io_buffer_unmap(struct g39_io_buffer *io_buffer)
{
 int i;

#ifdef __G39_USB__
    if(io_buffer->mem!=NULL)
    {
        vunmap(io_buffer->mem);
        io_buffer->mem=NULL;
    }
#endif //__G39_USB__

    if(io_buffer->pages)
    {
        if(io_buffer->changed)
        {
            for(i=0;i<io_buffer->page_count;i++)
            {
                SetPageDirty(io_buffer->pages[i]);
                put_page(io_buffer->pages[i]);
            }
        }
        else
        {
            for(i=0;i<io_buffer->page_count;i++)
            {
                put_page(io_buffer->pages[i]);
            }
        }

        io_buffer->changed=0;

        kfree(io_buffer->pages);
        io_buffer->pages=NULL;
        io_buffer->page_count=0;
    }
}


static int g39_io_buffer_prepare(uintptr_t user_buffer,uint32_t size,int write,struct g39_io_buffer **io_buffer_r)
{
 struct g39_io_buffer *io_buffer;
 int page_count,locked_page_count,i;

    if(unlikely(user_buffer==0 || (user_buffer & (~PAGE_MASK))!=0 || size==0))
    {
        return -EINVAL;
    }

    page_count=(int)(((user_buffer+size-1)&PAGE_MASK)>>PAGE_SHIFT)-(int)(user_buffer>>PAGE_SHIFT)+1;

    io_buffer=kmalloc(sizeof(struct g39_io_buffer),GFP_KERNEL);

    if(unlikely(io_buffer==NULL))
    {
        return -ENOMEM;
    }

    io_buffer->pages=kmalloc(page_count*sizeof(struct page*),GFP_KERNEL);

    if(unlikely(io_buffer->pages==NULL))
    {
        kfree(io_buffer);
        return -ENOMEM;
    }

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31)
    locked_page_count=get_user_pages_fast((uintptr_t)user_buffer,page_count,write,io_buffer->pages);
#else
    down_read(&current->mm->mmap_sem);
    locked_page_count=get_user_pages(current,current->mm,(uintptr_t)user_buffer,page_count,write,1,io_buffer->pages,NULL);
    up_read(&current->mm->mmap_sem);
#endif

    if(unlikely(locked_page_count<0))
    {
        kfree(io_buffer->pages);
        kfree(io_buffer);
        return -EFAULT;
    }

    if(unlikely(locked_page_count!=page_count))
    {
        for(i=0;i<locked_page_count;i++)
        {
            put_page(io_buffer->pages[i]);
        }

        kfree(io_buffer->pages);
        kfree(io_buffer);
        return -EFAULT;
    }

    io_buffer->user_buffer=user_buffer;
    io_buffer->size=size;
    io_buffer->page_count=locked_page_count;
    io_buffer->changed=0;
#ifdef __G39_USB__
    io_buffer->mem=NULL;
#endif //__G39_USB__

#ifdef __G39_PCI__
    io_buffer->sg_list=NULL;
    io_buffer->sg_segment_count=0;
    io_buffer->sg_pci_dev=NULL;
#endif //__G39_PCI__

    *io_buffer_r=io_buffer;

    return 0;
}

#ifdef __G39_USB__

static int g39_io_buffer_map(struct g39_io_buffer *io_buffer)
{
    if(unlikely(io_buffer->mem!=NULL))
    {
        return 0;
    }

    io_buffer->mem=vmap(io_buffer->pages,io_buffer->page_count,0,PAGE_KERNEL);

    if(unlikely(io_buffer->mem==NULL))
    {
        return -ENOMEM;
    }

    return 0;
}

#endif //__G39_USB__

#ifdef __G39_PCI__

inline static void g39_io_buffer_sg_sync_for_cpu(struct g39_io_buffer *io_buffer)
{
    pci_dma_sync_sg_for_cpu(io_buffer->sg_pci_dev,io_buffer->sg_list,io_buffer->page_count,io_buffer->sg_direction);
}

inline static void g39_io_buffer_sg_sync_for_device(struct g39_io_buffer *io_buffer)
{
    pci_dma_sync_sg_for_device(io_buffer->sg_pci_dev,io_buffer->sg_list,io_buffer->page_count,io_buffer->sg_direction);
}

static int g39_io_buffer_sg_map(struct g39_io_buffer *io_buffer,struct pci_dev *pci_dev,int direction)
{
 struct scatterlist *sg_list;
 int i;
 int count;
 uint32_t size;

    sg_list=kcalloc(io_buffer->page_count,sizeof(*sg_list),GFP_KERNEL);

    if(unlikely(sg_list==NULL))
    {
        return -ENOMEM;
    }

    size=io_buffer->size;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
    sg_init_table(sg_list,io_buffer->page_count);
#endif

    for(i=0;i<io_buffer->page_count;i++)
    {
        if(likely(size>=PAGE_SIZE))
        {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
            sg_set_page(&sg_list[i],io_buffer->pages[i],PAGE_SIZE,0);
#else
            sg_list[i].page=io_buffer->pages[i];
            sg_list[i].offset=0;
            sg_list[i].length=PAGE_SIZE;
#endif

            size-=PAGE_SIZE;
        }
        else
        {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
            sg_set_page(&sg_list[i],io_buffer->pages[i],size,0);
#else
            sg_list[i].page=io_buffer->pages[i];
            sg_list[i].offset=0;
            sg_list[i].length=size;
#endif

            size=0;
        }
    }

    count=pci_map_sg(pci_dev,sg_list,io_buffer->page_count,direction);

    if(unlikely(count==0))
    {
        kfree(sg_list);
        return -ENOMEM;
    }

    io_buffer->sg_list=sg_list;
    io_buffer->sg_segment_count=count;
    io_buffer->sg_pci_dev=pci_dev;
    io_buffer->sg_direction=direction;

    return 0;
}

static void g39_io_buffer_sg_unmap(struct g39_io_buffer *io_buffer)
{
    if(io_buffer->sg_list)
    {
        g39_io_buffer_sg_sync_for_cpu(io_buffer);

        pci_unmap_sg(io_buffer->sg_pci_dev,io_buffer->sg_list,io_buffer->page_count,io_buffer->sg_direction);
        kfree(io_buffer->sg_list);

        io_buffer->sg_list=NULL;
    }
}

#endif //__G39_PCI__

static void g39_io_buffer_free(struct g39_io_buffer *io_buffer)
{
 int i;

#ifdef __G39_USB__
    if(io_buffer->mem!=NULL)
    {
        vunmap(io_buffer->mem);
    }
#endif //__G39_USB__

#ifdef __G39_PCI__
    g39_io_buffer_sg_unmap(io_buffer);
#endif //__G39_PCI__

    if(io_buffer->pages!=NULL)
    {
        if(io_buffer->changed)
        {
            for(i=0;i<io_buffer->page_count;i++)
            {
                if(!PageReserved(io_buffer->pages[i]))
                {
                    SetPageDirty(io_buffer->pages[i]);
                }

                put_page(io_buffer->pages[i]);
            }
        }
        else
        {
            for(i=0;i<io_buffer->page_count;i++)
            {
                put_page(io_buffer->pages[i]);
            }
        }

        kfree(io_buffer->pages);
    }

    kfree(io_buffer);
}

static int g39_io_stream_get_complete_buffer(struct g39ddc *dev,struct g39_io_stream *io_stream,uintptr_t *user_buffer,uint8_t swp_active)
{
 struct g39_io_buffer *io_buffer;
 unsigned long flags;
 struct g39_complete_io_buffers *buffers;

    buffers=&io_stream->complete_buffers;

    for(;;)
    {
        down_read(&io_stream->rw_semaphore);

        if(unlikely(!io_stream->active || !dev->swp_active!=!swp_active))
        {
            up_read(&io_stream->rw_semaphore);
            return -EPERM;
        }

        spin_lock_irqsave(&buffers->spinlock,flags);

        if(unlikely(list_empty(&buffers->list)))
        {
            io_buffer=NULL;
        }
        else
        {
            io_buffer=list_entry(buffers->list.next,struct g39_io_buffer,list_entry);

            list_del(&io_buffer->list_entry);
        }

        spin_unlock_irqrestore(&buffers->spinlock,flags);

        up_read(&io_stream->rw_semaphore);

        if(likely(io_buffer!=NULL))
        {
            break;
        }

        if(unlikely(down_interruptible(&buffers->semaphore)<0))
        {
            return -EINTR;
        }
    }

    *user_buffer=io_buffer->user_buffer;

    g39_io_buffer_free(io_buffer);

    return 0;
}

#ifdef __G39_USB__

static struct g39_io_buffer* g39_io_stream_get_pending_buffer(struct g39_io_stream *io_stream)
{
 unsigned long flags;
 struct g39_pending_io_buffers *buffers;
 struct g39_io_buffer *io_buffer;

    buffers=&io_stream->pending_buffers;

    spin_lock_irqsave(&buffers->spinlock,flags);

    if(unlikely(list_empty(&buffers->list)))
    {
        io_buffer=NULL;
    }
    else
    {
        io_buffer=list_entry(buffers->list.next,struct g39_io_buffer,list_entry);

        list_del(buffers->list.next);
    }

    spin_unlock_irqrestore(&buffers->spinlock,flags);

    return io_buffer;
}

#endif //__G39_USB__

#ifdef __G39_PCI__

static struct g39_io_buffer* g39_io_stream_get_pending_buffer_nolock(struct g39_io_stream *io_stream)
{
 struct g39_pending_io_buffers *buffers;
 struct g39_io_buffer *io_buffer;

    buffers=&io_stream->pending_buffers;

    if(unlikely(list_empty(&buffers->list)))
    {
        io_buffer=NULL;
    }
    else
    {
        io_buffer=list_entry(buffers->list.next,struct g39_io_buffer,list_entry);

        list_del(buffers->list.next);
    }

    return io_buffer;
}

#endif //__G39_PCI__

static void g39_io_stream_put_complete_buffer(struct g39_io_stream *io_stream,struct g39_io_buffer *io_buffer)
{
 unsigned long flags;
 struct g39_complete_io_buffers *buffers;

    buffers=&io_stream->complete_buffers;

    g39_io_buffer_unmap(io_buffer);

    spin_lock_irqsave(&buffers->spinlock,flags);

    list_add_tail(&io_buffer->list_entry,&buffers->list);

    up(&buffers->semaphore);

    spin_unlock_irqrestore(&buffers->spinlock,flags);
}

#ifdef __G39_USB__

static void g39_io_stream_put_pending_buffer(struct g39_io_stream *io_stream,struct g39_io_buffer *io_buffer)
{
 unsigned long flags;
 struct g39_pending_io_buffers *buffers;

    buffers=&io_stream->pending_buffers;

    spin_lock_irqsave(&buffers->spinlock,flags);

    list_add_tail(&io_buffer->list_entry,&buffers->list);

    spin_unlock_irqrestore(&buffers->spinlock,flags);
}

static int g39_io_stream_put_pending_buffer2(struct g39_io_stream *io_stream,struct g39_io_buffer *io_buffer)
{
 unsigned long flags;
 struct g39_pending_io_buffers *buffers;
 int first;

    buffers=&io_stream->pending_buffers;

    spin_lock_irqsave(&buffers->spinlock,flags);

    first=list_empty(&buffers->list);

    list_add_tail(&io_buffer->list_entry,&buffers->list);

    spin_unlock_irqrestore(&buffers->spinlock,flags);

    return first;
}

#endif //__G39_USB__

inline static void g39_io_stream_put_pending_buffer_nolock(struct g39_io_stream *io_stream,struct g39_io_buffer *io_buffer)
{
    list_add_tail(&io_buffer->list_entry,&io_stream->pending_buffers.list);
}

static void g39_io_stream_free_buffers(struct g39_io_stream *io_stream)
{
 struct g39_io_buffer *io_buffer;

    while(!list_empty(&io_stream->pending_buffers.list))
    {
        io_buffer=list_entry(io_stream->pending_buffers.list.next,struct g39_io_buffer,list_entry);

        list_del(&io_buffer->list_entry);

        g39_io_buffer_free(io_buffer);
    }

    INIT_LIST_HEAD(&io_stream->pending_buffers.list);

    while(!list_empty(&io_stream->complete_buffers.list))
    {
        io_buffer=list_entry(io_stream->complete_buffers.list.next,struct g39_io_buffer,list_entry);

        list_del(&io_buffer->list_entry);

        g39_io_buffer_free(io_buffer);
    }

    INIT_LIST_HEAD(&io_stream->complete_buffers.list);
}

static dev_t g39_dev0;

static struct class g39_class=
{
    .name=G39_DEVICE_NAME
};

static struct g39ddc *g39_registered_devices[G39_MAX_DEVICE_COUNT]={0};

static struct g39_mutex g39_reg_mutex;

static int g39_set_power(struct g39ddc *dev,uint8_t power,uint8_t lock);

static int g39_fo_open(struct inode *node, struct file *file);
static int g39_fo_release(struct inode *node, struct file *file);
static int g39_fo_mmap(struct file *file,struct vm_area_struct *vma);

#ifdef HAVE_UNLOCKED_IOCTL
static long g39_fo_ioctl(struct file *file, unsigned int cmd, unsigned long arg);
#else
static int g39_fo_ioctl(struct inode *node,struct file *file, unsigned int cmd, unsigned long arg);
#endif

#if defined(CONFIG_COMPAT) && defined(HAVE_COMPAT_IOCTL)
static long g39_fo_ioctl_compat(struct file *file, unsigned int cmd, unsigned long arg);
#endif

static struct file_operations g39_file_operations=
{
    .open=g39_fo_open,
    .release=g39_fo_release,
#ifdef HAVE_UNLOCKED_IOCTL
    .unlocked_ioctl=g39_fo_ioctl,
#else
    .ioctl=g39_fo_ioctl,
#endif //HAVE_UNLOCKED_IOCTL

#if defined(CONFIG_COMPAT) && defined(HAVE_COMPAT_IOCTL)
    .compat_ioctl=g39_fo_ioctl_compat,
#endif //defined(CONFIG_COMPAT) && defined(HAVE_COMPAT_IOCTL)
    .mmap=g39_fo_mmap,
};

#if LINUX_VERSION_CODE<KERNEL_VERSION(4,15,0)
static void g39_wd_timer(unsigned long arg)
{
 struct g39ddc *dev=(struct g39ddc*)arg;
#else
static void g39_wd_timer(struct timer_list *t)
{
 struct g39ddc *dev=from_timer(dev,t,wd_timer);
#endif

    queue_work(dev->workqueue,&dev->wd_work);
}

static void g39_wd_work_handler(struct work_struct *work)
{
 struct g39ddc *dev;
 int status;

    dev=container_of(work,struct g39ddc,wd_work);

    G39_DEV_LOCK;

    if(likely(dev->state.open))
    {
        if(dev->state.power)
        {
            status=dev->func_wd(dev);

            if(likely(status==0))
            {
                dev->wd_counter=0;
            }
            else if(status>0)
            {
                dev->wd_counter++;

                if(unlikely(dev->wd_counter>=24))
                {
                    dev->state.temperature=1;
                    g39_set_power(dev,0,0);
                    dev->wd_counter=0;
                }
            }
        }

        dev->wd_timer.expires=jiffies+2*HZ;
        add_timer(&dev->wd_timer);
    }

    G39_DEV_UNLOCK;
}

static void g39_initialize(struct g39ddc *dev)
{
 int i;

    dev->state.connected=1;
    g39_mutex_init(&dev->mutex);
    init_waitqueue_head(&dev->power_wait_queue);

    dev->last_power_off_time=current_kernel_time();

    for(i=0;i<G39_IO_STREAM_COUNT;i++)
    {
        init_rwsem(&dev->io_stream[i].rw_semaphore);

        INIT_LIST_HEAD(&dev->io_stream[i].complete_buffers.list);
        sema_init(&dev->io_stream[i].complete_buffers.semaphore,0);
        spin_lock_init(&dev->io_stream[i].complete_buffers.spinlock);

        INIT_LIST_HEAD(&dev->io_stream[i].pending_buffers.list);
        spin_lock_init(&dev->io_stream[i].pending_buffers.spinlock);
    }

    INIT_WORK(&dev->wd_work,g39_wd_work_handler);

#if LINUX_VERSION_CODE<KERNEL_VERSION(4,15,0)
    init_timer(&dev->wd_timer);

    dev->wd_timer.function=g39_wd_timer;
    dev->wd_timer.data=(unsigned long)dev;
#else
    timer_setup(&dev->wd_timer,g39_wd_timer,0);
#endif
}

static void g39_uninitialize(struct g39ddc *dev)
{
    g39_mutex_destroy(&dev->mutex);
}

static void g39_delete(struct g39ddc *dev)
{
    g39_set_power(dev,0,1);

    dev->func_delete(dev);

    g39_uninitialize(dev);

    kfree(dev);
}

static void g39_release_callback(struct device *dev)
{
 struct g39ddc *g39ddc;

    g39ddc=container_of(dev,struct g39ddc,dev);

    g39_delete(g39ddc);
}

static int g39_register(struct g39ddc *dev)
{
 int status;
 int index;
 dev_t dev_no;

    G39_REG_LOCK;

    for(index=0;index<G39_MAX_DEVICE_COUNT;index++)
    {
        if(g39_registered_devices[index]==NULL)
        {
            break;
        }
    }

    if(index>=G39_MAX_DEVICE_COUNT)
    {
        G39_REG_UNLOCK;

        G39_DEV_ERROR(dev->p_dev,"no free device slot available!\n");
        return -ENFILE;
    }

    dev->index=index;

    dev_no=MKDEV(MAJOR(g39_dev0),index);

    cdev_init(&dev->cdev,&g39_file_operations);

    dev->cdev.owner=THIS_MODULE;

    status=cdev_add(&dev->cdev,dev_no,1);

    if(unlikely(status<0))
    {
        G39_REG_UNLOCK;
        G39_DEV_ERROR(dev->p_dev,"char device registration failed!\n");
        return status;
    }

    memset(&dev->dev,0,sizeof(dev->dev));

    dev->dev.devt=dev_no;
    dev->dev.class=&g39_class;
    dev->dev.release=g39_release_callback;

#if LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,26)
    dev_set_name(&dev->dev,G39_DEVICE_NAME"%d",index);
#else
    snprintf(dev->dev.bus_id,sizeof(dev->dev.bus_id),G39_DEVICE_NAME"%d",index);
#endif

    status=device_register(&dev->dev);

    if(unlikely(status<0))
    {
        cdev_del(&dev->cdev);
        G39_REG_UNLOCK;
        G39_DEV_ERROR(dev->p_dev,"failed to register device!\n");
        return status;
    }

    g39_registered_devices[index]=dev;

    G39_REG_UNLOCK;

    return 0;
}

static void g39_unregister(struct g39ddc *dev)
{
    G39_REG_LOCK;

    if(likely(g39_registered_devices[dev->index]==dev))
    {
        g39_registered_devices[dev->index]=NULL;

        G39_REG_UNLOCK;

        cdev_del(&dev->cdev);
        device_unregister(&dev->dev);
    }
    else
    {
        G39_REG_UNLOCK;
    }
}

static int g39_set_power(struct g39ddc *dev,uint8_t power,uint8_t lock)
{
 int status;
 struct timespec time;
 int i;

    power=power?1:0;

    if(lock)
    {
        G39_DEV_LOCK;
    }

    if(dev->state.power==power)
    {
        status=0;
    }
    else
    {
        if(power)
        {
            dev->state.suspended=0;
            dev->state.temperature=0;

            time=current_kernel_time();
            time.tv_nsec-=500000000;

            if(time.tv_nsec<0)
            {
                time.tv_sec--;
                time.tv_nsec+=1000000000;
            }

            if((dev->last_power_off_time.tv_sec==0 && dev->last_power_off_time.tv_nsec==0) ||
               timespec_compare(&time,&dev->last_power_off_time)<0)
            {
                msleep(500);
            }
        }
        else
        {
            dev->func_if_stop(dev);
            dev->func_swp_stop(dev);
            dev->func_dac_stop(dev);

            for(i=0;i<G39_CHANNEL_COUNT;i++)
            {
                dev->func_ddc2_stop(dev,i);
                dev->func_ddc1_stop(dev,i);
                dev->func_ddc1_playback_stop(dev,i);
            }
        }

        status=dev->func_set_power(dev,power);

        if(likely(status>=0))
        {
            if(power)
            {
                dev->state.power=1;
                msleep(100);
            }
            else
            {
                dev->state.power=0;
                dev->state.fpga=0;
                dev->last_power_off_time=current_kernel_time();

                wake_up_all(&dev->power_wait_queue);
            }
        }
    }

    if(lock)
    {
        G39_DEV_UNLOCK;
    }

    return status;
}

static int g39_configure_fpga(struct g39ddc *dev,void __user *user_buffer,uint32_t size)
{
 int status;

    if(unlikely(size==0 || user_buffer==NULL))
    {
        return -EINVAL;
    }

    if(unlikely(!access_ok(VERIFY_READ,user_buffer,size)))
    {
        return -EFAULT;
    }

    G39_DEV_LOCK;

    if(unlikely(!dev->state.power))
    {
        G39_DEV_UNLOCK;
        return -EPERM;
    }

    status=dev->func_configure_fpga(dev,user_buffer,size);

    if(likely(status>=0))
    {
        dev->state.fpga=1;

        msleep(50);
    }
    else
    {
        G39_DEV_ERROR(dev->p_dev,"failed to initialize FPGA!\n");
    }

    G39_DEV_UNLOCK;

    return status;
}

static int g39_set_spi_regs(struct g39ddc *dev,struct g39_spi_register __user *user_buffer,uint16_t count,uint16_t id)
{
 int status;

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        status=dev->func_set_spi_regs(dev,user_buffer,count,id);
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static int g39_set_spi2_regs(struct g39ddc *dev,struct g39_spi2_register __user *user_buffer,uint16_t count,uint16_t id)
{
 int status;

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        status=dev->func_set_spi2_regs(dev,user_buffer,count,id);
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static int g39_set_noise_blanker(struct g39ddc *dev,uint16_t value)
{
 int status;

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        status=dev->func_set_noise_blanker(dev,value);
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static int g39_if_start(struct g39ddc *dev,uint16_t period)
{
 int status;

    if(unlikely(period==0))
    {
        return -EINVAL;
    }

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        if(unlikely(dev->io_stream[G39_IO_STREAM_IN_IF].active || dev->swp_active))
        {
            status=-EBUSY;
        }
        else
        {
            status=dev->func_if_start(dev,period);
        }
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static void g39_if_stop(struct g39ddc *dev)
{
    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        dev->func_if_stop(dev);
    }

    G39_DEV_UNLOCK;
}


inline static int g39_get_complete_buffer(struct g39ddc *dev,uint32_t stream_index,unsigned long arg,uint8_t swp_active)
{
 uintptr_t user_buffer;
 int status;

    if(unlikely(stream_index>=G39_IO_STREAM_COUNT))
    {
        return -EINVAL;
    }

    if(unlikely(!access_ok(VERIFY_WRITE,(void __user*)arg,sizeof(void*))))
    {
        return -EFAULT;
    }

    status=g39_io_stream_get_complete_buffer(dev,&dev->io_stream[stream_index],&user_buffer,swp_active);

    if(unlikely(status<0))
    {
        return status;
    }

    return __put_user(user_buffer,(uintptr_t __user*)arg);
}

#if defined(CONFIG_COMPAT) && defined(HAVE_COMPAT_IOCTL)

inline static int g39_get_complete_buffer_compat(struct g39ddc *dev,uint32_t stream_index,unsigned long arg,uint8_t swp_active)
{
 uintptr_t user_buffer;
 int status;

    if(unlikely(!access_ok(VERIFY_WRITE,(void __user*)compat_ptr(arg),sizeof(compat_uptr_t))))
    {
        return -EFAULT;
    }

    status=g39_io_stream_get_complete_buffer(dev,&dev->io_stream[stream_index],&user_buffer,swp_active);

    if(unlikely(status<0))
    {
        return status;
    }

    return __put_user(ptr_to_compat((void __user*)user_buffer),(compat_uptr_t __user*)compat_ptr(arg));
}

#endif //defined(CONFIG_COMPAT) && defined(HAVE_COMPAT_IOCTL)


static int g39_put_buffer(struct g39ddc *dev,uint32_t stream_index,void __user *user_buffer,int write,int swp_active)
{
 struct g39_io_buffer *io_buffer;
 struct g39_io_stream *io_stream;
 int status;

    io_stream=dev->io_stream+stream_index;

    down_read(&io_stream->rw_semaphore);

    if(unlikely(!io_stream->active || !dev->swp_active!=!swp_active))
    {
        up_read(&io_stream->rw_semaphore);
        return -EPERM;
    }

    status=g39_io_buffer_prepare((uintptr_t)user_buffer,io_stream->buffer_size,write,&io_buffer);

    if(unlikely(status))
    {
        up_read(&io_stream->rw_semaphore);
        return status;
    }

    status=dev->func_put_buffer(dev,stream_index,io_buffer);

    up_read(&io_stream->rw_semaphore);

    if(unlikely(status<0))
    {
        g39_io_buffer_free(io_buffer);
    }

    return status;
}

static int g39_get_adc_level(struct g39ddc *dev,uint16_t *level)
{
 int status;

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        status=dev->func_get_adc_level(dev,level);
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static int g39_ddc1_start(struct g39ddc *dev,uint32_t channel,struct g39_ddc1_params *params,uint32_t __user *user_coeffs,uint32_t buffer_size,uint32_t bytes_per_sec)
{
 int status;
 uint32_t *coeffs;
 struct g39_io_stream *io_stream;

    if(unlikely(buffer_size>G39_MAX_BUFFER_SIZE))
    {
        return -E2BIG;
    }

    coeffs=kmalloc(params->fir_len*sizeof(uint32_t)/2,GFP_KERNEL);

    if(unlikely(coeffs==NULL))
    {
        return -ENOMEM;
    }

    if(unlikely(copy_from_user(coeffs,user_coeffs,params->fir_len*sizeof(uint32_t)/2)!=0))
    {
        kfree(coeffs);
        return -EFAULT;
    }

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        io_stream=dev->io_stream+G39_IO_STREAM_IN_DDC1_0+channel;

        if(unlikely(io_stream->active || dev->swp_active))
        {
            status=-EBUSY;
        }
        else
        {
            io_stream->buffer_size=buffer_size;
            io_stream->bytes_per_sec=bytes_per_sec;

            dev->ddc1_params[channel]=*params;

            status=dev->func_ddc1_start(dev,channel,coeffs);
        }
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    kfree(coeffs);

    return status;
}

static void g39_ddc1_stop(struct g39ddc *dev,uint32_t channel)
{
    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        dev->func_ddc2_stop(dev,channel);
        dev->func_ddc1_stop(dev,channel);
    }

    G39_DEV_UNLOCK;
}

static int g39_ddc1_set_frequency(struct g39ddc *dev,uint32_t channel,uint32_t tuning_word)
{
 int status;

    G39_DEV_LOCK;

    if(likely(dev->state.fpga && dev->io_stream[G39_IO_STREAM_IN_DDC1_0+channel].active))
    {
        dev->ddc1_params[channel].tuning_word=tuning_word;

        status=dev->func_ddc1_update_params(dev,channel);
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static int g39_ddc2_start(struct g39ddc *dev,uint32_t channel,void __user *params_user)
{
 int status;
 struct g39_ddc2_start_params *params;
 struct g39_io_stream *io_stream;

    params=kmalloc(sizeof(struct g39_ddc2_start_params),GFP_KERNEL);

    if(unlikely(params==NULL))
    {
        return -ENOMEM;
    }

    if(unlikely(copy_from_user(params,params_user,sizeof(*params))!=0))
    {
        kfree(params);
        return -EFAULT;
    }

    if(unlikely(params->buffer_size>G39_MAX_BUFFER_SIZE))
    {
        kfree(params);
        return -E2BIG;
    }

    if(unlikely(((params->fir1.ctrl2.len+1)/2)>ARRAY_SIZE(params->fir1.coeffs) ||
        ((params->fir2.ctrl2.len+1)/2)>ARRAY_SIZE(params->fir2.coeffs)))

    {
        kfree(params);
        return -EINVAL;
    }

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        io_stream=dev->io_stream+G39_IO_STREAM_IN_DDC2_0+channel;

        if(unlikely(io_stream->active || dev->swp_active))
        {
            status=-EBUSY;
        }
        else
        {
            io_stream->buffer_size=params->buffer_size;
            io_stream->bytes_per_sec=params->bytes_per_sec;

            status=dev->func_ddc2_start(dev,channel,params);
        }
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    kfree(params);

    return status;
}

static void g39_ddc2_stop(struct g39ddc *dev,uint32_t channel)
{
    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        dev->func_ddc2_stop(dev,channel);
    }

    G39_DEV_UNLOCK;
}

static int g39_ddc2_set_frequency(struct g39ddc *dev,uint32_t channel,uint32_t tuning_word)
{
 int status;

    G39_DEV_LOCK;

    if(likely(dev->state.fpga && dev->io_stream[G39_IO_STREAM_IN_DDC2_0+channel].active))
    {
        status=dev->func_ddc2_set_frequency(dev,channel,tuning_word);
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static int g39_swp_start(struct g39ddc *dev,uint16_t settling_time)
{
 int status;
 int i;

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        for(i=0;i<G39_IO_STREAM_COUNT;i++)
        {
            if(dev->io_stream[i].active)
            {
                G39_DEV_UNLOCK;
                return -EBUSY;
            }
        }

        dev->swp_settling_time=settling_time&0xFFF;

        if(unlikely(dev->swp_settling_time==0))
        {
            dev->swp_settling_time=1;
        }

        dev->swp_settling_time=settling_time?settling_time:1;

        if(settling_time&0x8000)
        {
            dev->swp_settling_time|=0x8000;
        }

        dev->io_stream[G39_IO_STREAM_IN_IF].buffer_size=65536*sizeof(int16_t);

        status=dev->func_swp_start(dev);
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static void g39_swp_stop(struct g39ddc *dev)
{
    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        dev->func_swp_stop(dev);
    }

    G39_DEV_UNLOCK;
}

static int g39_swp_snapshot(struct g39ddc *dev)
{
  int status;

    G39_DEV_LOCK;

    if(likely(dev->swp_active))
    {
        status=dev->func_swp_snapshot(dev);
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static int g39_ddc1_playback_start(struct g39ddc *dev,uint32_t channel,struct g39_ddc1_playback_params *params)
{
 int status;

    if(unlikely(params->bytes_per_sec==0 || (params->bytes_per_sec&1)))
    {
        return -EINVAL;
    }

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        if(unlikely(dev->io_stream[G39_IO_STREAM_OUT_DDC1_0+channel].active ||
                    dev->io_stream[G39_IO_STREAM_IN_DDC1_0+channel].active || dev->swp_active))
        {
            status=-EBUSY;
        }
        else
        {
            dev->io_stream[G39_IO_STREAM_OUT_DDC1_0+channel].bytes_per_sec=params->bytes_per_sec;
            dev->io_stream[G39_IO_STREAM_OUT_DDC1_0+channel].buffer_size=params->buffer_size;

            status=dev->func_ddc1_playback_start(dev,channel,params->divider);
        }
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static void g39_ddc1_playback_stop(struct g39ddc *dev,uint32_t channel)
{
    G39_DEV_LOCK;

    if(dev->state.fpga)
    {
        dev->func_ddc1_playback_stop(dev,channel);
    }

    G39_DEV_UNLOCK;
}

static int g39_dac_start(struct g39ddc *dev,struct g39_dac_params *params)
{
 int status;

    if(unlikely(params->dac_control==0 || params->bytes_per_sec==0))
    {
        return -EINVAL;
    }

    G39_DEV_LOCK;

    if(likely(dev->state.fpga))
    {
        if(unlikely(dev->io_stream[G39_IO_STREAM_OUT_DAC].active || dev->swp_active))
        {
            status=-EBUSY;
        }
        else
        {
            dev->io_stream[G39_IO_STREAM_OUT_DAC].bytes_per_sec=params->bytes_per_sec;
            dev->io_stream[G39_IO_STREAM_OUT_DAC].buffer_size=params->buffer_size;

            status=dev->func_dac_start(dev,params->dac_control);
        }
    }
    else
    {
        status=-EPERM;
    }

    G39_DEV_UNLOCK;

    return status;
}

static void g39_dac_stop(struct g39ddc *dev)
{
    G39_DEV_LOCK;

    if(dev->state.fpga)
    {
        dev->func_dac_stop(dev);
    }

    G39_DEV_UNLOCK;
}

static void g39_cancel_buffers(struct g39ddc *dev,int stream_index)
{
 struct g39_io_stream *io_stream;

    io_stream=&dev->io_stream[stream_index];

    down_read(&io_stream->rw_semaphore);

    if(io_stream->active)
    {
        dev->func_cancel_buffers(dev,stream_index);
    }

    up_read(&io_stream->rw_semaphore);
}

static int g39_wait_for_power_off(struct g39ddc *dev)
{
 DECLARE_WAITQUEUE(wait_queue_entry,current);

    add_wait_queue(&dev->power_wait_queue,&wait_queue_entry);

    for(;;)
    {
        G39_DEV_LOCK;

        if(!dev->state.power || !dev->state.open || !dev->state.connected)
        {
            G39_DEV_UNLOCK;
            break;
        }

        G39_DEV_UNLOCK;

        set_current_state(TASK_INTERRUPTIBLE);

        schedule();

        set_current_state(TASK_RUNNING);

        if(signal_pending(current))
        {
            remove_wait_queue(&dev->power_wait_queue,&wait_queue_entry);
            return -EINTR;
        }
    }

    remove_wait_queue(&dev->power_wait_queue,&wait_queue_entry);
    return 0;
}

#ifdef __G39_USB__

static int g39_usb_initialize(struct g39ddc_usb *dev);

static int g39_usb_setup(struct g39ddc_usb *dev)
{
 int i,num_endpoints,cnt;
 int status;
 struct usb_endpoint_descriptor *ep_desc;


    status=usb_set_interface(dev->usb_dev,0,0);

    if(unlikely(status<0))
    {
        G39_DEV_ERROR(&dev->usb_dev->dev,"failed to set interface and alternate number!\n");
        return status;
    }

    num_endpoints=dev->usb_interface->cur_altsetting->desc.bNumEndpoints;

    cnt=0;

    for(i=0;i<num_endpoints;i++)
    {
        ep_desc=&dev->usb_interface->cur_altsetting->endpoint[i].desc;

        switch(ep_desc->bEndpointAddress & USB_ENDPOINT_NUMBER_MASK)
        {
            case G39_USB_EP_OUT_CTRL:
                if(le16_to_cpu(ep_desc->wMaxPacketSize)==255)
                {
                    ep_desc->wMaxPacketSize=cpu_to_le16(128);
                }

            case G39_USB_EP_IN_FPGA:
            case G39_USB_EP_IN_CTRL:
            case G39_USB_EP_OUT_FPGA:
                cnt++;
                break;
        }

        usb_control_msg(dev->usb_dev, usb_sndctrlpipe(dev->usb_dev, 0),
                              USB_REQ_CLEAR_FEATURE, USB_RECIP_ENDPOINT,
                              USB_ENDPOINT_HALT,ep_desc->bEndpointAddress, NULL, 0,
                              USB_CTRL_SET_TIMEOUT);
    }


    if(unlikely(cnt!=4))
    {
       G39_DEV_ERROR(&dev->usb_dev->dev,"required enpoints not found!\n");
       return -ENODEV;
    }

    usb_control_msg(dev->usb_dev, usb_sndctrlpipe(dev->usb_dev,0),
                0x0F,USB_TYPE_VENDOR|USB_DIR_OUT,3,0, NULL,0,G39_USB_TIMEOUT);

    return 0;
}


static void g39_usb_delete(struct g39ddc_usb *dev)
{
    usb_put_intf(dev->usb_interface);
    usb_put_dev(dev->usb_dev);
}

static int g39_usb_open(struct g39ddc_usb *dev)
{
    return 0;
}



static void g39_usb_close(struct g39ddc_usb *dev)
{
 int len;
 static uint8_t packet_s[12]={0x04,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0xA8,0x00,0x00,0x00};
 uint8_t *packet;

    packet=kmalloc(sizeof(packet_s),GFP_KERNEL);

    if(packet!=NULL)
    {
        memcpy(packet,packet_s,sizeof(packet_s));

        usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_CTRL),packet,sizeof(packet_s),&len,G39_USB_TIMEOUT);

        kfree(packet);
    }
}


static int g39_usb_control1_read(struct g39ddc_usb *dev,const uint16_t *params,void __user *user_buffer,uint16_t buffer_size,void __user *transferred)
{
 int status;
 void *buffer;

    if(unlikely(transferred!=NULL && !access_ok(transferred,VERIFY_WRITE,sizeof(uint16_t))))
    {
        return -EFAULT;
    }

    if(buffer_size!=0)
    {
        if(!access_ok(user_buffer,VERIFY_WRITE,buffer_size))
        {
            return -EFAULT;
        }

        buffer=kmalloc(buffer_size,GFP_KERNEL);

        if(unlikely(buffer==NULL))
        {
            return -ENOMEM;
        }
    }
    else
    {
        buffer=NULL;
    }

    status=usb_control_msg(dev->usb_dev,usb_rcvctrlpipe(dev->usb_dev,0),
                params[0],USB_TYPE_VENDOR|USB_DIR_IN,
                params[1],params[2],buffer,buffer_size,G39_USB_TIMEOUT);

    if(likely(status>=0))
    {
        if(transferred!=NULL)
        {
            if(unlikely(__put_user(status,(uint16_t __user*)transferred)<0))
            {
                if(buffer!=NULL)
                {
                    kfree(buffer);
                }
                return -EFAULT;
            }
        }

        if(buffer!=NULL)
        {
            if(status>0)
            {
                if(unlikely(__copy_to_user(user_buffer,buffer,status)!=0))
                {
                    kfree(buffer);
                    return -EFAULT;
                }
            }

            kfree(buffer);
        }

        return 0;
    }
    else
    {
        return status;
    }
}

static int g39_usb_control1_write(struct g39ddc_usb *dev,const uint16_t *params,void __user *user_buffer,uint16_t buffer_size,void __user *transferred)
{
 int status;
 void *buffer;

    if(unlikely(transferred!=NULL && !access_ok(transferred,VERIFY_WRITE,sizeof(uint16_t))))
    {
        return -EFAULT;
    }

    if(buffer_size!=0)
    {
        buffer=kmalloc(buffer_size,GFP_KERNEL);

        if(unlikely(buffer==NULL))
        {
            return -ENOMEM;
        }

        if(unlikely(copy_from_user(buffer,user_buffer,buffer_size)!=0))
        {
            kfree(buffer);
            return -EFAULT;
        }
    }
    else
    {
        buffer=NULL;
    }

    status=usb_control_msg(dev->usb_dev,usb_sndctrlpipe(dev->usb_dev,0),
                params[0],USB_TYPE_VENDOR|USB_DIR_OUT,
                params[1],params[2],buffer,buffer_size,G39_USB_TIMEOUT);

    if(buffer!=NULL)
    {
        kfree(buffer);
    }

    if(likely(status>=0))
    {
        if(transferred!=NULL)
        {
            if(unlikely(__put_user(status,(uint16_t __user*)transferred)<0))
            {
                return -EFAULT;
            }
        }

        return 0;
    }
    else
    {
        return status;
    }
}

static int g39_usb_control2_read(struct g39ddc_usb *dev,void __user *user_buffer,uint16_t buffer_size,void __user *transferred)
{
 int status;
 int len;
 void *buffer;

    if(unlikely(transferred!=NULL && !access_ok(transferred,VERIFY_WRITE,sizeof(uint16_t))))
    {
        return -EFAULT;
    }

    if(buffer_size!=0)
    {
        if(!access_ok(user_buffer,VERIFY_WRITE,buffer_size))
        {
            return -EFAULT;
        }

        buffer=kmalloc(buffer_size,GFP_KERNEL);

        if(unlikely(buffer==NULL))
        {
            return -ENOMEM;
        }
    }
    else
    {
        buffer=NULL;
    }

    status=usb_bulk_msg(dev->usb_dev,usb_rcvbulkpipe(dev->usb_dev,G39_USB_EP_IN_CTRL),buffer,buffer_size,&len,G39_USB_TIMEOUT);

    if(likely(status>=0))
    {
        if(transferred!=NULL)
        {
            if(unlikely(__put_user(len,(uint16_t __user*)transferred)<0))
            {
                if(buffer!=NULL)
                {
                    kfree(buffer);
                }
                return -EFAULT;
            }
        }

        if(buffer!=NULL)
        {
            if(len>0)
            {
                if(unlikely(__copy_to_user(user_buffer,buffer,len)!=0))
                {
                    kfree(buffer);
                    return -EFAULT;
                }
            }

            kfree(buffer);
        }

        return 0;
    }
    else
    {
        if(buffer!=NULL)
        {
            kfree(buffer);
        }

        return status;
    }
}


static int g39_usb_control2_write(struct g39ddc_usb *dev,void __user *user_buffer,uint16_t buffer_size,void __user *transferred)
{
 int status;
 int len;
 void *buffer;

    if(unlikely(transferred!=NULL && !access_ok(transferred,VERIFY_WRITE,sizeof(uint16_t))))
    {
        return -EFAULT;
    }

    if(buffer_size!=0)
    {
        buffer=kmalloc(buffer_size,GFP_KERNEL);

        if(unlikely(buffer==NULL))
        {
            return -ENOMEM;
        }

        if(unlikely(copy_from_user(buffer,user_buffer,buffer_size)!=0))
        {
            kfree(buffer);
            return -EFAULT;
        }
    }
    else
    {
        buffer=NULL;
    }

    status=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_CTRL),buffer,buffer_size,&len,G39_USB_TIMEOUT);

    if(buffer!=NULL)
    {
        kfree(buffer);
    }

    if(unlikely(status<0))
    {
        return status;
    }

    if(transferred!=NULL)
    {
        if(unlikely(__put_user(len,(uint16_t __user *)transferred)<0))
        {
            return -EFAULT;
        }
    }

    return 0;
}

static int g39_usb_set_power(struct g39ddc_usb *dev,uint8_t power)
{
 int len;
 uint8_t packet_s[12]={0x04,0x00,0x00,0x00,0x11,0x00,0x00,0x00,power?0x03:0x01,0x00,0x00,0x00};
 uint8_t *packet;
 int result;

    packet=kmalloc(sizeof(packet_s),GFP_KERNEL);

    if(packet==NULL)
    {
        return -ENOMEM;
    }

    memcpy(packet,packet_s,sizeof(packet_s));

    result=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_CTRL),packet,sizeof(packet_s),&len,G39_USB_TIMEOUT);

    kfree(packet);

    return result;
}

static int g39_usb_configure_fpga(struct g39ddc_usb *dev,void __user *user_buffer,uint32_t size)
{
 void *buffer;
 int status;
 int bytes_to_send;
 int len;
 uint32_t *packet;


    buffer=kmalloc(128,GFP_KERNEL);

    if(unlikely(buffer==NULL))
    {
        return -ENOMEM;
    }

    packet=(uint32_t*)buffer;

    packet[0]=cpu_to_le32(size);
    packet[1]=cpu_to_le32(0x10);

    status=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_CTRL),packet,8,&len,G39_USB_TIMEOUT);

    if(unlikely(status<0))
    {
        kfree(buffer);
        return status;
    }

    while(size!=0)
    {
        if(size>=128)
        {
            bytes_to_send=128;
        }
        else
        {
            bytes_to_send=size;
        }

        if(unlikely(copy_from_user(buffer,user_buffer,bytes_to_send)!=0))
        {
            kfree(buffer);
            return -EFAULT;
        }

        status=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_CTRL),buffer,bytes_to_send,&len,G39_USB_TIMEOUT);

        if(unlikely(status<0))
        {
            kfree(buffer);
            return status;
        }

        size-=len;
        user_buffer=(void __user*)((uintptr_t)user_buffer+len);
    }

    msleep(50);

    status=usb_bulk_msg(dev->usb_dev,usb_rcvbulkpipe(dev->usb_dev,G39_USB_EP_IN_CTRL),packet,12,&len,G39_USB_TIMEOUT);

    if(unlikely(status<0))
    {
        kfree(buffer);
        return -EIO;
    }

    if(unlikely(len!=12 || le32_to_cpu(packet[0])!=4 || le32_to_cpu(packet[1])!=0x80000010 || le32_to_cpu(packet[2])!=0))
    {
        kfree(buffer);
        return -EIO;
    }

    kfree(buffer);

    return 0;
}

static int g39_usb_fpga_cmd(struct g39ddc_usb *dev,uint16_t target,uint16_t id,uint16_t *data,uint32_t size)
{
 uint32_t i;
 uint16_t c;
 int sent;
 uint16_t *packet;//[64];
 int result;

    if(unlikely(size>(64-9)*sizeof(uint16_t)))
    {
        return -E2BIG;
    }

    packet=kmalloc(64*sizeof(uint16_t),GFP_KERNEL);

    if(packet==NULL)
    {
        return -ENOMEM;
    }

    memset(packet,0,64*sizeof(uint16_t));

    packet[0]=cpu_to_le16(0x4041);
    packet[1]=cpu_to_le16(0x4243);
    packet[2]=cpu_to_le16(0x4445);
    packet[3]=cpu_to_le16(0x4647);
    packet[4]=cpu_to_le16(target);
    packet[5]=cpu_to_le16(id);
    //packet[6]=0;
    packet[7]=cpu_to_le16(10000);

    memcpy(packet+9,data,size);

    c=0;

    for(i=0;i<64;i++)
    {
        c+=le16_to_cpu(packet[i]);
    }

    c=(~c)+1;

    packet[8]=cpu_to_le16(c);

    result=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_FPGA),packet,sizeof(64*sizeof(uint16_t)),&sent,G39_USB_TIMEOUT);

    kfree(packet);

    return result;
}

static int g39_usb_set_spi_regs(struct g39ddc_usb *dev,struct g39_spi2_register __user *user_buffer,uint16_t count,uint16_t id)
{
 uint32_t packet_size,i;
 uint32_t buffer_size;
 int status;
 uint16_t *packet;
 struct g39_spi_register *buffer,*data;

    buffer_size=count*sizeof(struct g39_spi_register);

    buffer=kmalloc(buffer_size,GFP_KERNEL);

    if(unlikely(buffer==NULL))
    {
        return -ENOMEM;
    }

    if(unlikely(copy_from_user(buffer,user_buffer,buffer_size)!=0))
    {
        kfree(buffer);
        return -EFAULT;
    }

    packet_size=buffer_size+sizeof(uint16_t);

    packet=kmalloc(packet_size,GFP_KERNEL);

    if(unlikely(packet==NULL))
    {
        kfree(buffer);
        return -ENOMEM;
    }

    packet[0]=cpu_to_le32(count);

    data=(struct g39_spi_register*)(packet+1);

    for(i=0;i<count;i++)
    {
        data[i].address=cpu_to_le16(buffer[i].address);
        data[i].value=cpu_to_le32(buffer[i].value);
    }

    status=g39_usb_fpga_cmd(dev,3,id,packet,packet_size);

    kfree(packet);
    kfree(buffer);

    return status;
}

static int g39_usb_set_spi2_regs(struct g39ddc_usb *dev,struct g39_spi2_register __user *user_buffer,uint16_t count,uint16_t id)
{
 uint32_t size;
 uint32_t i;
 uint16_t *packet,*data;
 int status;
 int sent;
 struct g39_spi2_register *buffer;

    size=count*sizeof(struct g39_spi2_register);

    buffer=kmalloc(size,GFP_KERNEL);

    if(unlikely(buffer==NULL))
    {
        return -ENOMEM;
    }


    if(unlikely(copy_from_user(buffer,user_buffer,size)!=0))
    {
        kfree(buffer);
        return -EFAULT;
    }

    size=(count*6+3)*sizeof(uint16_t);

    packet=kmalloc(size,GFP_KERNEL);

    if(unlikely(packet==NULL))
    {
        kfree(buffer);
        return -ENOMEM;
    }

    packet[0]=cpu_to_le16(0x4041);
    packet[1]=cpu_to_le16(3+count*6);
    packet[2]=cpu_to_le16(1);

    data=packet+3;

    for(i=0;i<count;i++)
    {
        data[0]=cpu_to_le16(4);
        *(uint32_t*)(data+1)=cpu_to_le32(buffer[i].u32[0]);
        data[3]=cpu_to_le16(5);
        *(uint32_t*)(data+4)=cpu_to_le32(buffer[i].u32[1]);
        data+=6;
    }

    status=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_FPGA),packet,size,&sent,G39_USB_TIMEOUT);

    kfree(packet);
    kfree(buffer);

    return status;
}

static int g39_usb_set_reg(struct g39ddc_usb *dev,uint16_t address,uint32_t value)
{
 int sent;
 int status;
 uint16_t *packet;

    packet=kmalloc(6*sizeof(uint16_t),GFP_KERNEL);

    if(unlikely(packet==NULL))
    {
        return -ENOMEM;
    }

    packet[0]=cpu_to_le16(0x4041);
    packet[1]=cpu_to_le16(6);
    packet[2]=cpu_to_le16(1);
    packet[3]=cpu_to_le16(address);
    *(uint32_t*)(packet+4)=cpu_to_le32(value);

    status=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_FPGA),packet,6*sizeof(uint16_t),&sent,G39_USB_TIMEOUT);

    kfree(packet);

    return status;
}

static int g39_usb_set_reg_rep(struct g39ddc_usb *dev,uint16_t address,uint32_t *values,uint32_t count)
{
 int sent;
 uint32_t i,size;
 uint32_t *data;
 uint16_t *packet;
 int status;

    size=count*sizeof(uint32_t)+4*sizeof(uint16_t);

    packet=kmalloc(size,GFP_KERNEL);

    if(unlikely(packet==NULL))
    {
        return -ENOMEM;
    }

    packet[0]=cpu_to_le16(0x4041);
    packet[1]=cpu_to_le16(2*count+4);
    packet[2]=cpu_to_le16(2);
    packet[3]=cpu_to_le16(address);

    data=(uint32_t*)(packet+4);

    for(i=0;i<count;i++)
    {
        data[i]=cpu_to_le32(values[i]);
    }

    status=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_FPGA),packet,size,&sent,G39_USB_TIMEOUT);

    kfree(packet);

    return status;
}

static int g39_usb_set_regs(struct g39ddc_usb *dev,uint16_t *address,uint32_t *value,uint32_t count)
{
 int sent;
 int status;
 uint32_t size,i;
 uint16_t *data,*packet;

    size=3*sizeof(uint16_t)*(count+1);

    packet=kmalloc(size,GFP_KERNEL);

    if(unlikely(packet==NULL))
    {
        return -ENOMEM;
    }

    packet[0]=cpu_to_le16(0x4041);
    packet[1]=cpu_to_le16(3+3*count);
    packet[2]=cpu_to_le16(1);

    data=packet+3;

    for(i=0;i<count;i++,data+=3)
    {
        data[0]=cpu_to_le16(address[i]);
        data[1]=cpu_to_le16((uint16_t)((value[i])&0xFFFF));
        data[2]=cpu_to_le16((uint16_t)((value[i])>>16));
    }

    status=usb_bulk_msg(dev->usb_dev,usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_FPGA),packet,size,&sent,G39_USB_TIMEOUT);

    kfree(packet);

    return status;
}

static int g39_usb_set_noise_blanker(struct g39ddc_usb *dev,uint16_t value)
{
    return g39_usb_set_reg(dev,9,value);
}

#if LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,19)
static void g39_usb_in_urb_complete_callback(struct urb *urb)
#else
static void g39_usb_in_urb_complete_callback(struct urb *urb,struct pt_regs *regs)
#endif
{
 struct g39ddc_usb *dev;
 struct g39_usb_buffer *buffer;
 unsigned long flags;

    buffer=urb->context;
    dev=buffer->dev;

    spin_lock_irqsave(&dev->input.complete_usb_buffers.spinlock,flags);

    buffer->pending=0;

    list_add_tail(&buffer->list_entry,&dev->input.complete_usb_buffers.list);

    spin_unlock_irqrestore(&dev->input.complete_usb_buffers.spinlock,flags);

    wake_up_process(dev->input.thread);
}


#if LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,19)
static void g39_usb_in_readout_urb_complete_callback(struct urb *urb)
#else
static void g39_usb_in_readout_urb_complete_callback(struct urb *urb,struct pt_regs *regs)
#endif
{
    up(urb->context);
}

static void g39_usb_update_input_buffer_size(struct g39ddc_usb *dev)
{
 uint32_t bytes_per_sec;
 uint32_t buffer_size;
 int i;

    bytes_per_sec=0;

    for(i=G39_IO_STREAM_IN_FIRST;i<=G39_IO_STREAM_IN_LAST;i++)
    {
        if(dev->g39_dev.io_stream[i].active)
        {
            bytes_per_sec+=dev->g39_dev.io_stream[i].bytes_per_sec;
        }
    }

    bytes_per_sec+=bytes_per_sec*6/G39_USB_INPUT_PACKET_PAYLOAD_SIZE;

    buffer_size=(bytes_per_sec*10/1000+2047)&(~2047);

    if(buffer_size<G39_USB_INPUT_BUFFER_MIN_SIZE)
    {
        buffer_size=G39_USB_INPUT_BUFFER_MIN_SIZE;
    }
    else if(buffer_size>G39_USB_INPUT_BUFFER_MAX_SIZE)
    {
        buffer_size=G39_USB_INPUT_BUFFER_MAX_SIZE;
    }

    dev->input.buffer_size=buffer_size;
}

static int g39_usb_input_stream_thread(void *param)
{
 struct g39ddc_usb *dev=param;
 int i;
 int status;
 int loop;
 uint32_t rest_buffer_size;
 struct g39_usb_buffer *usb_buffer;
 struct g39_usb_buffers *usb_buffers;
 struct g39_usb_input_packet *packet;
 uint32_t stream_index;
 uint32_t size;
 unsigned long flags;
 struct g39_io_stream *io_stream;
 struct g39_io_buffer **io_buffers;
 uint32_t remain;
 struct semaphore semaphore;
 uint8_t *packet_data;
 uint32_t transferred[G39_IO_STREAM_COUNT];
 uint8_t rest_buffer[sizeof(struct g39_usb_input_packet)];

    rest_buffer_size=0;
    usb_buffers=&dev->input.complete_usb_buffers;
    loop=1;

    for(i=0;i<G39_USB_INPUT_BUFFER_COUNT;i++)
    {
        usb_fill_bulk_urb(dev->input.buffer[i].urb,dev->usb_dev,
                          usb_rcvbulkpipe(dev->usb_dev,G39_USB_EP_IN_FPGA),
                          dev->input.buffer[i].mem+sizeof(struct g39_usb_input_packet),
                          dev->input.buffer_size,g39_usb_in_urb_complete_callback,
                          dev->input.buffer+i);

        dev->input.buffer[i].pending=1;

        status=usb_submit_urb(dev->input.buffer[i].urb,GFP_KERNEL);

        if(unlikely(status<0))
        {
            dev->input.buffer[i].pending=0;
            break;
        }
    }

    if(likely(status>=0))
    {
        io_buffers=dev->io_buffer;

        while(loop && !kthread_should_stop())
        {
            spin_lock_irqsave(&usb_buffers->spinlock,flags);

            if(unlikely(list_empty(&usb_buffers->list)))
            {
                usb_buffer=NULL;
            }
            else
            {
                usb_buffer=list_entry(usb_buffers->list.next,struct g39_usb_buffer,list_entry);
                list_del(&usb_buffer->list_entry);
            }

            spin_unlock_irqrestore(&usb_buffers->spinlock,flags);

            if(likely(usb_buffer!=NULL))
            {
                if(likely(usb_buffer->urb->status==0 && usb_buffer->urb->actual_length>0))
                {
                    size=usb_buffer->urb->actual_length;

                    if(rest_buffer_size!=0)
                    {
                        packet=(struct g39_usb_input_packet*)(usb_buffer->mem+sizeof(struct g39_usb_input_packet)-rest_buffer_size);
                        memcpy(packet,rest_buffer,rest_buffer_size);
                        size+=rest_buffer_size;
                    }
                    else
                    {
                        packet=(struct g39_usb_input_packet*)(usb_buffer->mem+sizeof(struct g39_usb_input_packet));
                    }

                    while(loop && size>=sizeof(struct g39_usb_input_packet))
                    {
                        if(unlikely(packet->sync[0]!=0x41 || packet->sync[1]!=0x40))
                        {
                            do
                            {
                                size--;
                                packet=(struct g39_usb_input_packet*)(((uintptr_t)packet)+1);
                            }
                            while(size>=2 && (packet->sync[0]!=0x41 || packet->sync[1]!=0x40));

                            if(size<sizeof(struct g39_usb_input_packet))
                            {
                                break;
                            }
                        }

                        switch(packet->source)
                        {
                            case 1: //DDC1[0]
                            case 2: //DDC1[1]
                            case 3: //DDC2[0]
                            case 4: //DDC2[1]
                                stream_index=G39_IO_STREAM_IN_DDC1_0+packet->source-1;
                                io_stream=&dev->g39_dev.io_stream[stream_index];

                                down_read(&io_stream->rw_semaphore);

                                if(io_stream->active && io_stream->id==le16_to_cpu(packet->id))
                                {
                                    i=G39_USB_INPUT_PACKET_PAYLOAD_SIZE;
                                    packet_data=packet->data;

                                    while(i)
                                    {
                                        if(unlikely(io_buffers[stream_index]==NULL))
                                        {
                                            io_buffers[stream_index]=g39_io_stream_get_pending_buffer(io_stream);

                                            if(unlikely(io_buffers[stream_index]==NULL))
                                            {
                                                break;
                                            }

                                            if(unlikely(g39_io_buffer_map(io_buffers[stream_index])<0))
                                            {
                                                g39_io_buffer_free(io_buffers[stream_index]);
                                                io_buffers[stream_index]=NULL;
                                                loop=0;
                                                break;
                                            }

                                            io_buffers[stream_index]->changed=1;
                                            transferred[stream_index]=0;
                                        }

                                        remain=io_stream->buffer_size-transferred[stream_index];

                                        if(i>=remain)
                                        {
                                            memcpy(io_buffers[stream_index]->mem+transferred[stream_index],
                                                   packet_data,remain);

                                            g39_io_stream_put_complete_buffer(io_stream,io_buffers[stream_index]);
                                            io_buffers[stream_index]=NULL;

                                            i-=remain;
                                            packet_data+=remain;
                                        }
                                        else
                                        {
                                            memcpy(io_buffers[stream_index]->mem+transferred[stream_index],
                                                   packet_data,i);

                                            transferred[stream_index]+=i;
                                            break;
                                        }
                                    }
                                }

                                up_read(&io_stream->rw_semaphore);
                                break;

                            case 0: //IF
                                io_stream=&dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF];

                                down_read(&io_stream->rw_semaphore);

                                if(io_stream->active && io_stream->id==le16_to_cpu(packet->id))
                                {
                                    if(packet->flags & 0x80)
                                    {
                                        if(likely(io_buffers[G39_IO_STREAM_IN_IF]==NULL))
                                        {
                                            io_buffers[G39_IO_STREAM_IN_IF]=g39_io_stream_get_pending_buffer(io_stream);

                                            if(unlikely(io_buffers[G39_IO_STREAM_IN_IF]==NULL))
                                            {
                                                up_read(&io_stream->rw_semaphore);
                                                break;
                                            }

                                            if(unlikely(g39_io_buffer_map(io_buffers[G39_IO_STREAM_IN_IF])<0))
                                            {
                                                g39_io_buffer_free(io_buffers[G39_IO_STREAM_IN_IF]);
                                                io_buffers[G39_IO_STREAM_IN_IF]=NULL;

                                                up_read(&io_stream->rw_semaphore);
                                                loop=0;
                                                break;
                                            }

                                            io_buffers[G39_IO_STREAM_IN_IF]->changed=1;
                                            transferred[G39_IO_STREAM_IN_IF]=0;
                                        }

                                        memcpy(io_buffers[G39_IO_STREAM_IN_IF]->mem,packet->data,G39_USB_INPUT_PACKET_PAYLOAD_SIZE);
                                        transferred[G39_IO_STREAM_IN_IF]=G39_USB_INPUT_PACKET_PAYLOAD_SIZE;
                                    }
                                    else
                                    {
                                        if(likely(dev->io_buffer[G39_IO_STREAM_IN_IF]!=NULL))
                                        {
                                            remain=io_stream->buffer_size-transferred[G39_IO_STREAM_IN_IF];

                                            if(remain<=G39_USB_INPUT_PACKET_PAYLOAD_SIZE)
                                            {
                                                memcpy(io_buffers[G39_IO_STREAM_IN_IF]->mem+
                                                       transferred[G39_IO_STREAM_IN_IF],
                                                       packet->data,remain);
                                                transferred[G39_IO_STREAM_IN_IF]+=remain;

                                                g39_io_stream_put_complete_buffer(io_stream,io_buffers[G39_IO_STREAM_IN_IF]);

                                                io_buffers[G39_IO_STREAM_IN_IF]=NULL;
                                            }
                                            else
                                            {
                                                memcpy(io_buffers[G39_IO_STREAM_IN_IF]->mem+
                                                       transferred[G39_IO_STREAM_IN_IF],
                                                       packet->data,G39_USB_INPUT_PACKET_PAYLOAD_SIZE);
                                                transferred[G39_IO_STREAM_IN_IF]+=G39_USB_INPUT_PACKET_PAYLOAD_SIZE;
                                            }
                                        }
                                    }
                                }

                                up_read(&io_stream->rw_semaphore);
                                break;
                        }

                        packet++;
                        size-=sizeof(struct g39_usb_input_packet);
                    }

                    rest_buffer_size=size;

                    if(size!=0)
                    {
                        memcpy(rest_buffer,packet,size);
                    }
                }

                if(likely(loop))
                {
                    usb_fill_bulk_urb(usb_buffer->urb,dev->usb_dev,
                                      usb_rcvbulkpipe(dev->usb_dev,G39_USB_EP_IN_FPGA),
                                      usb_buffer->mem+sizeof(struct g39_usb_input_packet),
                                      dev->input.buffer_size,g39_usb_in_urb_complete_callback,
                                      usb_buffer);

                    usb_buffer->pending=1;

                    status=usb_submit_urb(usb_buffer->urb,GFP_KERNEL);

                    if(unlikely(status<0))
                    {
                        usb_buffer->pending=0;
                        break;
                    }
                }
            }
            else
            {
                set_current_state(TASK_INTERRUPTIBLE);

                if(!kthread_should_stop())
                {
                    schedule();
                }

                set_current_state(TASK_RUNNING);
            }
        }
    }

    for(i=0;i<G39_USB_INPUT_BUFFER_COUNT;i++)
    {
        if(dev->input.buffer[i].pending)
        {
            usb_kill_urb(dev->input.buffer[i].urb);
        }
    }

    usb_buffer=dev->input.buffer+0;
    sema_init(&semaphore,0);

    for(i=0;i<100;i++)
    {
        usb_fill_bulk_urb(usb_buffer->urb,dev->usb_dev,
                          usb_rcvbulkpipe(dev->usb_dev,G39_USB_EP_IN_FPGA),
                          usb_buffer->mem+sizeof(struct g39_usb_input_packet),
                          G39_USB_INPUT_BUFFER_MAX_SIZE,g39_usb_in_readout_urb_complete_callback,
                          &semaphore);

        status=usb_submit_urb(usb_buffer->urb,GFP_KERNEL);

        if(unlikely(status<0))
        {
            break;
        }

        if(down_timeout(&semaphore,HZ/10)!=0)
        {
            usb_kill_urb(usb_buffer->urb);
            break;
        }
    }

    do
    {
        set_current_state(TASK_INTERRUPTIBLE);

        if(!kthread_should_stop())
        {
            schedule();
        }

        set_current_state(TASK_RUNNING);
    }
    while(!kthread_should_stop());

    return 0;
}


static int g39_usb_swp_thread(void *param)
{
 struct g39ddc_usb *dev=param;
 int i;
 int status;
 int loop;
 uint32_t rest_buffer_size;
 struct g39_usb_buffer *usb_buffer;
 struct g39_usb_buffers *usb_buffers;
 struct g39_usb_input_packet *packet;
 uint32_t size;
 unsigned long flags;
 struct g39_io_stream *io_stream;
 struct g39_io_buffer **io_buffers;
 uint32_t remain;
 uint32_t transferred;
 uint8_t rest_buffer[sizeof(struct g39_usb_input_packet)];

    rest_buffer_size=0;
    usb_buffers=&dev->input.complete_usb_buffers;
    loop=1;
    transferred=0;

    for(i=0;i<G39_USB_INPUT_BUFFER_COUNT;i++)
    {
        usb_fill_bulk_urb(dev->input.buffer[i].urb,dev->usb_dev,
                          usb_rcvbulkpipe(dev->usb_dev,G39_USB_EP_IN_FPGA),
                          dev->input.buffer[i].mem+sizeof(struct g39_usb_input_packet),
                          G39_USB_INPUT_SWP_BUFFER_SIZE,g39_usb_in_urb_complete_callback,
                          dev->input.buffer+i);

        dev->input.buffer[i].pending=1;

        status=usb_submit_urb(dev->input.buffer[i].urb,GFP_KERNEL);

        if(unlikely(status<0))
        {
            dev->input.buffer[i].pending=0;
            break;
        }
    }

    if(likely(status>=0))
    {
        io_stream=&dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF];

        io_buffers=dev->io_buffer;

        while(loop && !kthread_should_stop())
        {
            spin_lock_irqsave(&usb_buffers->spinlock,flags);

            if(unlikely(list_empty(&usb_buffers->list)))
            {
                usb_buffer=NULL;
            }
            else
            {
                usb_buffer=list_entry(usb_buffers->list.next,struct g39_usb_buffer,list_entry);
                list_del(&usb_buffer->list_entry);
            }

            spin_unlock_irqrestore(&usb_buffers->spinlock,flags);

            if(likely(usb_buffer!=NULL))
            {
                if(likely(usb_buffer->urb->status==0 && usb_buffer->urb->actual_length>0))
                {
                    size=usb_buffer->urb->actual_length;

                    if(rest_buffer_size!=0)
                    {
                        packet=(struct g39_usb_input_packet*)(usb_buffer->mem+sizeof(struct g39_usb_input_packet)-rest_buffer_size);
                        memcpy(packet,rest_buffer,rest_buffer_size);
                        size+=rest_buffer_size;
                    }
                    else
                    {
                        packet=(struct g39_usb_input_packet*)(usb_buffer->mem+sizeof(struct g39_usb_input_packet));
                    }

                    while(loop && size>=sizeof(struct g39_usb_input_packet))
                    {
                        if(unlikely(packet->sync[0]!=0x41 || packet->sync[1]!=0x40))
                        {
                            do
                            {
                                size--;
                                packet=(struct g39_usb_input_packet*)(((uintptr_t)packet)+1);
                            }
                            while(size>=2 && (packet->sync[0]!=0x41 || packet->sync[1]!=0x40));

                            if(size<sizeof(struct g39_usb_input_packet))
                            {
                                break;
                            }
                        }

                        if(packet->source==0)
                        {
                            for(;;)
                            {
                                down_read(&io_stream->rw_semaphore);

                                if(io_stream->active && io_stream->id==le16_to_cpu(packet->id))
                                {
                                    if(packet->flags & 0x80)
                                    {
                                        if(likely(io_buffers[G39_IO_STREAM_IN_IF]==NULL))
                                        {
                                            io_buffers[G39_IO_STREAM_IN_IF]=g39_io_stream_get_pending_buffer(io_stream);

                                            if(unlikely(io_buffers[G39_IO_STREAM_IN_IF]==NULL))
                                            {
                                                up_read(&io_stream->rw_semaphore);
                                                break;
                                            }

                                            if(unlikely(g39_io_buffer_map(io_buffers[G39_IO_STREAM_IN_IF])<0))
                                            {
                                                g39_io_buffer_free(io_buffers[G39_IO_STREAM_IN_IF]);
                                                io_buffers[G39_IO_STREAM_IN_IF]=NULL;

                                                up_read(&io_stream->rw_semaphore);
                                                loop=0;
                                                break;
                                            }

                                            io_buffers[G39_IO_STREAM_IN_IF]->changed=1;
                                            transferred=0;
                                        }

                                        memcpy(io_buffers[G39_IO_STREAM_IN_IF]->mem,packet->data+2,G39_USB_INPUT_PACKET_PAYLOAD_SIZE-2);
                                        transferred=G39_USB_INPUT_PACKET_PAYLOAD_SIZE-2;
                                    }
                                    else
                                    {
                                        if(likely(dev->io_buffer[G39_IO_STREAM_IN_IF]!=NULL))
                                        {
                                            remain=io_stream->buffer_size-transferred;

                                            if(remain<=G39_USB_INPUT_PACKET_PAYLOAD_SIZE)
                                            {
                                                memcpy(io_buffers[G39_IO_STREAM_IN_IF]->mem+
                                                       transferred,
                                                       packet->data,remain);
                                                transferred+=remain;

                                                g39_io_stream_put_complete_buffer(io_stream,io_buffers[G39_IO_STREAM_IN_IF]);

                                                io_buffers[G39_IO_STREAM_IN_IF]=NULL;
                                            }
                                            else
                                            {
                                                memcpy(io_buffers[G39_IO_STREAM_IN_IF]->mem+
                                                       transferred,
                                                       packet->data,G39_USB_INPUT_PACKET_PAYLOAD_SIZE);
                                                transferred+=G39_USB_INPUT_PACKET_PAYLOAD_SIZE;
                                            }
                                        }
                                    }
                                }

                                up_read(&io_stream->rw_semaphore);
                                break;
                            }
                        }

                        packet++;
                        size-=sizeof(struct g39_usb_input_packet);
                    }

                    rest_buffer_size=size;

                    if(size!=0)
                    {
                        memcpy(rest_buffer,packet,size);
                    }
                }

                if(likely(loop))
                {
                    usb_fill_bulk_urb(usb_buffer->urb,dev->usb_dev,
                                      usb_rcvbulkpipe(dev->usb_dev,G39_USB_EP_IN_FPGA),
                                      usb_buffer->mem+sizeof(struct g39_usb_input_packet),
                                      G39_USB_INPUT_SWP_BUFFER_SIZE,g39_usb_in_urb_complete_callback,
                                      usb_buffer);

                    usb_buffer->pending=1;

                    status=usb_submit_urb(usb_buffer->urb,GFP_KERNEL);

                    if(unlikely(status<0))
                    {
                        usb_buffer->pending=0;
                        break;
                    }
                }
            }
            else
            {
                set_current_state(TASK_INTERRUPTIBLE);

                if(!kthread_should_stop())
                {
                    schedule();
                }

                set_current_state(TASK_RUNNING);
            }
        }
    }

    for(i=0;i<G39_USB_INPUT_BUFFER_COUNT;i++)
    {
        if(dev->input.buffer[i].pending)
        {
            usb_kill_urb(dev->input.buffer[i].urb);
        }
    }


    do
    {
        set_current_state(TASK_INTERRUPTIBLE);

        if(!kthread_should_stop())
        {
            schedule();
        }

        set_current_state(TASK_RUNNING);
    }
    while(!kthread_should_stop());

    return 0;
}

#if LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,19)
static void g39_usb_out_urb_complete_callback(struct urb *urb)
#else
static void g39_usb_out_urb_complete_callback(struct urb *urb,struct pt_regs *regs)
#endif
{
 struct g39ddc_usb *dev;
 struct g39_usb_buffer *buffer;
 unsigned long flags;

    buffer=urb->context;
    dev=buffer->dev;

    spin_lock_irqsave(&dev->output.avail_usb_buffers.spinlock,flags);

    buffer->pending=0;

    list_add_tail(&buffer->list_entry,&dev->output.avail_usb_buffers.list);

    spin_unlock_irqrestore(&dev->output.avail_usb_buffers.spinlock,flags);

    wake_up_process(dev->output.thread);
}

static int g39_usb_output_stream_thread(void *param)
{
 int index,i,j;
 struct g39ddc_usb *dev;
 struct g39_io_stream *io_stream;
 struct g39_usb_buffer *usb_buffer;
 struct g39_usb_buffers *usb_buffers;
 unsigned long flags;
 int usb_buffer_size;
 int no_data;
 int status;
 uint32_t divider;
 struct g39_usb_output_packet *packet;
 uint8_t dac,dac2;
 uint8_t data;
 uint32_t bytes_to_copy;
 struct g39_io_buffer **io_buffers;
 uint32_t packet_size;
 uint32_t size[G39_IO_STREAM_OUT_COUNT];
 uint8_t *mem[G39_IO_STREAM_OUT_COUNT];
 uint8_t enabled[G39_IO_STREAM_OUT_COUNT];
 uint8_t first_packet[G39_IO_STREAM_OUT_COUNT];
 uint32_t ddc1_divider[G39_CHANNEL_COUNT];


    dev=param;

    usb_buffer=NULL;
    usb_buffers=&dev->output.avail_usb_buffers;
    index=0;
    usb_buffer_size=0;
    dac=0;
    packet=0;

    for(i=0;i<G39_IO_STREAM_OUT_COUNT;i++)
    {
        first_packet[i]=1;
        enabled[i]=0;
        size[i]=0;
        mem[i]=NULL;
    }

    for(i=0;i<G39_CHANNEL_COUNT;i++)
    {
        ddc1_divider[i]=0;
    }

    io_buffers=dev->io_buffer+G39_IO_STREAM_OUT_FIRST;

    while(!kthread_should_stop())
    {
        if(usb_buffer==NULL)
        {
            spin_lock_irqsave(&usb_buffers->spinlock,flags);

            if(unlikely(list_empty(&usb_buffers->list)))
            {
                spin_unlock_irqrestore(&usb_buffers->spinlock,flags);

                set_current_state(TASK_INTERRUPTIBLE);

                if(!kthread_should_stop())
                {
                    schedule();
                }

                set_current_state(TASK_RUNNING);

                continue;
            }
            else
            {
                usb_buffer=list_entry(usb_buffers->list.next,struct g39_usb_buffer,list_entry);
                list_del(&usb_buffer->list_entry);
                spin_unlock_irqrestore(&usb_buffers->spinlock,flags);

                usb_buffer_size=0;

                packet=(struct g39_usb_output_packet*)usb_buffer->mem;
            }
        }

        data=1;

        for(i=0;i<G39_CHANNEL_COUNT;i++)
        {
            divider=dev->output.ddc1_divider[i];

            if(ddc1_divider[i]!=divider)
            {
                if((ddc1_divider[i] & 0xFFFF)!=0 && divider!=0)
                {
                    packet->sync=cpu_to_le16(0x4041);
                    packet->command=cpu_to_le16(1);
                    packet->address=cpu_to_le16(USB_REG_DDC2_INPUT_SWITCH_0+(USB_REG_DDC2_INPUT_SWITCH_1-USB_REG_DDC2_INPUT_SWITCH_0)*i);
                    packet->length=cpu_to_le16(3+1+2);
                    packet->data32[0]=cpu_to_le32(ddc1_divider[i] & 0xFFFF0000);

                    usb_buffer_size+=2*(3+1+2);

                    packet=(struct g39_usb_output_packet*)((uintptr_t)packet+2*(3+1+2));
                }

                first_packet[i]=1;

                ddc1_divider[i]=divider;

                packet->sync=cpu_to_le16(0x4041);
                packet->command=cpu_to_le16(1);
                packet->address=cpu_to_le16(USB_REG_DDC2_INPUT_SWITCH_0+(USB_REG_DDC2_INPUT_SWITCH_1-USB_REG_DDC2_INPUT_SWITCH_0)*i);
                packet->length=cpu_to_le16(3+1+2);
                packet->data32[0]=cpu_to_le32(divider);

                usb_buffer_size+=2*(3+1+2);

                packet=(struct g39_usb_output_packet*)((uintptr_t)packet+2*(3+1+2));
                enabled[i]=(divider & 0xFFFF)?1:0;
                data=0;
            }
        }

        dac2=dev->g39_dev.dac;

        if(dac!=dac2)
        {
            dac=dac2;

            packet->sync=cpu_to_le16(0x4041);
            packet->command=cpu_to_le16(1);
            packet->address=cpu_to_le16(USB_REG_DAC_CONTROL);
            packet->length=cpu_to_le16(3+1+2);
            packet->data32[0]=cpu_to_le32(dac?dev->output.dac_control:0);

            packet=(struct g39_usb_output_packet*)((uintptr_t)packet+2*(3+1+2));
            enabled[G39_IO_STREAM_OUT_DAC-G39_IO_STREAM_OUT_FIRST]=dac;
            data=0;
        }

        if(likely(data))
        {
            for(i=0;i<10;i++)
            {
                no_data=1;

                for(j=0;j<G39_IO_STREAM_OUT_COUNT;j++)
                {
                    if(enabled[index])
                    {
                        io_stream=&dev->g39_dev.io_stream[G39_IO_STREAM_OUT_FIRST+index];

                        down_read(&io_stream->rw_semaphore);

                        if(io_stream->active)
                        {
                            bytes_to_copy=dev->output.bytes_per_packet[index];

                            if(likely((usb_buffer_size+bytes_to_copy)<=G39_USB_OUTPUT_BUFFER_SIZE))
                            {
                                packet_size=0;

                                while(bytes_to_copy!=0)
                                {
                                    if(io_buffers[index]==NULL)
                                    {
                                        io_buffers[index]=g39_io_stream_get_pending_buffer(io_stream);

                                        if(unlikely(io_buffers[index]==NULL))
                                        {
                                            break;
                                        }

                                        if(unlikely(g39_io_buffer_map(io_buffers[index])<0))
                                        {
                                            g39_io_buffer_free(io_buffers[index]);
                                            io_buffers[index]=NULL;
                                            break;
                                        }

                                        size[index]=io_buffers[index]->size;
                                        mem[index]=io_buffers[index]->mem;
                                    }


                                    if(bytes_to_copy<size[index])
                                    {
                                        memcpy(packet->data8+packet_size,mem[index],bytes_to_copy);
                                        packet_size+=bytes_to_copy;
                                        size[index]-=bytes_to_copy;
                                        mem[index]+=bytes_to_copy;
                                        break;
                                    }

                                    memcpy(packet->data8+packet_size,mem[index],size[index]);
                                    packet_size+=size[index];
                                    bytes_to_copy-=size[index];

                                    g39_io_stream_put_complete_buffer(io_stream,io_buffers[index]);
                                    io_buffers[index]=NULL;
                                }

                                packet_size&=~1;

                                if(packet_size!=0)
                                {
                                    packet_size+=2*(3+1);

                                    packet->sync=cpu_to_le16(0x4041);
                                    packet->length=cpu_to_le16(packet_size>>1);
                                    packet->address=cpu_to_le16(index);

                                    if(first_packet[index])
                                    {
                                        packet->command=cpu_to_le16(3);
                                        first_packet[index]=0;
                                    }
                                    else
                                    {
                                        packet->command=0;
                                    }

                                    usb_buffer_size+=packet_size;
                                    packet=(struct g39_usb_output_packet*)((uintptr_t)packet+packet_size);

                                    no_data=0;
                                }
                            }
                            else
                            {
                                up_read(&io_stream->rw_semaphore);
                                no_data=1;
                                break;
                            }
                        }

                        up_read(&io_stream->rw_semaphore);
                    }

                    index=(index+1)%G39_IO_STREAM_OUT_COUNT;
                }

                if(no_data)
                {
                    break;
                }
            }
        }

        if(usb_buffer_size!=0)
        {
            usb_fill_bulk_urb(usb_buffer->urb,dev->usb_dev,
                              usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_FPGA),
                              usb_buffer->mem,
                              usb_buffer_size,g39_usb_out_urb_complete_callback,
                              usb_buffer);

            usb_buffer->pending=1;

            status=usb_submit_urb(usb_buffer->urb,GFP_KERNEL);

            if(unlikely(status<0))
            {
                usb_buffer->pending=0;
                break;
            }

            usb_buffer=NULL;
        }
        else
        {
            set_current_state(TASK_INTERRUPTIBLE);

            if(!kthread_should_stop())
            {
                schedule();
            }

            set_current_state(TASK_RUNNING);
        }
    }

    while(usb_buffer==NULL)
    {
        spin_lock_irqsave(&usb_buffers->spinlock,flags);

        if(list_empty(&usb_buffers->list))
        {
            spin_unlock_irqrestore(&usb_buffers->spinlock,flags);

            set_current_state(TASK_INTERRUPTIBLE);

            schedule();

            set_current_state(TASK_RUNNING);
        }
        else
        {
            usb_buffer=list_entry(usb_buffers->list.next,struct g39_usb_buffer,list_entry);
            list_del(&usb_buffer->list_entry);

            spin_unlock_irqrestore(&usb_buffers->spinlock,flags);
        }
    }

    usb_buffer_size=0;
    packet=(struct g39_usb_output_packet*)usb_buffer->mem;

    for(i=0;i<G39_CHANNEL_COUNT;i++)
    {
        if(enabled[i])
        {
            packet->sync=cpu_to_le16(0x4041);
            packet->length=cpu_to_le16(3+1+2);
            packet->command=cpu_to_le16(1);
            packet->address=cpu_to_le16(USB_REG_DDC2_INPUT_SWITCH_0+(USB_REG_DDC2_INPUT_SWITCH_1-USB_REG_DDC2_INPUT_SWITCH_0)*i);
            packet->data32[0]=cpu_to_le32(ddc1_divider[i] & 0xFFFF0000);

            usb_buffer_size+=2*(3+1+2);

            packet=(struct g39_usb_output_packet*)((uintptr_t)packet+2*(3+1+2));
        }
    }

    if(enabled[i]) //DAC
    {
        packet->sync=cpu_to_le16(0x4041);
        packet->length=cpu_to_le16(3+1+2);
        packet->command=cpu_to_le16(1);
        packet->address=cpu_to_le16(USB_REG_DAC_CONTROL);
        packet->data32[0]=0;

        usb_buffer_size+=2*(3+1+2);
    }

    if(usb_buffer_size!=0)
    {
        usb_fill_bulk_urb(usb_buffer->urb,dev->usb_dev,
                          usb_sndbulkpipe(dev->usb_dev,G39_USB_EP_OUT_FPGA),
                          usb_buffer->mem,
                          usb_buffer_size,g39_usb_out_urb_complete_callback,
                          usb_buffer);

        usb_buffer->pending=1;

        status=usb_submit_urb(usb_buffer->urb,GFP_KERNEL);

        if(unlikely(status<0))
        {
            usb_buffer->pending=0;
        }
    }

    for(i=0;i<G39_USB_OUTPUT_BUFFER_COUNT;i++)
    {
        while(dev->output.buffer[i].pending)
        {
            set_current_state(TASK_INTERRUPTIBLE);

            schedule();

            set_current_state(TASK_RUNNING);
        }
    }

    do
    {
        set_current_state(TASK_INTERRUPTIBLE);

        if(!kthread_should_stop())
        {
            schedule();
        }

        set_current_state(TASK_RUNNING);
    }
    while(!kthread_should_stop());

    return 0;
}

static int g39_usb_start_stream(struct g39ddc_usb *dev,uint32_t stream_index)
{
 struct task_struct *thread;
 int i;
 int buffer_size;
 uint32_t bytes_per_packet;

    if(G39_IS_INPUT_STREAM(stream_index))
    {
        if(dev->input.stream_counter==0)
        {
            if(dev->g39_dev.swp_active && stream_index==G39_IO_STREAM_IN_IF)
            {
                buffer_size=G39_USB_INPUT_SWP_BUFFER_SIZE;
            }
            else
            {
                buffer_size=G39_USB_INPUT_BUFFER_MAX_SIZE;
            }


            for(i=0;i<G39_USB_INPUT_BUFFER_COUNT;i++)
            {
                dev->input.buffer[i].pending=0;

                dev->input.buffer[i].urb=usb_alloc_urb(0,GFP_KERNEL);

                if(unlikely(dev->input.buffer[i].urb==NULL))
                {
                    while(i--)
                    {
                        kfree(dev->input.buffer[i].mem);
                        dev->input.buffer[i].mem=NULL;

                        usb_free_urb(dev->input.buffer[i].urb);
                        dev->input.buffer[i].urb=NULL;
                    }

                    return -ENOMEM;
                }

                dev->input.buffer[i].mem=kmalloc(buffer_size+sizeof(struct g39_usb_input_packet),GFP_KERNEL);

                if(dev->input.buffer[i].mem==NULL)
                {
                    usb_free_urb(dev->input.buffer[i].urb);
                    dev->input.buffer[i].urb=NULL;

                    while(i--)
                    {
                        kfree(dev->input.buffer[i].mem);
                        dev->input.buffer[i].mem=NULL;

                        usb_free_urb(dev->input.buffer[i].urb);
                        dev->input.buffer[i].urb=NULL;
                    }

                    return -ENOMEM;
                }
            }

            INIT_LIST_HEAD(&dev->input.complete_usb_buffers.list);

            if(unlikely(dev->g39_dev.swp_active && stream_index==G39_IO_STREAM_IN_IF))
            {
                thread=kthread_create(g39_usb_swp_thread,dev,G39_DEVICE_NAME);
            }
            else
            {
                thread=kthread_create(g39_usb_input_stream_thread,dev,G39_DEVICE_NAME);
            }

            if(unlikely(IS_ERR(thread)))
            {
                for(i=0;i<G39_USB_INPUT_BUFFER_COUNT;i++)
                {
                    kfree(dev->input.buffer[i].mem);
                    dev->input.buffer[i].mem=NULL;

                    usb_free_urb(dev->input.buffer[i].urb);
                    dev->input.buffer[i].urb=NULL;
                }

                return PTR_ERR(thread);
            }

            dev->input.thread=thread;

            wake_up_process(thread);
        }

        dev->input.stream_counter++;

        dev->g39_dev.io_stream[stream_index].active=1;

        if(stream_index>=G39_IO_STREAM_IN_FIRST && stream_index<=G39_IO_STREAM_IN_LAST)
        {
            g39_usb_update_input_buffer_size(dev);
        }
    }
    else
    {
        bytes_per_packet=((dev->g39_dev.io_stream[stream_index].bytes_per_sec/1000+11)/12)*12;

        if(unlikely(bytes_per_packet>(G39_USB_OUTPUT_BUFFER_SIZE-sizeof(struct g39_usb_output_packet))))
        {
            bytes_per_packet=G39_USB_OUTPUT_BUFFER_SIZE-sizeof(struct g39_usb_output_packet);
        }
        else if(unlikely(bytes_per_packet<168))
        {
            bytes_per_packet=168;
        }

        dev->output.bytes_per_packet[stream_index-G39_IO_STREAM_OUT_FIRST]=bytes_per_packet;

        if(dev->output.stream_counter==0)
        {
            INIT_LIST_HEAD(&dev->output.avail_usb_buffers.list);

            for(i=0;i<G39_USB_OUTPUT_BUFFER_COUNT;i++)
            {
                dev->output.buffer[i].pending=0;

                dev->output.buffer[i].urb=usb_alloc_urb(0,GFP_KERNEL);

                if(unlikely(dev->output.buffer[i].urb==NULL))
                {
                    while(i--)
                    {
                        kfree(dev->output.buffer[i].mem);
                        dev->output.buffer[i].mem=NULL;

                        usb_free_urb(dev->output.buffer[i].urb);
                        dev->output.buffer[i].urb=NULL;
                    }

                    return -ENOMEM;
                }

                dev->output.buffer[i].mem=kmalloc(G39_USB_OUTPUT_BUFFER_SIZE,GFP_KERNEL);

                if(dev->output.buffer[i].mem==NULL)
                {
                    usb_free_urb(dev->output.buffer[i].urb);
                    dev->output.buffer[i].urb=NULL;

                    while(i--)
                    {
                        kfree(dev->output.buffer[i].mem);
                        dev->output.buffer[i].mem=NULL;

                        usb_free_urb(dev->output.buffer[i].urb);
                        dev->output.buffer[i].urb=NULL;
                    }

                    return -ENOMEM;
                }

                list_add_tail(&dev->output.buffer[i].list_entry,&dev->output.avail_usb_buffers.list);
            }


            thread=kthread_create(g39_usb_output_stream_thread,dev,G39_DEVICE_NAME);

            if(unlikely(IS_ERR(thread)))
            {
                for(i=0;i<G39_USB_OUTPUT_BUFFER_COUNT;i++)
                {
                    kfree(dev->output.buffer[i].mem);
                    dev->output.buffer[i].mem=NULL;

                    usb_free_urb(dev->output.buffer[i].urb);
                    dev->output.buffer[i].urb=NULL;
                }

                return PTR_ERR(thread);
            }

            dev->output.thread=thread;

            wake_up_process(thread);
        }

        dev->output.stream_counter++;

        dev->g39_dev.io_stream[stream_index].active=1;
    }

    return 0;
}

static void g39_usb_stop_stream(struct g39ddc_usb *dev,uint32_t stream_index)
{
 struct g39_io_stream *io_stream;
 int i;

    io_stream=dev->g39_dev.io_stream+stream_index;

    down_write(&io_stream->rw_semaphore);

    io_stream->active=0;

    up(&io_stream->complete_buffers.semaphore);

    up_write(&io_stream->rw_semaphore);

    if(G39_IS_INPUT_STREAM(stream_index))
    {
        dev->input.stream_counter--;

        if(dev->input.stream_counter==0)
        {
            kthread_stop(dev->input.thread);

            dev->input.thread=NULL;

            for(i=0;i<G39_USB_INPUT_BUFFER_COUNT;i++)
            {
                kfree(dev->input.buffer[i].mem);
                dev->input.buffer[i].mem=NULL;

                usb_free_urb(dev->input.buffer[i].urb);
                dev->input.buffer[i].urb=NULL;
            }
        }

        g39_io_stream_free_buffers(io_stream);

        if(dev->io_buffer[stream_index]!=NULL)
        {
            g39_io_buffer_free(dev->io_buffer[stream_index]);
            dev->io_buffer[stream_index]=NULL;
        }

        g39_usb_update_input_buffer_size(dev);
    }
    else
    {
        dev->output.stream_counter--;

        if(dev->output.stream_counter==0)
        {
            kthread_stop(dev->output.thread);

            dev->output.thread=NULL;

            for(i=0;i<G39_USB_OUTPUT_BUFFER_COUNT;i++)
            {
                kfree(dev->output.buffer[i].mem);
                dev->output.buffer[i].mem=NULL;

                usb_free_urb(dev->output.buffer[i].urb);
                dev->output.buffer[i].urb=NULL;
            }
        }

        g39_io_stream_free_buffers(io_stream);

        if(dev->io_buffer[stream_index]!=NULL)
        {
            g39_io_buffer_free(dev->io_buffer[stream_index]);
            dev->io_buffer[stream_index]=NULL;
        }
    }
}

static int g39_usb_if_start(struct g39ddc_usb *dev,uint16_t period)
{
 int status;
 uint16_t addresses[2];
 uint32_t values[2];


    dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].buffer_size=65536*sizeof(int16_t)+sizeof(uint16_t);
    dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].bytes_per_sec=dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].buffer_size*1000/period;

    addresses[0]=USB_REG_COMMAND_ID;
    values[0]=++dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].id;

    addresses[1]=USB_REG_IF_BUFF_CTRL;
    values[1]=period;

    status=g39_usb_set_regs(dev,addresses,values,2);

    if(unlikely(status<0))
    {
        return status;
    }

    status=g39_usb_start_stream(dev,G39_IO_STREAM_IN_IF);

    if(unlikely(status<0))
    {
        g39_usb_set_reg(dev,USB_REG_IF_BUFF_CTRL,0);
    }

    return status;
}

static void g39_usb_if_stop(struct g39ddc_usb *dev)
{
    if(likely(dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].active))
    {
        g39_usb_set_reg(dev,USB_REG_IF_BUFF_CTRL,0);

        g39_usb_stop_stream(dev,G39_IO_STREAM_IN_IF);
    }
}

static int g39_usb_put_buffer(struct g39ddc_usb *dev,uint32_t stream_index,struct g39_io_buffer *io_buffer)
{
    if(likely(G39_IS_INPUT_STREAM(stream_index)))
    {
        g39_io_stream_put_pending_buffer(&dev->g39_dev.io_stream[stream_index],io_buffer);
    }
    else
    {
        if(g39_io_stream_put_pending_buffer2(&dev->g39_dev.io_stream[stream_index],io_buffer))
        {
            wake_up_process(dev->output.thread);
        }
    }

    return 0;
}

static int g39_usb_mmap(struct g39ddc_usb *dev,struct vm_area_struct *vma)
{
    return -ENOSYS;
}

static int g39_usb_get_adc_level(struct g39ddc_usb *dev,uint16_t *level)
{
    return -ENOSYS;
}

static int g39_usb_wd(struct g39ddc_usb *dev)
{
 uint8_t *state;//[2];
 int status;


    state=kmalloc(2,GFP_KERNEL);

    if(state==NULL)
    {
        return -1;
    }

    status=usb_control_msg(dev->usb_dev, usb_rcvctrlpipe(dev->usb_dev,0),
                0x10,USB_TYPE_VENDOR|USB_DIR_IN,0,0,state,2,G39_USB_TIMEOUT);

    if(unlikely(status<0))
    {
        kfree(state);
        return -1;
    }

    status=(state[0]>=68);

    kfree(state);

    return status;
}

static int g39_usb_ddc1_setup(struct g39ddc_usb *dev,uint32_t channel,int enabled,uint32_t *coeffs)
{
 struct g39_ddc1_params *params;
 uint32_t reg_offset;
 int status;
 uint32_t i,j,tmp;
 uint16_t addresses[4];
 uint32_t values[4];

    params=dev->g39_dev.ddc1_params+channel;

    reg_offset=(USB_REG_DDC1_CTRL_1-USB_REG_DDC1_CTRL_0)*channel;

    addresses[0]=USB_REG_DDC1_CTRL_0+reg_offset;
    values[0]=params->control_word & 0xFFFE;

    addresses[1]=USB_REG_DDC1_TUNING_WORD_0+reg_offset;
    values[1]=params->tuning_word;

    addresses[2]=USB_REG_DDC1_CIC_CTRL_0+reg_offset;
    values[2]=params->cic1 | (((uint32_t)params->cic2)<<16);

    addresses[3]=USB_REG_DDC1_FIR_CTRL_0+reg_offset;
    values[3]=(params->fir_len>>1) | (((uint32_t)params->fir_control)<<16);

    if(coeffs!=NULL)
    {
        status=g39_usb_set_regs(dev,addresses,values,4);

        if(unlikely(status))
        {
            return status;
        }

        status=g39_usb_set_reg(dev,USB_REG_COEFF_WR_CTRL,1<<channel);

        for(i=0,j=(params->fir_len>>1)-1;i<params->fir_len>>2;i++,j--)
        {
            tmp=coeffs[i];
            coeffs[i]=coeffs[j];
            coeffs[j]=tmp;
        }

        status=g39_usb_set_reg_rep(dev,USB_REG_DDC1_FIR_COEFF_0+channel,coeffs,params->fir_len>>1);

        if(unlikely(status))
        {
            return status;
        }

        if(enabled)
        {
            status=g39_usb_set_reg(dev,USB_REG_DDC1_CTRL_0+reg_offset,params->control_word);
        }
        else
        {
            status=g39_usb_set_reg(dev,USB_REG_DDC1_CTRL_0+reg_offset,params->control_word & 0xFFFE);
        }
    }
    else
    {
        if(enabled)
        {
            values[0]=params->control_word;
        }

        status=g39_usb_set_regs(dev,addresses,values,4);
    }

    return status;
}

static int g39_usb_ddc1_start(struct g39ddc_usb *dev,uint32_t channel,uint32_t *coeffs)
{
 int status;

    status=g39_usb_ddc1_setup(dev,channel,1,coeffs);

    if(unlikely(status<0))
    {
        return status;
    }

    status=g39_usb_start_stream(dev,G39_IO_STREAM_IN_DDC1_0+channel);

    if(unlikely(status<0))
    {
        g39_usb_ddc1_setup(dev,channel,0,NULL);
    }

    return status;;
}

static void g39_usb_ddc1_stop(struct g39ddc_usb *dev,uint32_t channel)
{
    if(likely(dev->g39_dev.io_stream[G39_IO_STREAM_IN_DDC1_0+channel].active))
    {
        g39_usb_ddc1_setup(dev,channel,0,NULL);

        g39_usb_stop_stream(dev,G39_IO_STREAM_IN_DDC1_0+channel);
    }
}

static int g39_usb_ddc1_update_params(struct g39ddc_usb *dev,uint32_t channel)
{
    return g39_usb_ddc1_setup(dev,channel,1,NULL);
}

static int g39_usb_read_config(struct g39ddc_usb *dev)
{
 uint32_t size;
 uint32_t to_read;
 uint16_t offset;
 uint8_t *p;
 int status;

    size=8192;
    p=dev->g39_dev.config+4;
    offset=0;

    while(size)
    {
        if(size>=1024)
        {
            to_read=1024;
        }
        else
        {
            to_read=size;
        }

        status=usb_control_msg(dev->usb_dev,usb_rcvctrlpipe(dev->usb_dev,0),
                0x0D,USB_TYPE_VENDOR|USB_DIR_IN,offset,0,p,to_read,G39_USB_TIMEOUT);

        if(unlikely(status<=0))
        {
            return -EIO;
        }

        p+=status;
        size-=status;
        offset+=status;
    }

    return 0;
}

static int g39_usb_ddc2_start(struct g39ddc_usb *dev,uint32_t channel,struct g39_ddc2_start_params *params)
{
 uint32_t reg_offset;
 int status;
 struct g39_io_stream *io_stream;
 uint16_t address[4];
 uint32_t value[4];

    reg_offset=channel*(USB_REG_DDC2_SHIFT_CONTROL_1-USB_REG_DDC2_SHIFT_CONTROL_0);
    io_stream=&dev->g39_dev.io_stream[G39_IO_STREAM_IN_DDC2_0+channel];

    address[0]=USB_REG_COMMAND_ID;
    value[0]=((3+channel)<<16) | (++io_stream->id);

    address[1]=USB_REG_DDC2_FIR1_CONTROL1_0+reg_offset;
    value[1]=params->fir1.ctrl1.u32 & (~1);

    status=g39_usb_set_regs(dev,address,value,2);

    if(unlikely(status))
    {
        return status;
    }

    if(!params->fir1.ctrl1.bypass)
    {
        address[0]=USB_REG_DDC2_FIR1_CONTROL2_0+reg_offset;
        value[0]=params->fir1.ctrl2.u32;

        address[1]=USB_REG_COEFF_WR_CTRL;
        value[1]=1<<(2+channel);

        status=g39_usb_set_regs(dev,address,value,2);

        if(unlikely(status))
        {
            return status;
        }

        status=g39_usb_set_reg_rep(dev,USB_REG_DDC2_FIR_1_COEFF_0+channel,params->fir1.coeffs,(params->fir1.ctrl2.len+1)>>1);

        if(unlikely(status))
        {
            return status;
        }
    }


    status=g39_usb_set_reg(dev,USB_REG_DDC2_FIR2_CONTROL1_0+reg_offset,params->fir2.ctrl1.u32 & (~1));

    if(unlikely(status))
    {
        return status;
    }

    if(!params->fir2.ctrl1.bypass)
    {
        address[0]=USB_REG_DDC2_FIR2_CONTROL2_0+reg_offset;
        value[0]=params->fir2.ctrl2.u32;

        address[1]=USB_REG_COEFF_WR_CTRL;
        value[1]=1<<(4+channel);

        status=g39_usb_set_regs(dev,address,value,2);

        if(unlikely(status))
        {
            return status;
        }

        status=g39_usb_set_reg_rep(dev,USB_REG_DDC2_FIR_2_COEFF_0+channel,params->fir2.coeffs,(params->fir2.ctrl2.len+1)>>1);

        if(unlikely(status))
        {
            return status;
        }
    }

    address[0]=USB_REG_DDC2_RETIMER_0+reg_offset;
    value[0]=params->retimer.u32;

    address[1]=USB_REG_DDC2_FIR1_CONTROL1_0+reg_offset;
    value[1]=params->fir1.ctrl1.u32;

    address[2]=USB_REG_DDC2_FIR2_CONTROL1_0+reg_offset;
    value[2]=params->fir2.ctrl1.u32;

    address[3]=USB_REG_DDC2_SHIFT_CONTROL_0+reg_offset;
    value[3]=0x00000001;

    status=g39_usb_set_regs(dev,address,value,4);

    if(unlikely(status))
    {
        return status;
    }

    status=g39_usb_start_stream(dev,G39_IO_STREAM_IN_DDC2_0+channel);

    if(unlikely(status<0))
    {
        address[0]=USB_REG_DDC2_SHIFT_CONTROL_0+reg_offset;
        value[0]=0;

        address[1]=USB_REG_DDC2_FIR1_CONTROL1_0+reg_offset;
        value[1]=0;

        address[2]=USB_REG_DDC2_FIR2_CONTROL1_0+reg_offset;
        value[2]=0;

        g39_usb_set_regs(dev,address,value,3);
    }

    return status;
}

static void g39_usb_ddc2_stop(struct g39ddc_usb *dev,uint32_t channel)
{
 uint32_t reg_offset;
 uint16_t address[3];
 uint32_t value[3];

    if(likely(dev->g39_dev.io_stream[G39_IO_STREAM_IN_DDC2_0+channel].active))
    {
        reg_offset=channel*(USB_REG_DDC2_SHIFT_CONTROL_1-USB_REG_DDC2_SHIFT_CONTROL_0);

        address[0]=USB_REG_DDC2_SHIFT_CONTROL_0+reg_offset;
        value[0]=0;

        address[1]=USB_REG_DDC2_FIR1_CONTROL1_0+reg_offset;
        value[1]=0;

        address[2]=USB_REG_DDC2_FIR2_CONTROL1_0+reg_offset;
        value[2]=0;

        g39_usb_set_regs(dev,address,value,3);

        g39_usb_stop_stream(dev,G39_IO_STREAM_IN_DDC2_0+channel);
    }
}

static int g39_usb_ddc2_set_frequency(struct g39ddc_usb *dev,uint32_t channel,uint32_t tuning_word)
{
    return g39_usb_set_reg(dev,USB_REG_DDC2_SHIFT_TUNING_0+(USB_REG_DDC2_SHIFT_TUNING_1-USB_REG_DDC2_SHIFT_TUNING_0)*channel,tuning_word);
}

static int g39_usb_swp_start(struct g39ddc_usb *dev)
{
 int status;

    dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].id++;

    status=g39_usb_set_reg(dev,USB_REG_COMMAND_ID,dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].id);

    if(unlikely(status!=0))
    {
        return status;
    }


    dev->g39_dev.swp_active=1;

    status=g39_usb_start_stream(dev,G39_IO_STREAM_IN_IF);

    if(unlikely(status))
    {
        dev->g39_dev.swp_active=0;
    }

    return status;
}

static void g39_usb_swp_stop(struct g39ddc_usb *dev)
{
    if(likely(dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].active && dev->g39_dev.swp_active))
    {
        g39_usb_stop_stream(dev,G39_IO_STREAM_IN_IF);
        dev->g39_dev.swp_active=0;
    }
}

static int g39_usb_swp_snapshot(struct g39ddc_usb *dev)
{
    return g39_usb_set_reg(dev,USB_REG_IF_BUFF_CTRL,dev->g39_dev.swp_settling_time | 0x80000000);
}

static int g39_usb_ddc1_playback_start(struct g39ddc_usb *dev,uint32_t channel,uint32_t divider)
{
 int status;

    dev->output.ddc1_divider[channel]=divider;

    status=g39_usb_start_stream(dev,G39_IO_STREAM_OUT_DDC1_0+channel);

    if(unlikely(status))
    {
        dev->output.ddc1_divider[channel]=0;
    }

    return status;
}

static void g39_usb_ddc1_playback_stop(struct g39ddc_usb *dev,uint32_t channel)
{
    if(dev->g39_dev.io_stream[G39_IO_STREAM_OUT_DDC1_0+channel].active)
    {
        g39_usb_stop_stream(dev,G39_IO_STREAM_OUT_DDC1_0+channel);
        dev->output.ddc1_divider[channel]=0;
    }
}

static int g39_usb_dac_start(struct g39ddc_usb *dev,uint32_t dac_control)
{
 int status;

    dev->output.dac_control=dac_control;

    status=g39_usb_start_stream(dev,G39_IO_STREAM_OUT_DAC);

    if(unlikely(status))
    {
        dev->output.dac_control=0;
        g39_usb_set_reg(dev,USB_REG_DAC_CONTROL,0);
    }

    return status;
}

static void g39_usb_dac_stop(struct g39ddc_usb *dev)
{
    if(dev->g39_dev.io_stream[G39_IO_STREAM_OUT_DAC].active)
    {
        g39_usb_stop_stream(dev,G39_IO_STREAM_OUT_DAC);
        dev->output.dac_control=0;
    }
}

static void g39_usb_cancel_buffers(struct g39ddc_usb *dev,int stream_index)
{
 struct g39_io_stream *io_stream;
 struct g39_io_buffer *io_buffer;

    io_stream=&dev->g39_dev.io_stream[stream_index];

    while((io_buffer=g39_io_stream_get_pending_buffer(io_stream)))
    {
        g39_io_stream_put_complete_buffer(io_stream,io_buffer);
    }
}

static int g39_usb_initialize(struct g39ddc_usb *dev)
{
 int i;
 int status;

    dev->g39_dev.func_delete=(g39_func_delete_t)g39_usb_delete;
    dev->g39_dev.func_open=(g39_func_open_t)g39_usb_open;
    dev->g39_dev.func_close=(g39_func_close_t)g39_usb_close;
    dev->g39_dev.func_set_power=(g39_func_set_power_t)g39_usb_set_power;
    dev->g39_dev.func_configure_fpga=(g39_func_configure_fpga_t)g39_usb_configure_fpga;
    dev->g39_dev.func_set_spi_regs=(g39_func_set_spi_regs_t)g39_usb_set_spi_regs;
    dev->g39_dev.func_set_spi2_regs=(g39_func_set_spi2_regs_t)g39_usb_set_spi2_regs;
    dev->g39_dev.func_set_noise_blanker=(g39_func_set_noise_blanker_t)g39_usb_set_noise_blanker;
    dev->g39_dev.func_put_buffer=(g39_func_put_buffer_t)g39_usb_put_buffer;
    dev->g39_dev.func_if_start=(g39_func_if_start_t)g39_usb_if_start;
    dev->g39_dev.func_if_stop=(g39_func_if_stop_t)g39_usb_if_stop;
    dev->g39_dev.func_get_adc_level=(g39_func_get_adc_level_t)g39_usb_get_adc_level;
    dev->g39_dev.func_usb_control1_read=(g39_func_usb_control1_t)g39_usb_control1_read;
    dev->g39_dev.func_usb_control1_write=(g39_func_usb_control1_t)g39_usb_control1_write;
    dev->g39_dev.func_usb_control2_read=(g39_func_usb_control2_t)g39_usb_control2_read;
    dev->g39_dev.func_usb_control2_write=(g39_func_usb_control2_t)g39_usb_control2_write;
    dev->g39_dev.func_mmap=(g39_func_mmap_t)g39_usb_mmap;
    dev->g39_dev.func_wd=(g39_func_wd_t)g39_usb_wd;
    dev->g39_dev.func_ddc1_start=(g39_func_ddc1_start_t)g39_usb_ddc1_start;
    dev->g39_dev.func_ddc1_stop=(g39_func_ddc1_stop_t)g39_usb_ddc1_stop;
    dev->g39_dev.func_ddc1_update_params=(g39_func_ddc1_update_params_t)g39_usb_ddc1_update_params;
    dev->g39_dev.func_ddc2_start=(g39_func_ddc2_start_t)g39_usb_ddc2_start;
    dev->g39_dev.func_ddc2_stop=(g39_func_ddc2_stop_t)g39_usb_ddc2_stop;
    dev->g39_dev.func_ddc2_set_frequency=(g39_func_ddc2_set_frequency_t)g39_usb_ddc2_set_frequency;
    dev->g39_dev.func_swp_start=(g39_func_swp_start_t)g39_usb_swp_start;
    dev->g39_dev.func_swp_stop=(g39_func_swp_stop_t)g39_usb_swp_stop;
    dev->g39_dev.func_swp_snapshot=(g39_func_swp_snapshot_t)g39_usb_swp_snapshot;
    dev->g39_dev.func_ddc1_playback_start=(g39_func_ddc1_playback_start_t)g39_usb_ddc1_playback_start;
    dev->g39_dev.func_ddc1_playback_stop=(g39_func_ddc1_playback_stop_t)g39_usb_ddc1_playback_stop;
    dev->g39_dev.func_dac_start=(g39_func_dac_start_t)g39_usb_dac_start;
    dev->g39_dev.func_dac_stop=(g39_func_dac_stop_t)g39_usb_dac_stop;
    dev->g39_dev.func_cancel_buffers=(g39_func_cancel_buffers_t)g39_usb_cancel_buffers;

    spin_lock_init(&dev->input.complete_usb_buffers.spinlock);

    dev->input.buffer_size=G39_USB_INPUT_BUFFER_MAX_SIZE;

    for(i=0;i<G39_USB_INPUT_BUFFER_COUNT;i++)
    {
        dev->input.buffer[i].dev=dev;
    }

    spin_lock_init(&dev->output.avail_usb_buffers.spinlock);

    for(i=0;i<G39_USB_OUTPUT_BUFFER_COUNT;i++)
    {
        dev->output.buffer[i].dev=dev;
    }

    status=g39_usb_setup(dev);

    if(unlikely(status))
    {
        return status;
    }

    status=g39_usb_read_config(dev);

    if(unlikely(status))
    {
        G39_DEV_ERROR(&dev->usb_dev->dev,"failed to read configuration data!\n");
        return status;
    }

    return 0;
}

static int g39_usb_probe(struct usb_interface *interface, const struct usb_device_id *id)
{
 int status;
 struct g39ddc_usb *dev;
 struct usb_device *usb_dev;

    usb_dev=interface_to_usbdev(interface);

    G39_DEV_INFO(&usb_dev->dev,"probe\n");

    if(unlikely(id->idVendor!=G39_USB_VID || id->idProduct!=G39_USB_PID))
    {
        G39_DEV_ERROR(&usb_dev->dev,"unsuported device!\n");
        return -ENODEV;
    }

    if(unlikely(usb_dev->speed<USB_SPEED_HIGH))
    {
        G39_DEV_ERROR(&usb_dev->dev,"device is not running at top speed!\n");
        return -ENODEV;
    }

    dev=kmalloc(sizeof(*dev),GFP_KERNEL);

    if(unlikely(dev==NULL))
    {
        G39_DEV_ERROR(&usb_dev->dev,"failed to allocate memory for device object!\n");
        return -ENOMEM;
    }

    memset(dev,0,sizeof(*dev));

    dev->g39_dev.config[0]=G39_INTERFACE_TYPE_USB;
    dev->usb_dev=usb_get_dev(usb_dev);
    dev->g39_dev.p_dev=&usb_dev->dev;
    dev->usb_interface=usb_get_intf(interface);

    usb_set_intfdata(interface,dev);

    g39_initialize(&dev->g39_dev);

    status=g39_usb_initialize(dev);

    if(unlikely(status<0))
    {
        g39_uninitialize(&dev->g39_dev);
        usb_set_intfdata(interface,NULL);
        usb_put_intf(dev->usb_interface);
        usb_put_dev(dev->usb_dev);
        kfree(dev);
        return status;
    }

    status=g39_register(&dev->g39_dev);

    if(unlikely(status<0))
    {
        g39_delete(&dev->g39_dev);
        return status;
    }

    return 0;
}

static void g39_usb_disconnect(struct usb_interface *interface)
{
 struct g39ddc_usb *dev;

    dev=usb_get_intfdata(interface);

    G39_DEV_INFO(&dev->usb_dev->dev,"disconnect\n");

    usb_set_intfdata(interface,NULL);

    if(likely(dev!=NULL))
    {
        dev->g39_dev.state.connected=0;

        wake_up_all(&dev->g39_dev.power_wait_queue);

        g39_unregister(&dev->g39_dev);
    }
}

#ifdef CONFIG_PM

static int g39_usb_suspend(struct usb_interface *interface,pm_message_t message)
{
 struct g39ddc_usb *dev;

    dev=usb_get_intfdata(interface);

    if(likely(dev!=NULL))
    {
        G39_DEV_LOCK2;

        if(dev->g39_dev.state.power)
        {
            dev->g39_dev.state.suspended=1;
            g39_set_power(&dev->g39_dev,0,0);
        }

        G39_DEV_UNLOCK2;
    }

    return 0;
}

static int g39_usb_resume(struct usb_interface *interface)
{
 struct g39ddc_usb *dev;
 int status;

    dev=usb_get_intfdata(interface);

    if(likely(dev!=NULL))
    {
        status=g39_usb_setup(dev);

        if(unlikely(status!=0))
        {
            return status;
        }
    }

    return 0;
}

#endif //CONFIG_PM


static struct usb_device_id g39_usb_id_table[]=
{
    {USB_DEVICE(G39_USB_VID,G39_USB_PID)},
    {},
};

static struct usb_driver g39_usb_driver=
{
    .name=G39_DEVICE_NAME,
    .probe=g39_usb_probe,
    .disconnect=g39_usb_disconnect,
    .id_table=g39_usb_id_table,
#ifdef CONFIG_PM
    .suspend=g39_usb_suspend,
    .resume=g39_usb_resume,
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
    .reset_resume=g39_usb_resume
#endif //KERNEL_VERSION(2,6,23)
#endif //CONFIG_PM
};

MODULE_DEVICE_TABLE(usb,g39_usb_id_table);

#endif //__G39_USB__

#ifdef __G39_PCI__

static int g39_pci_initialize(struct g39ddc_pci *dev);


static void g39_pci_setup(struct g39ddc_pci *dev)
{
 uint32_t i,j;

    bar4_write32(GPIO_INT_MASK_SET,0xFFFFFFFF);

    bar4_write32(INT_CFG0,0);
    bar4_write32(INT_CFG1,0);
    bar4_write32(INT_CFG2,0);
    bar4_write32(INT_CFG3,0);

    i=bar4_read32(PCI_SYS_CFG_SYSTEM);
    i&=~(0x3000);
	bar4_write32(PCI_SYS_CFG_SYSTEM,i);
	bar4_read32(PCI_SYS_CFG_SYSTEM);

    i=bar4_read32(PCI_SYS_CFG_SYSTEM);
    i&=~0xC000;
    bar4_write32(PCI_SYS_CFG_SYSTEM,i);
    bar4_read32(PCI_SYS_CFG_SYSTEM);

    if((bar4_read32(CLK_CSR) & 0x0FF7F3FF)!=0x0001F07C)
    {
        bar4_write32(CLK_CSR,0x0001F07C | 0x8080000);
        bar4_read32(CLK_CSR);

        bar4_write32(CLK_CSR,0x0001F07C);

        for(j=0;j<10;j++)
        {
            i=bar4_read32(CLK_CSR);

            if((i & 0xC0000000)==0xC0000000)
            {
                break;
            }

            msleep(50);
        }
    }

    i=bar4_read32(PCI_SYS_CFG_SYSTEM);
    i|=0x4000;

    bar4_write32(PCI_SYS_CFG_SYSTEM,i);
    bar4_read32(PCI_SYS_CFG_SYSTEM);


    msleep(250);


    bar4_write32(GPIO_INT_TYPE,GPIO_BIT_DMA_IRQ);
    bar4_write32(GPIO_INT_VALUE,GPIO_BIT_DMA_IRQ);

    bar4_read32(GPIO_INT_STATUS);
    bar4_read32(INT_STAT);
}

static int g39_pci_probe(struct pci_dev *pci_dev, const struct pci_device_id *id)
{
 int status;
 struct g39ddc_pci *dev;
 unsigned long start,len;

    G39_DEV_INFO(&pci_dev->dev,"probe\n");

    if(pci_dev->vendor!=G39_PCI_VENDOR || pci_dev->device!=G39_PCI_DEVICE ||
       pci_dev->subsystem_vendor!=G39_PCI_SUBSYSTEM_VENDOR || pci_dev->subsystem_device!=G39_PCI_SUBSYSTEM_DEVICE)
    {
        G39_DEV_ERROR(&pci_dev->dev,"unsuported device!\n");
        return -ENODEV;
    }

    dev=kmalloc(sizeof(struct g39ddc_pci),GFP_KERNEL);

    if(unlikely(dev==NULL))
    {
        G39_DEV_ERROR(&pci_dev->dev,"failed to allocate memory for device object!\n");
        return -ENOMEM;
    }

    memset(dev,0,sizeof(*dev));

    dev->pci_dev=pci_dev_get(pci_dev);
    dev->g39_dev.p_dev=&pci_dev->dev;
    dev->g39_dev.config[0]=G39_INTERFACE_TYPE_PCIE;

    status=pci_enable_device(dev->pci_dev);

    if(unlikely(status))
    {
        G39_DEV_ERROR(&pci_dev->dev,"failed to enable device!\n");
        goto error_kfree;
    }

    pci_set_drvdata(dev->pci_dev,dev);

    if(likely(pci_set_dma_mask(dev->pci_dev,DMA_BIT_MASK(64))==0))
    {
        if(unlikely(pci_set_consistent_dma_mask(dev->pci_dev,DMA_BIT_MASK(64))!=0))
        {
            if(unlikely(pci_set_consistent_dma_mask(dev->pci_dev,DMA_BIT_MASK(32))!=0))
            {
                G39_DEV_ERROR(&pci_dev->dev,"no suitable DMA available!\n");
                status=-EIO;
                goto error_pci_disable_device;
            }
        }

        dev->dma_64=1;
    }
    else
    {
        if(unlikely(pci_set_dma_mask(dev->pci_dev,DMA_BIT_MASK(32))))
        {
            G39_DEV_ERROR(&pci_dev->dev,"no suitable DMA available!\n");
            status=-EIO;
            goto error_pci_disable_device;
        }

        if(unlikely(pci_set_consistent_dma_mask(dev->pci_dev,DMA_BIT_MASK(32))))
        {
            G39_DEV_ERROR(&pci_dev->dev,"no suitable DMA available!\n");
            status=-EIO;
            goto error_pci_disable_device;
        }
    }

    pci_set_master(dev->pci_dev);

    status=pci_request_regions(dev->pci_dev,pci_name(dev->pci_dev));

    if(unlikely(status))
    {
        G39_DEV_ERROR(&pci_dev->dev,"failed to reserve PCI I/O and memory resources!\n");
        goto error_pci_disable_device;
    }

    /*if(pci_enable_msi(dev->pci_dev)==0)
    {
        dev->msi=1;
    }
    else
    {
        pci_disable_msi(dev->pci_dev);
    }*/

    if(unlikely((pci_resource_flags(dev->pci_dev,0) & IORESOURCE_MEM)==0 ||
                (pci_resource_flags(dev->pci_dev,4) & IORESOURCE_MEM)==0))
    {
        G39_DEV_ERROR(&pci_dev->dev,"device memory resources not found\n");
        status=-EIO;
        goto error_pci_release_regions;
    }

    start=pci_resource_start(dev->pci_dev,0);
    len=pci_resource_len(dev->pci_dev,0);

    dev->bar0=ioremap_nocache(start,len);

    if(unlikely(dev->bar0==NULL))
    {
        G39_DEV_ERROR(&pci_dev->dev,"failed to map memory resource!\n");
        status=-ENOMEM;
        goto error_pci_release_regions;
    }

    start=pci_resource_start(dev->pci_dev,4);
    len=pci_resource_len(dev->pci_dev,4);

    dev->bar4=ioremap_nocache(start,len);

    if(unlikely(dev->bar4==NULL))
    {
        G39_DEV_ERROR(&pci_dev->dev,"failed to map memory resource!\n");
        status=-ENOMEM;
        goto error_unmap_bar0;
    }

    g39_initialize(&dev->g39_dev);

    status=g39_pci_initialize(dev);

    if(unlikely(status))
    {
        g39_uninitialize(&dev->g39_dev);
        goto error_unmap_bar4;
    }

    status=g39_register(&dev->g39_dev);

    if(unlikely(status<0))
    {
        g39_delete(&dev->g39_dev);
        return status;
    }

    return 0;

    error_unmap_bar4:
        iounmap(dev->bar4);

    error_unmap_bar0:
        iounmap(dev->bar0);

    error_pci_release_regions:
        if(dev->msi)
        {
            pci_disable_msi(dev->pci_dev);
        }

        pci_release_regions(dev->pci_dev);

    error_pci_disable_device:
        pci_set_drvdata(dev->pci_dev,NULL);
        pci_disable_device(dev->pci_dev);

    error_kfree:
        pci_dev_put(dev->pci_dev);

        kfree(dev);

    return status;
}

static void g39_pci_remove(struct pci_dev *pci_dev)
{
 struct g39ddc_pci *dev;

    dev=pci_get_drvdata(pci_dev);

    if(likely(dev!=NULL))
    {
        pci_set_drvdata(pci_dev,NULL);

        dev->g39_dev.state.connected=0;

        g39_unregister(&dev->g39_dev);
    }
}


static void g39_pci_delete(struct g39ddc_pci *dev)
{
 int i;

    iounmap(dev->bar4);
    iounmap(dev->bar0);

    if(dev->msi)
    {
        pci_disable_msi(dev->pci_dev);
    }

    pci_disable_device(dev->pci_dev);
    pci_release_regions(dev->pci_dev);

    pci_dev_put(dev->pci_dev);

    for(i=0;i<G39_IO_STREAM_COUNT;i++)
    {
        g39_mutex_destroy(&dev->dma_channel[i].mutex);
    }
}


static int g39_pci_usb_control1(struct g39ddc_pci *dev,const uint16_t *params,void __user *user_buffer,uint16_t buffer_size,void __user *transferred)
{
    return -ENOSYS;
}

static int g39_pci_usb_control2(struct g39ddc_pci *dev,void __user *user_buffer,uint16_t buffer_size,void __user *transferred)
{
    return -ENOSYS;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19)
irqreturn_t g39_pci_interrupt_handler(int irqno, void *dev_id)
#else
irqreturn_t g39_pci_interrupt_handler(int irqno, void *dev_id, struct pt_regs *regs)
#endif
{
 struct g39ddc_pci *dev;
 unsigned long flags;
 uint32_t int_stat;
 uint32_t gpio_int_stat;

    dev=dev_id;

    spin_lock_irqsave(&dev->irq_spinlock,flags);

    int_stat=bar4_read32(INT_STAT);

    if((int_stat & bar4_read32(INT_CFG0))==0)
    {
        spin_unlock_irqrestore(&dev->irq_spinlock,flags);
        return IRQ_NONE;
    }

    if(unlikely(int_stat & 1))
    {
        bar4_read32(TWI_IRT_STATUS);
    }

    if(unlikely(int_stat & 2))
    {
        bar4_read32(FCL_IRQ);
    }

    if(likely(int_stat & IRQ_GPIO))
    {
        gpio_int_stat=bar4_read32(GPIO_INT_STATUS);

        if(likely(dev->g39_dev.state.fpga && (gpio_int_stat & GPIO_BIT_DMA_IRQ)))
        {
            dev->dma_status_out|=bar0_read32(REG_DMA_IRQ_STATUS_OUT);
            dev->dma_status_in|=bar0_read32(REG_DMA_IRQ_STATUS_IN);

            queue_work(dev->g39_dev.workqueue,&dev->irq_work);
        }
    }

    bar4_write32(INT_STAT,0xFFFFFFF3);
    bar4_read32(INT_STAT);

    spin_unlock_irqrestore(&dev->irq_spinlock,flags);

    return IRQ_HANDLED;
}

static void g39_pci_free_sg_list_memory(struct g39ddc_pci *dev)
{
 uint32_t i,j;

    for(i=0;i<G39_IO_STREAM_COUNT;i++)
    {
        for(j=0;j<2;j++)
        {
            if(likely(dev->dma_channel[i].subchannel[j].sg_mem_cpu!=NULL))
            {
                pci_free_consistent(dev->pci_dev,dev->dma_sg_list_memory_size+PAGE_SIZE-1,
                    dev->dma_channel[i].subchannel[j].sg_mem_cpu,
                    dev->dma_channel[i].subchannel[j].sg_mem);

                dev->dma_channel[i].subchannel[j].sg_mem_cpu=NULL;
                dev->dma_channel[i].subchannel[j].sg_mem=0;
                dev->dma_channel[i].subchannel[j].aligned_sg_mem_cpu=NULL;
                dev->dma_channel[i].subchannel[j].aligned_sg_mem=0;
            }
        }
    }
}

static int g39_pci_allocate_sg_list_memory(struct g39ddc_pci *dev)
{
 uint32_t i,j;
 void *addr;
 dma_addr_t dma_addr;

    for(i=0;i<G39_IO_STREAM_COUNT;i++)
    {
        for(j=0;j<2;j++)
        {
            addr=pci_alloc_consistent(dev->pci_dev,dev->dma_sg_list_memory_size+PAGE_SIZE-1,&dma_addr);

            if(unlikely(addr==NULL))
            {
                g39_pci_free_sg_list_memory(dev);
                return -ENOMEM;
            }

            dev->dma_channel[i].subchannel[j].sg_mem_cpu=addr;
            dev->dma_channel[i].subchannel[j].sg_mem=dma_addr;

            dev->dma_channel[i].subchannel[j].aligned_sg_mem_cpu=(void*)(((uintptr_t)addr+PAGE_SIZE-1) & PAGE_MASK);
            dev->dma_channel[i].subchannel[j].aligned_sg_mem=(dma_addr+PAGE_SIZE-1) & PAGE_MASK;
        }
    }

    return 0;
}

static int g39_pci_open(struct g39ddc_pci *dev)
{
 int status;

    status=g39_pci_allocate_sg_list_memory(dev);

    if(unlikely(status))
    {
        G39_DEV_ERROR(&dev->pci_dev->dev,"failed to allocate coherent DMA memory!\n");
        return status;
    }

#if LINUX_VERSION_CODE>=KERNEL_VERSION(2,6,22)
    status=request_irq(dev->pci_dev->irq,g39_pci_interrupt_handler,IRQF_SHARED,G39_DEVICE_NAME,dev);
#else
    status=request_irq(dev->pci_dev->irq,g39_pci_interrupt_handler,SA_SHIRQ|SA_INTERRUPT,G39_DEVICE_NAME,dev);
#endif

    if(unlikely(status))
    {
        G39_DEV_ERROR(&dev->pci_dev->dev,"failed to register interrupt handler!\n");
        g39_pci_free_sg_list_memory(dev);
        return status;
    }

    return 0;
}

static void g39_pci_close(struct g39ddc_pci *dev)
{
    free_irq(dev->pci_dev->irq,dev);
    g39_pci_free_sg_list_memory(dev);
}

static int g39_pci_set_power(struct g39ddc_pci *dev,uint8_t power)
{
 unsigned long flags;

    if(power)
    {
        if(unlikely(dev->disabled))
        {
            return -EIO;
        }

        gpio_set(GPIO_BIT_FE_POWER);

        if(likely(dev->g39_dev.config[1]))
        {
            msleep(1);

            if((bar4_read32(GPIO_INPUT_VALUE) & GPIO_BIT_POWER_STATE)==0)
            {
                gpio_clr(GPIO_BIT_FE_POWER);
                return -EIO;
            }
        }
    }
    else
    {
        bar4_write32(GPIO_INT_MASK_SET,GPIO_BIT_DMA_IRQ);

        spin_lock_irqsave(&dev->irq_spinlock,flags);

        bar4_write32(INT_CFG0,0);
        bar4_read32(INT_CFG0);

        spin_unlock_irqrestore(&dev->irq_spinlock,flags);

        //flush_workqueue(dev->g39_dev.workqueue);

        gpio_clr(GPIO_BIT_FE_POWER|GPIO_BIT_FPGA_RESET);
    }

    return 0;
}

static int g39_pci_configure_fpga(struct g39ddc_pci *dev,void __user *user_buffer,uint32_t size)
{
 uint8_t *buffer;
 uint8_t u8;
 uint32_t u32;
 uint32_t i,j;
 unsigned long flags;

    buffer=vmalloc(size);

    if(buffer==NULL)
    {
        return -ENOMEM;
    }

    if(unlikely(copy_from_user(buffer,user_buffer,size)!=0))
    {
        vfree(buffer);
        return -EFAULT;
    }


    u32=bar4_read32(PCI_SYS_CFG_SYSTEM);
    u32&=~0xC000;
    bar4_write32(PCI_SYS_CFG_SYSTEM,u32);
    bar4_read32(PCI_SYS_CFG_SYSTEM);

    gpio_clr(GPIO_BIT_FPGA_RESET);


	u32&=~(0x3000);
	bar4_write32(PCI_SYS_CFG_SYSTEM,u32);
	bar4_read32(PCI_SYS_CFG_SYSTEM);


    bar4_write32(FCL_CLK_DIV,0);

    bar4_write32(FCL_CTRL,0x40);

    u32=bar4_read32(FCL_CTRL);

    if(unlikely(u32!=0x40))
    {
        goto error_fcl;
    }


    bar4_write32(FCL_CTRL,0x00);

    bar4_read32(FCL_IRQ);

    bar4_write32(FCL_EN,0x17);

    bar4_write32(FCL_IODATA_OUT,0x04);
    bar4_write32(FCL_IODATA_OUT,0x00);

	for(i=0;i<1000000;i++)
	{
        if((bar4_read32(FCL_STATUS) & 0x28)==0)
        {
            break;
        }
	}

    if(unlikely(i>=1000000))
    {
        goto error_fcl;
    }

    bar4_write32(FCL_IODATA_OUT,0x04);

    udelay(40);

    for(i=0;i<1000000;i++)
    {
		if((bar4_read32(FCL_STATUS) & 0x20)!=0)
		{
		    break;
		}
	}

    if(unlikely(i>=1000000))
    {
        goto error_fcl;
    }

    udelay(40);

	for(i=0;i<size;i++)
	{
        u8=buffer[i];

		for(j=0;j<8;j++,u8>>=1)
		{
			if(u8&1)
			{
				bar4_write32(FCL_IODATA_OUT,0x06);
                bar4_write32(FCL_IODATA_OUT,0x07);
			}
			else
			{
                bar4_write32(FCL_IODATA_OUT,0x04);
                bar4_write32(FCL_IODATA_OUT,0x05);
			}
		}

		if((i & 0x7F) == 0)
		{
            u32=bar4_read32(FCL_STATUS);

			if(unlikely((u32 & 0x28)==0))
			{
			    goto error_fcl;
			}
		}
	}

	for(i=0;i<16;i++)
	{
        bar4_write32(FCL_IODATA_OUT,0x04);
        bar4_write32(FCL_IODATA_OUT,0x05);
	}

    bar4_write32(FCL_IODATA_OUT,0x04);

    u32=bar4_read32(FCL_STATUS);

	if(unlikely((u32 & 0x8)==0))
	{
	    goto error_fcl;
    }

    u32=bar4_read32(PCI_SYS_CFG_SYSTEM);
    u32|=0x1000;
    bar4_write32(PCI_SYS_CFG_SYSTEM,u32);
    bar4_read32(PCI_SYS_CFG_SYSTEM);

    msleep(50);

    u32|=0x4000;
    bar4_write32(PCI_SYS_CFG_SYSTEM,u32);
    bar4_read32(PCI_SYS_CFG_SYSTEM);
    gpio_set(GPIO_BIT_FPGA_RESET);

    msleep(40);

    bar0_write32(REG_DMA_CONFIG,dev->dma_64?1:0);

    bar4_read32(GPIO_INT_STATUS);

    spin_lock_irqsave(&dev->irq_spinlock,flags);

    bar4_write32(INT_STAT,IRQ_GPIO);
    bar4_write32(INT_CFG0,bar4_read32(INT_CFG0) | IRQ_GPIO);
    bar4_read32(INT_CFG0);

    bar4_write32(GPIO_INT_MASK_CLR,GPIO_BIT_DMA_IRQ);

    spin_unlock_irqrestore(&dev->irq_spinlock,flags);

    vfree(buffer);

    return 0;

    error_fcl:
        /*gpio_clr(GPIO_BIT_FPGA_RESET);

        u32=bar4_read32(PCI_SYS_CFG_SYSTEM);
        u32|=0x1000;
        bar4_write32(PCI_SYS_CFG_SYSTEM,u32);
        bar4_read32(PCI_SYS_CFG_SYSTEM);*/

        u32|=0x4000;
        bar4_write32(PCI_SYS_CFG_SYSTEM,u32);
        bar4_read32(PCI_SYS_CFG_SYSTEM);

        vfree(buffer);

    return -EIO;
}

static int g39_pci_set_spi_regs(struct g39ddc_pci *dev,struct g39_spi_register __user *user_buffer,uint16_t count,uint16_t id)
{
 struct g39_spi_register *buffer;
 uint32_t size;
 uint32_t i;

    size=count*sizeof(struct g39_spi_register);

    buffer=kmalloc(size,GFP_KERNEL);

    if(unlikely(buffer==NULL))
    {
        return -ENOMEM;
    }

    if(unlikely(copy_from_user(buffer,user_buffer,size)!=0))
    {
        kfree(buffer);
        return -EFAULT;
    }

    for(i=0;i<count;i++)
    {
        bar0_write32(REG_SPI,(buffer[i].address<<24) | buffer[i].value);
        bar0_read32(REG_SPI);

        udelay(200);
    }

    kfree(buffer);

    return 0;
}

static int g39_pci_set_spi2_regs(struct g39ddc_pci *dev,struct g39_spi2_register __user *user_buffer,uint16_t count,uint16_t id)
{
 struct g39_spi2_register *buffer;
 uint32_t size;
 uint32_t i;

    size=count*sizeof(struct g39_spi2_register);

    buffer=kmalloc(size,GFP_KERNEL);

    if(unlikely(buffer==NULL))
    {
        return -ENOMEM;
    }

    if(unlikely(copy_from_user(buffer,user_buffer,size)!=0))
    {
        kfree(buffer);
        return -EFAULT;
    }

    for(i=0;i<count;i++)
    {
        bar0_write32(REG_SPI2_LO,buffer[i].u32[0]);
        bar0_write32(REG_SPI2_HI,buffer[i].u32[1]);
        bar0_read32(REG_SPI2_HI);

        udelay(250);
    }

    kfree(buffer);

    return 0;
}

static int g39_pci_set_noise_blanker(struct g39ddc_pci *dev,uint16_t value)
{
    bar0_write32(REG_DATA_LIMIT,value);
    return 0;
}


static int g39_pci_mmap(struct g39ddc_pci *dev,struct vm_area_struct *vma)
{
 int status;
 unsigned long size;

    size=vma->vm_end-vma->vm_start;

    if(unlikely(vma->vm_pgoff!=0 || size!=pci_resource_len(dev->pci_dev,4)))
    {
        return -EINVAL;
    }

    vma->vm_flags|=VM_IO;
    vma->vm_page_prot=pgprot_noncached(vma->vm_page_prot);

    status=io_remap_pfn_range(vma,vma->vm_start,(pci_resource_start(dev->pci_dev,4))>>PAGE_SHIFT,
				       size,vma->vm_page_prot);

    if(unlikely(status))
    {
        return -EAGAIN;
    }

    return 0;
}

static void g39_pci_start_dma_transfer(struct g39ddc_pci *dev,struct g39_io_buffer *io_buffer,struct g39_dma_channel *channel,uint32_t subchannel_index)
{
 struct scatterlist *sg;
 int i,j;
 struct g39_dma_subchannel *subchannel;
 struct g39_sg_segment_32 *sg32;
 struct g39_sg_segment_64 *sg64;
 uint32_t reg;

    if(channel->out)
    {
        reg=REG_DMA_PTR_OUT_BASE+channel->index*4*4+subchannel_index*4*2;
    }
    else
    {
        reg=REG_DMA_PTR_IN_BASE+channel->index*4*4+subchannel_index*4*2;

        io_buffer->changed=1;
    }

    g39_io_buffer_sg_sync_for_device(io_buffer);

    subchannel=channel->subchannel+subchannel_index;

    subchannel->io_buffer=io_buffer;

    if(dev->dma_64)
    {
        sg64=subchannel->aligned_sg_mem_cpu;

        j=0;


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
        for_each_sg(io_buffer->sg_list,sg,io_buffer->sg_segment_count,i)
        {
            sg64[j].address=sg_dma_address(sg);
            sg64[j].length=sg_dma_len(sg)>>2;

            j++;

            if(j>=G39_DMA_PAGE_SIZE/sizeof(struct g39_sg_segment_64))
            {
                j=0;
                sg64=(struct g39_sg_segment_64*)((uintptr_t)sg64+G39_DMA_PAGE_SIZE);
            }
        }
#else
        sg=io_buffer->sg_list;

        for(i=0;i<io_buffer->sg_segment_count;i++)
        {
            sg64[j].address=sg_dma_address(sg);
            sg64[j].length=sg_dma_len(sg)>>2;

            j++;

            if(j>=G39_DMA_PAGE_SIZE/sizeof(struct g39_sg_segment_64))
            {
                j=0;
                sg64=(struct g39_sg_segment_64*)((uintptr_t)sg64+G39_DMA_PAGE_SIZE);
            }

            sg++;
        }
#endif

        sg64[j].address=0;
        sg64[j].length=0;

        bar0_write32(reg+4,((uint64_t)subchannel->aligned_sg_mem)>>32);
        bar0_write32(reg,((uint64_t)subchannel->aligned_sg_mem)&0xFFFFFFFF);
    }
    else
    {
        sg32=subchannel->aligned_sg_mem_cpu;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
        for_each_sg(io_buffer->sg_list,sg,io_buffer->sg_segment_count,i)
        {
            sg32[i].address=sg_dma_address(sg);
            sg32[i].length=sg_dma_len(sg)>>2;
        }
#else
        sg=io_buffer->sg_list;

        for(i=0;i<io_buffer->sg_segment_count;i++)
        {
            sg32[i].address=sg_dma_address(sg);
            sg32[i].length=sg_dma_len(sg)>>2;

            sg++;
        }
#endif

        sg32[i].address=0;
        sg32[i].length=0;

        bar0_write32(reg,subchannel->aligned_sg_mem);
    }
}

static void g39_pci_disable_dma_channel(struct g39ddc_pci *dev,uint32_t stream_index)
{
 struct g39_dma_channel *channel;
 struct g39_io_stream *io_stream;
 unsigned long flags;
 int i;

    channel=dev->dma_channel+stream_index;
    io_stream=dev->g39_dev.io_stream+stream_index;

    down_write(&io_stream->rw_semaphore);

    io_stream->active=0;

    up(&io_stream->complete_buffers.semaphore);

    up_write(&io_stream->rw_semaphore);

    spin_lock_irqsave(&dev->irq_spinlock,flags);

    if(unlikely(channel->out))
    {
        dev->dma_enabled_out&=~(1<<channel->index);

        bar0_write32(REG_DMA_ENABLE_OUT,dev->dma_enabled_out);

        dev->dma_subchannel_mask_out&=~(3<<(channel->index<<1));
        dev->dma_status_out&=dev->dma_subchannel_mask_out;

        spin_unlock_irqrestore(&dev->irq_spinlock,flags);

        bar0_read32(REG_DMA_ENABLE_OUT);
    }
    else
    {
        dev->dma_enabled_in&=~(1<<channel->index);

        bar0_write32(REG_DMA_ENABLE_IN,dev->dma_enabled_in);

        dev->dma_subchannel_mask_in&=~(3<<(channel->index<<1));
        dev->dma_status_in&=dev->dma_subchannel_mask_in;

        spin_unlock_irqrestore(&dev->irq_spinlock,flags);

        bar0_read32(REG_DMA_ENABLE_IN);
    }


    g39_io_stream_free_buffers(io_stream);

    for(i=0;i<2;i++)
    {
        if(channel->subchannel[i].io_buffer)
        {
            g39_io_buffer_free(channel->subchannel[i].io_buffer);
            channel->subchannel[i].io_buffer=NULL;
        }
    }
}

static int g39_pci_enable_dma_channel(struct g39ddc_pci *dev,uint32_t stream_index)
{
 struct g39_dma_channel *channel;
 unsigned long flags;

    if(unlikely((dev->g39_dev.io_stream[stream_index].buffer_size & 0xFF)!=0))
    {
        return -EINVAL;
    }

    channel=dev->dma_channel+stream_index;

    if(unlikely(channel->out))
    {
        bar0_write32(REG_DMA_FIFO_RESET,(1<<channel->index));
    }
    else
    {
        bar0_write32(REG_DMA_FIFO_RESET,(1<<(16+channel->index)));
    }

    channel->subchannel_index=0;

    spin_lock_irqsave(&dev->irq_spinlock,flags);

    if(unlikely(channel->out))
    {
        dev->dma_enabled_out|=1<<channel->index;

        bar0_write32(REG_DMA_ENABLE_OUT,dev->dma_enabled_out);

        dev->dma_subchannel_mask_out|=3<<(channel->index<<1);

        spin_unlock_irqrestore(&dev->irq_spinlock,flags);

        bar0_read32(REG_DMA_ENABLE_OUT);
    }
    else
    {
        dev->dma_enabled_in|=1<<channel->index;

        bar0_write32(REG_DMA_ENABLE_IN,dev->dma_enabled_in);

        dev->dma_subchannel_mask_in|=3<<(channel->index<<1);

        spin_unlock_irqrestore(&dev->irq_spinlock,flags);

        bar0_read32(REG_DMA_ENABLE_IN);
    }

    dev->g39_dev.io_stream[stream_index].active=1;

    return 0;
}

static int g39_pci_finish_dma_transfer(struct g39ddc_pci *dev,struct g39_io_stream *io_stream,struct g39_dma_channel *channel)
{
 struct g39_io_buffer *io_buffer;
 uint32_t index;

    if(likely(channel->subchannel[channel->subchannel_index].io_buffer))
    {
        g39_io_buffer_sg_unmap(channel->subchannel[channel->subchannel_index].io_buffer);
        g39_io_stream_put_complete_buffer(io_stream,channel->subchannel[channel->subchannel_index].io_buffer);
        channel->subchannel[channel->subchannel_index].io_buffer=NULL;

        io_buffer=g39_io_stream_get_pending_buffer_nolock(io_stream);

        if(likely(io_buffer!=NULL))
        {
            g39_pci_start_dma_transfer(dev,io_buffer,channel,channel->subchannel_index);
        }
    }

    index=1-channel->subchannel_index;

    if(channel->subchannel[index].io_buffer)
    {
        channel->subchannel_index=index;
        return 1;
    }

    return 0;
}

static void g39_pci_irq_work_handler(struct work_struct *work)
{
 struct g39ddc_pci *dev;
 uint32_t dma_status_in;
 uint32_t dma_status_out;
 unsigned long flags;
 int i,j;
 struct g39_io_stream *io_stream;
 struct g39_dma_channel *channel;

    dev=container_of(work,struct g39ddc_pci,irq_work);

    spin_lock_irqsave(&dev->irq_spinlock,flags);

    dma_status_in=dev->dma_status_in & dev->dma_subchannel_mask_in;
    dma_status_out=dev->dma_status_out & dev->dma_subchannel_mask_out;

    dev->dma_status_in=0;
    dev->dma_status_out=0;

    spin_unlock_irqrestore(&dev->irq_spinlock,flags);

    for(i=0;i<G39_DMA_CHANNEL_IN_COUNT;i++,dma_status_in>>=2)
    {
        if(dma_status_in&3)
        {
            io_stream=dev->g39_dev.io_stream+G39_IO_STREAM_IN_FIRST+i;

            down_read(&io_stream->rw_semaphore);

            if(io_stream->active)
            {
                channel=dev->dma_channel+G39_IO_STREAM_IN_FIRST+i;

                g39_mutex_lock(&channel->mutex);

                for(j=0;j<2;j++)
                {
                    if(dma_status_in & (1<<channel->subchannel_index))
                    {
                        if(!g39_pci_finish_dma_transfer(dev,io_stream,channel))
                        {
                            break;
                        }
                    }
                    else
                    {
                        break;
                    }
                }

                g39_mutex_unlock(&channel->mutex);
            }

            up_read(&io_stream->rw_semaphore);
        }
    }

    for(i=0;i<G39_DMA_CHANNEL_OUT_COUNT;i++,dma_status_out>>=2)
    {
        if(dma_status_out&3)
        {
            io_stream=dev->g39_dev.io_stream+G39_IO_STREAM_OUT_FIRST+i;

            down_read(&io_stream->rw_semaphore);

            if(io_stream->active)
            {
                channel=dev->dma_channel+G39_IO_STREAM_OUT_FIRST+i;

                g39_mutex_lock(&channel->mutex);

                for(j=0;j<2;j++)
                {
                    if(dma_status_out & (1<<channel->subchannel_index))
                    {
                        if(!g39_pci_finish_dma_transfer(dev,io_stream,channel))
                        {
                            break;
                        }
                    }
                    else
                    {
                        break;
                    }
                }

                g39_mutex_unlock(&channel->mutex);
            }

            up_read(&io_stream->rw_semaphore);
        }
    }
}

static int g39_pci_if_start(struct g39ddc_pci *dev,uint16_t period)
{
 int status;

    dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].buffer_size=65536*sizeof(int16_t);
    dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].bytes_per_sec=65536*sizeof(int16_t)*1000/period;

    bar0_write32(REG_IF_BUFF_CTRL,period);

    status=g39_pci_enable_dma_channel(dev,G39_IO_STREAM_IN_IF);

    if(unlikely(status))
    {
        bar0_write32(REG_IF_BUFF_CTRL,0);
    }

    return status;
}

static void g39_pci_if_stop(struct g39ddc_pci *dev)
{
    if(likely(dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].active))
    {
        g39_pci_disable_dma_channel(dev,G39_IO_STREAM_IN_IF);

        bar0_write32(REG_IF_BUFF_CTRL,0);
    }
}

static int g39_pci_put_buffer(struct g39ddc_pci *dev,uint32_t stream_index,struct g39_io_buffer *io_buffer)
{
 int status;
 struct g39_dma_channel *channel;

    channel=dev->dma_channel+stream_index;

    status=g39_io_buffer_sg_map(io_buffer,dev->pci_dev,channel->out?PCI_DMA_TODEVICE:PCI_DMA_FROMDEVICE);

    if(unlikely(status))
    {
        return status;
    }

    g39_mutex_lock(&channel->mutex);

    if(channel->subchannel[0].io_buffer==NULL)
    {
        if(channel->subchannel[1].io_buffer==NULL)
        {
            channel->subchannel_index=0;
        }

        g39_pci_start_dma_transfer(dev,io_buffer,channel,0);
    }
    else
    {
        if(channel->subchannel[1].io_buffer==NULL)
        {
            g39_pci_start_dma_transfer(dev,io_buffer,channel,1);
        }
        else
        {
            g39_io_stream_put_pending_buffer_nolock(dev->g39_dev.io_stream+stream_index,io_buffer);
        }
    }

    g39_mutex_unlock(&channel->mutex);

    return 0;
}

static int g39_pci_get_adc_level(struct g39ddc_pci *dev,uint16_t *level)
{
    *level=(uint16_t)bar0_read32(REG_ADC_LEVEL);
    return 0;
}

static int g39_pci_ddc1_setup(struct g39ddc_pci *dev,uint32_t channel,int enabled,uint32_t *coeffs)
{
 struct g39_ddc1_params *params;
 uint32_t reg_offset;
 uint32_t address;
 int i;

    params=dev->g39_dev.ddc1_params+channel;

    reg_offset=channel*REG_DDC1_COUNT*4;

    if(unlikely(coeffs!=NULL))
    {
        bar0_write32(REG_DDC1_CTRL1+reg_offset,params->control_word & 0xFFFE);
    }

    bar0_write32(REG_DDC1_TUNING_WORD+reg_offset,params->tuning_word);
    bar0_write32(REG_DDC1_CIC_CTRL1+reg_offset,params->cic1 | (((uint32_t)params->cic2)<<16));
    bar0_write32(REG_DDC1_FIR_CTRL1+reg_offset,(params->fir_len>>1) | (((uint32_t)params->fir_control)<<16));

    if(unlikely(coeffs!=NULL))
    {
        address=(channel<<28);

        bar0_write32(REG_FIR_COEFF,address | (1<<27));

        for(i=(params->fir_len>>1)-1;i>=0;i--)
        {
            bar0_write32(REG_FIR_COEFF,address | (coeffs[i]&0x3FFFF));
        }
    }

    if(enabled)
    {
        bar0_write32(REG_DDC1_CTRL1+reg_offset,params->control_word);
    }

    return 0;
}

static int g39_pci_ddc1_start(struct g39ddc_pci *dev,uint32_t channel,int32_t *coeffs)
{
 int status;

    g39_pci_ddc1_setup(dev,channel,1,coeffs);

    status=g39_pci_enable_dma_channel(dev,G39_IO_STREAM_IN_DDC1_0+channel);

    if(unlikely(status))
    {
        g39_pci_ddc1_setup(dev,channel,0,NULL);
    }

    return status;
}

static void g39_pci_ddc1_stop(struct g39ddc_pci *dev,uint32_t channel)
{
    if(likely(dev->g39_dev.io_stream[G39_IO_STREAM_IN_DDC1_0+channel].active))
    {
        g39_pci_disable_dma_channel(dev,G39_IO_STREAM_IN_DDC1_0+channel);

        g39_pci_ddc1_setup(dev,channel,0,NULL);
    }
}

static int g39_pci_ddc1_update_params(struct g39ddc_pci *dev,uint32_t channel)
{
    return g39_pci_ddc1_setup(dev,channel,1,NULL);
}

static int g39_pci_wd(struct g39ddc_pci *dev)
{
    return ((bar4_read32(GPIO_INPUT_VALUE)>>1)&0x7F)>=68;
}

static int g39_pci_read_config(struct g39ddc_pci *dev)
{
 uint32_t u32;
 uint32_t ctl;
 int i;
 uint8_t to_read;
 uint32_t size;
 uint8_t *p;
 int status;

    status=-EIO;

    size=8192;
    p=dev->g39_dev.config+4;

    u32=bar4_read32(LB_CTL);

    if((u32 & 0x10000)==0)
    {
        u32|=0x10000;
        bar4_write32(LB_CTL,u32);
    }

    ctl = (G39_I2C_DIV_A << 14) | (G39_I2C_DIV_B << 8) | 0x4E;

    bar4_write32(TWI_CTRL,ctl);

    ctl=bar4_read32(TWI_CTRL);

    for(i=0;i<G39_I2C_MAX_RETRY;i++)
    {
        if((bar4_read32(TWI_STATUS) & 0x0100)==0)
        {
            break;
        }
    }

    if(unlikely(i>=G39_I2C_MAX_RETRY))
    {
        G39_DEV_ERROR(&dev->pci_dev->dev,"read config:error:i2c is busy\n");
        goto error_exit;
    }

    udelay(100);

    bar4_read32(TWI_IRT_STATUS);

    bar4_write32(TWI_DATA,0);
    bar4_write32(TWI_DATA,0);

    bar4_write32(TWI_ADDRESS,0x51);

    for(i=0;i<G39_I2C_MAX_RETRY;i++)
    {
        u32=bar4_read32(TWI_IRT_STATUS);

        if(u32 & 0x01)
        {
            break;
        }

        if(u32 & 0x04)
        {
            G39_DEV_ERROR(&dev->pci_dev->dev,"read config:i2c:addr:nack\n");
            goto error_exit;
        }

        if(u32 & 0x08)
        {
            G39_DEV_ERROR(&dev->pci_dev->dev,"read config:i2c:addr:transfer timeout\n");
            goto error_exit;
        }
    }

    if(unlikely(i>=G39_I2C_MAX_RETRY))
    {
        G39_DEV_ERROR(&dev->pci_dev->dev,"read config:i2c:addr:transfer completion timeout\n");
        goto error_exit;
    }

    ctl|=0x01;

    bar4_write32(TWI_CTRL,ctl);

	while(size!=0)
    {
        if(size>G39_I2C_FIFO_SIZE)
        {
            to_read=G39_I2C_FIFO_SIZE;
            size-=G39_I2C_FIFO_SIZE;
        }
        else
        {
            to_read=size;
            size=0;
        }

        bar4_write32(TWI_TR_SIZE,to_read);

        bar4_write32(TWI_ADDRESS,0x51);

	    for(i=0;i<G39_I2C_MAX_RETRY;i++)
	    {
	        if(bar4_read32(TWI_IRT_STATUS) & 0x01)
	        {
	            break;
	        }
	    }

	    if(unlikely(i>=G39_I2C_MAX_RETRY))
	    {
	        G39_DEV_ERROR(&dev->pci_dev->dev,"i2c:data:transfer completion timeout\n");
	        goto error_exit;
	    }

        while(to_read--)
        {
            *p++=(uint8_t)(bar4_read32(TWI_DATA) & 0xFF);
		}
    }

    status=0;

    error_exit:

    bar4_write32(TWI_CTRL,ctl | 0x01);
    bar4_read32(TWI_CTRL);
    //bar4_write32(LB_CTL,bar4_read32(LB_CTL) & (~0x10000));
    //bar4_read32(LB_CTL);

    return status;
}

static int g39_pci_ddc2_start(struct g39ddc_pci *dev,uint32_t channel,struct g39_ddc2_start_params *params)
{
 uint32_t reg_offset;
 uint32_t address,len,i,*coeffs;
 int status;

    status=g39_pci_enable_dma_channel(dev,G39_IO_STREAM_IN_DDC2_0+channel);

    if(unlikely(status))
    {
        return status;
    }

    reg_offset=channel*REG_DDC2_COUNT*4;

    bar0_write32(REG_DDC2_FIR1_CONTROL1+reg_offset,params->fir1.ctrl1.u32 & (~1));

    if(!params->fir1.ctrl1.bypass)
    {
        bar0_write32(REG_DDC2_FIR1_CONTROL2+reg_offset,params->fir1.ctrl2.u32);

        address=(channel+2)<<28;

        bar0_write32(REG_FIR_COEFF,address | (1<<27));

        len=(params->fir1.ctrl2.len+1)>>1;
        coeffs=params->fir1.coeffs;

        for(i=0;i<len;i++)
        {
            bar0_write32(REG_FIR_COEFF,address | coeffs[i]);
        }
    }

    bar0_write32(REG_DDC2_FIR2_CONTROL1+reg_offset,params->fir2.ctrl1.u32 & (~1)); //disabled

    if(!params->fir2.ctrl1.bypass)
    {
        bar0_write32(REG_DDC2_FIR2_CONTROL2+reg_offset,params->fir2.ctrl2.u32);

        address=(channel+4)<<28;

        bar0_write32(REG_FIR_COEFF,address | (1<<27));

        len=(params->fir2.ctrl2.len+1)>>1;
        coeffs=params->fir2.coeffs;

        for(i=0;i<len;i++)
        {
            bar0_write32(REG_FIR_COEFF,address | coeffs[i]);
        }
    }

    bar0_write32(REG_DDC2_RETIMER+reg_offset,params->retimer.u32);

    bar0_write32(REG_DDC2_FIR1_CONTROL1+reg_offset,params->fir1.ctrl1.u32);
    bar0_write32(REG_DDC2_FIR2_CONTROL1+reg_offset,params->fir2.ctrl1.u32);
    bar0_write32(REG_DDC2_SHIFT_CONTROL+reg_offset,0x00000001);

    return 0;
}

static void g39_pci_ddc2_stop(struct g39ddc_pci *dev,uint32_t channel)
{
 uint32_t reg_offset;

    if(likely(dev->g39_dev.io_stream[G39_IO_STREAM_IN_DDC2_0+channel].active))
    {
        g39_pci_disable_dma_channel(dev,G39_IO_STREAM_IN_DDC2_0+channel);

        reg_offset=channel*REG_DDC2_COUNT*4;

        bar0_write32(REG_DDC2_SHIFT_CONTROL+reg_offset,0); //disable shifter
        bar0_write32(REG_DDC2_FIR1_CONTROL1+reg_offset,0);
        bar0_write32(REG_DDC2_FIR2_CONTROL1+reg_offset,0);
    }
}

static int g39_pci_ddc2_set_frequency(struct g39ddc_pci *dev,uint32_t channel,uint32_t tuning_word)
{
    bar0_write32(REG_DDC2_SHIFT_TUNING+channel*REG_DDC2_COUNT*4,tuning_word);

    return 0;
}

static int g39_pci_swp_start(struct g39ddc_pci *dev)
{
 int status;

    dev->g39_dev.swp_active=1;

    status=g39_pci_enable_dma_channel(dev,G39_IO_STREAM_IN_IF);

    if(unlikely(status))
    {
        dev->g39_dev.swp_active=0;
    }

    return status;
}

static void g39_pci_swp_stop(struct g39ddc_pci *dev)
{
    if(dev->g39_dev.io_stream[G39_IO_STREAM_IN_IF].active && dev->g39_dev.swp_active)
    {
        g39_pci_disable_dma_channel(dev,G39_IO_STREAM_IN_IF);

        if(dev->g39_dev.state.fpga)
        {
            bar0_write32(REG_IF_BUFF_CTRL,0);
        }

        dev->g39_dev.swp_active=0;
    }
}

static int g39_pci_swp_snapshot(struct g39ddc_pci *dev)
{
    bar0_write32(REG_IF_BUFF_CTRL,dev->g39_dev.swp_settling_time | 0x80000000);

    return 0;
}

static int g39_pci_ddc1_playback_start(struct g39ddc_pci *dev,uint32_t channel,uint32_t divider)
{
 int status;

    status=g39_pci_enable_dma_channel(dev,G39_IO_STREAM_OUT_DDC1_0+channel);

    if(unlikely(status))
    {
        return status;
    }

    bar0_write32(REG_DDC2_INPUT_SWITCH+channel*REG_DDC2_COUNT*4,divider);

    return 0;
}

static void g39_pci_ddc1_playback_stop(struct g39ddc_pci *dev,uint32_t channel)
{
    if(dev->g39_dev.io_stream[G39_IO_STREAM_OUT_DDC1_0+channel].active)
    {
        g39_pci_disable_dma_channel(dev,G39_IO_STREAM_OUT_DDC1_0+channel);

        if(dev->g39_dev.state.fpga)
        {
            bar0_write32(REG_DDC2_INPUT_SWITCH+channel*REG_DDC2_COUNT*4,0);
        }
    }
}

static int g39_pci_dac_start(struct g39ddc_pci *dev,uint32_t dac_control)
{
 int status;

    status=g39_pci_enable_dma_channel(dev,G39_IO_STREAM_OUT_DAC);

    if(unlikely(status))
    {
        return status;
    }

    bar0_write32(REG_DAC_CONTROL,dac_control);

    return 0;
}

static void g39_pci_dac_stop(struct g39ddc_pci *dev)
{
    if(dev->g39_dev.io_stream[G39_IO_STREAM_OUT_DAC].active)
    {
        g39_pci_disable_dma_channel(dev,G39_IO_STREAM_OUT_DAC);

        if(dev->g39_dev.state.fpga)
        {
            bar0_write32(REG_DAC_CONTROL,0);
        }
    }
}

static void g39_pci_cancel_buffers(struct g39ddc_pci *dev,int stream_index)
{
 struct g39_io_stream *io_stream;
 struct g39_io_buffer *io_buffer;
 struct g39_dma_channel *channel;

    io_stream=&dev->g39_dev.io_stream[stream_index];
    channel=&dev->dma_channel[stream_index];

    g39_mutex_lock(&channel->mutex);

    while((io_buffer=g39_io_stream_get_pending_buffer_nolock(io_stream)))
    {
        g39_io_buffer_sg_unmap(io_buffer);
        g39_io_stream_put_complete_buffer(io_stream,io_buffer);
    }

    g39_mutex_unlock(&channel->mutex);
}

static int g39_pci_initialize(struct g39ddc_pci *dev)
{
 uint32_t i;
 uint8_t j,k;
 int status;

    g39_pci_setup(dev);

    bar4_write32(GPIO_DIRECTION_MODE,GPIO_BIT_POWER_STATE|GPIO_BIT_DMA_IRQ|GPIO_BIT_TMP|
            GPIO_BIT_TMP_ERROR|GPIO_BIT_FAN_ERROR);

    gpio_clr(GPIO_BIT_FE_POWER|GPIO_BIT_CTL|GPIO_BIT_FPGA_RESET);

    bar4_write32(GPIO_OUTPUT_ENABLE,GPIO_BIT_FE_POWER|GPIO_BIT_CTL|GPIO_BIT_FPGA_RESET);

    gpio_clr(GPIO_BIT_FE_POWER|GPIO_BIT_CTL|GPIO_BIT_FPGA_RESET);

    msleep(5);

    gpio_set(GPIO_BIT_CTL);

    msleep(15);

    i=(bar4_read32(GPIO_INPUT_VALUE)>>1) & 0x7F;

    if(i==0x64)
    {
        dev->g39_dev.config[1]=1;

        bar4_write32(GPIO_DIRECTION_MODE,GPIO_BIT_POWER_STATE|GPIO_BIT_DMA_IRQ|GPIO_BIT_TMP);

        gpio_clr(GPIO_BIT_FAN_ERROR|GPIO_BIT_TMP_ERROR);

        bar4_write32(GPIO_OUTPUT_ENABLE,GPIO_BIT_FE_POWER|GPIO_BIT_CTL|GPIO_BIT_FPGA_RESET|GPIO_BIT_FAN_ERROR|GPIO_BIT_TMP_ERROR);

        j=0x5B;
        i=bar4_read32(GPIO_OUTPUT_VALUE);

        for(k=0;k<8;k++,j<<=1)
        {
            if(j&0x80)
            {
                i|=GPIO_BIT_FAN_ERROR;
            }
            else
            {
                i&=~GPIO_BIT_FAN_ERROR;
            }

            bar4_write32(GPIO_OUTPUT_VALUE,i);
            i=bar4_read32(GPIO_OUTPUT_VALUE);

            msleep(2);

            i|=GPIO_BIT_TMP_ERROR;
            bar4_write32(GPIO_OUTPUT_VALUE,i);
            i=bar4_read32(GPIO_OUTPUT_VALUE);

            msleep(3);

            i&=~GPIO_BIT_TMP_ERROR;
            bar4_write32(GPIO_OUTPUT_VALUE,i);
            i=bar4_read32(GPIO_OUTPUT_VALUE);

            msleep(2);
        }

        msleep(4);

        dev->g39_dev.config[2]=(bar4_read32(GPIO_INPUT_VALUE)>>1)&0x7F;

        i&=~(GPIO_BIT_TMP_ERROR|GPIO_BIT_FAN_ERROR);
        bar4_write32(GPIO_OUTPUT_VALUE,i);
        bar4_read32(GPIO_OUTPUT_VALUE);

        msleep(3);

        i&=~GPIO_BIT_CTL;
        bar4_write32(GPIO_OUTPUT_VALUE,i);
        bar4_read32(GPIO_OUTPUT_VALUE);

        msleep(1);

        bar4_write32(GPIO_OUTPUT_ENABLE,GPIO_BIT_FE_POWER|GPIO_BIT_FPGA_RESET|GPIO_BIT_CTL);

        bar4_write32(GPIO_DIRECTION_MODE,GPIO_BIT_POWER_STATE|GPIO_BIT_DMA_IRQ|GPIO_BIT_TMP|
                GPIO_BIT_TMP_ERROR|GPIO_BIT_FAN_ERROR);
    }
    else
    {
        bar4_write32(GPIO_OUTPUT_ENABLE,GPIO_BIT_FE_POWER|GPIO_BIT_FPGA_RESET);

        bar4_write32(GPIO_DIRECTION_MODE,GPIO_BIT_POWER_STATE|GPIO_BIT_DMA_IRQ|GPIO_BIT_TMP|
            GPIO_BIT_TMP_ERROR|GPIO_BIT_FAN_ERROR|GPIO_BIT_CTL);
    }


    for(i=0;i<3;i++)
    {
        status=g39_pci_read_config(dev);

        if(status==0)
        {
            break;
        }

        mdelay(100);
    }

    if(unlikely(status))
    {
        G39_DEV_ERROR(&dev->pci_dev->dev,"failed to read configuration data!\n");
        return status;
    }

    dev->g39_dev.func_delete=(g39_func_delete_t)g39_pci_delete;
    dev->g39_dev.func_open=(g39_func_open_t)g39_pci_open;
    dev->g39_dev.func_close=(g39_func_close_t)g39_pci_close;
    dev->g39_dev.func_set_power=(g39_func_set_power_t)g39_pci_set_power;
    dev->g39_dev.func_configure_fpga=(g39_func_configure_fpga_t)g39_pci_configure_fpga;
    dev->g39_dev.func_set_spi_regs=(g39_func_set_spi_regs_t)g39_pci_set_spi_regs;
    dev->g39_dev.func_set_spi2_regs=(g39_func_set_spi2_regs_t)g39_pci_set_spi2_regs;
    dev->g39_dev.func_set_noise_blanker=(g39_func_set_noise_blanker_t)g39_pci_set_noise_blanker;
    dev->g39_dev.func_put_buffer=(g39_func_put_buffer_t)g39_pci_put_buffer;
    dev->g39_dev.func_if_start=(g39_func_if_start_t)g39_pci_if_start;
    dev->g39_dev.func_if_stop=(g39_func_if_stop_t)g39_pci_if_stop;
    dev->g39_dev.func_get_adc_level=(g39_func_get_adc_level_t)g39_pci_get_adc_level;
    dev->g39_dev.func_usb_control1_read=(g39_func_usb_control1_t)g39_pci_usb_control1;
    dev->g39_dev.func_usb_control1_write=(g39_func_usb_control1_t)g39_pci_usb_control1;
    dev->g39_dev.func_usb_control2_read=(g39_func_usb_control2_t)g39_pci_usb_control2;
    dev->g39_dev.func_usb_control2_write=(g39_func_usb_control2_t)g39_pci_usb_control2;
    dev->g39_dev.func_mmap=(g39_func_mmap_t)g39_pci_mmap;
    dev->g39_dev.func_wd=(g39_func_wd_t)g39_pci_wd;
    dev->g39_dev.func_ddc1_start=(g39_func_ddc1_start_t)g39_pci_ddc1_start;
    dev->g39_dev.func_ddc1_stop=(g39_func_ddc1_stop_t)g39_pci_ddc1_stop;
    dev->g39_dev.func_ddc1_update_params=(g39_func_ddc1_update_params_t)g39_pci_ddc1_update_params;
    dev->g39_dev.func_ddc2_start=(g39_func_ddc2_start_t)g39_pci_ddc2_start;
    dev->g39_dev.func_ddc2_stop=(g39_func_ddc2_stop_t)g39_pci_ddc2_stop;
    dev->g39_dev.func_ddc2_set_frequency=(g39_func_ddc2_set_frequency_t)g39_pci_ddc2_set_frequency;
    dev->g39_dev.func_swp_start=(g39_func_swp_start_t)g39_pci_swp_start;
    dev->g39_dev.func_swp_stop=(g39_func_swp_stop_t)g39_pci_swp_stop;
    dev->g39_dev.func_swp_snapshot=(g39_func_swp_snapshot_t)g39_pci_swp_snapshot;
    dev->g39_dev.func_ddc1_playback_start=(g39_func_ddc1_playback_start_t)g39_pci_ddc1_playback_start;
    dev->g39_dev.func_ddc1_playback_stop=(g39_func_ddc1_playback_stop_t)g39_pci_ddc1_playback_stop;
    dev->g39_dev.func_dac_start=(g39_func_dac_start_t)g39_pci_dac_start;
    dev->g39_dev.func_dac_stop=(g39_func_dac_stop_t)g39_pci_dac_stop;
    dev->g39_dev.func_cancel_buffers=(g39_func_cancel_buffers_t)g39_pci_cancel_buffers;

    spin_lock_init(&dev->irq_spinlock);

    INIT_WORK(&dev->irq_work,g39_pci_irq_work_handler);

    for(i=0;i<G39_IO_STREAM_COUNT;i++)
    {
        if(i>=G39_IO_STREAM_OUT_FIRST && i<=G39_IO_STREAM_OUT_LAST)
        {
            dev->dma_channel[i].index=i-G39_IO_STREAM_OUT_FIRST;
            dev->dma_channel[i].out=1;
        }
        else
        {
            dev->dma_channel[i].index=i-G39_IO_STREAM_IN_FIRST;
            dev->dma_channel[i].out=0;
        }

        g39_mutex_init(&dev->dma_channel[i].mutex);
    }

    dev->dma_sg_list_memory_size=((G39_DMA_PAGE_SIZE*(dev->dma_64?5:3)+PAGE_SIZE-1)/PAGE_SIZE)*PAGE_SIZE;
    dev->dma_max_buffer_size=(dev->dma_sg_list_memory_size/
            (dev->dma_64?sizeof(struct g39_sg_segment_64):sizeof(struct g39_sg_segment_32))-1)*PAGE_SIZE;


    return 0;
}

#ifdef CONFIG_PM

static int g39_pci_suspend(struct pci_dev *pci_dev,pm_message_t state)
{
 struct g39ddc_pci *dev;

    dev=pci_get_drvdata(pci_dev);

    if(likely(dev!=NULL))
    {
        G39_DEV_LOCK2;

        if(dev->g39_dev.state.power)
        {
            dev->g39_dev.state.suspended=1;
            g39_set_power(&dev->g39_dev,0,0);
        }

        pci_save_state(pci_dev);

        if(pci_set_power_state(pci_dev,pci_choose_state(pci_dev,state))!=0)
        {
            pci_disable_device(pci_dev);
            dev->disabled=1;
        }

        G39_DEV_UNLOCK2;
    }

    return 0;
}

static int g39_pci_resume(struct pci_dev *pci_dev)
{
 struct g39ddc_pci *dev;
 int state;

    dev=pci_get_drvdata(pci_dev);

    if(likely(dev!=NULL))
    {
        if(dev->disabled)
        {
            state=pci_enable_device(pci_dev);

            if(unlikely(state!=0))
            {
                return state;
            }

            dev->disabled=0;
        }

        state=pci_set_power_state(pci_dev,PCI_D0);

        if(unlikely(state!=0))
        {
            pci_disable_device(pci_dev);
            dev->disabled=1;
            return state;
        }

        pci_restore_state(pci_dev);

        g39_pci_setup(dev);

        if(likely(dev->g39_dev.config[1]))
        {
            bar4_write32(GPIO_OUTPUT_ENABLE,GPIO_BIT_FE_POWER|GPIO_BIT_FPGA_RESET|GPIO_BIT_CTL);

            bar4_write32(GPIO_DIRECTION_MODE,GPIO_BIT_POWER_STATE|GPIO_BIT_DMA_IRQ|GPIO_BIT_TMP|
                    GPIO_BIT_TMP_ERROR|GPIO_BIT_FAN_ERROR);
        }
        else
        {
            bar4_write32(GPIO_OUTPUT_ENABLE,GPIO_BIT_FE_POWER|GPIO_BIT_FPGA_RESET);

            bar4_write32(GPIO_DIRECTION_MODE,GPIO_BIT_POWER_STATE|GPIO_BIT_DMA_IRQ|GPIO_BIT_TMP|
                GPIO_BIT_TMP_ERROR|GPIO_BIT_FAN_ERROR|GPIO_BIT_CTL);
        }
    }

    return 0;
}

#endif //CONFIG_PM


static struct pci_device_id g39_pci_id_table[]=
{
    {.vendor=G39_PCI_VENDOR,.device=G39_PCI_DEVICE,.subvendor=G39_PCI_SUBSYSTEM_VENDOR,.subdevice=G39_PCI_SUBSYSTEM_DEVICE},
    {},
};

static struct pci_driver g39_pci_driver=
{
    .name=G39_DEVICE_NAME,
    .probe=g39_pci_probe,
    .remove=g39_pci_remove,
    .id_table=g39_pci_id_table,
#ifdef CONFIG_PM
    .suspend=g39_pci_suspend,
    .resume=g39_pci_resume
#endif
};

MODULE_DEVICE_TABLE(pci,g39_pci_id_table);

#endif //__G39_PCI__

static int g39_fo_open(struct inode *node, struct file *file)
{
 uint32_t index;
 int status;
 struct g39ddc *dev;

    index=iminor(file->f_path.dentry->d_inode);

    G39_REG_LOCK;

    dev=g39_registered_devices[index];

    if(unlikely(dev==NULL))
    {
        G39_REG_UNLOCK;
        return -ENXIO;
    }

    get_device(&dev->dev);

    G39_DEV_LOCK;

    if(unlikely(dev->state.open))
    {
        G39_DEV_UNLOCK;
        G39_REG_UNLOCK;
        put_device(&dev->dev);
        return -EBUSY;
    }

    dev->state.open=1;

    G39_DEV_UNLOCK;

    dev->workqueue=create_singlethread_workqueue(G39_DEVICE_NAME);

    if(unlikely(dev->workqueue==NULL))
    {
        dev->state.open=0;
        G39_REG_UNLOCK;
        put_device(&dev->dev);
        return -EIO;
    }

    G39_DEV_LOCK;

    status=dev->func_open(dev);

    if(unlikely(status<0))
    {
        dev->state.open=0;
    }

    G39_DEV_UNLOCK;

    if(likely(status>=0))
    {
        file->private_data=dev;

        dev->wd_counter=0;

        dev->wd_timer.expires=jiffies+2*HZ;
        add_timer(&dev->wd_timer);
    }
    else
    {
        destroy_workqueue(dev->workqueue);
        dev->workqueue=NULL;
    }

    G39_REG_UNLOCK;

    if(unlikely(status<0))
    {
        put_device(&dev->dev);
    }

    return status;
}

static int g39_fo_release(struct inode *node, struct file *file)
{
 struct g39ddc *dev;

    dev=file->private_data;

    if(likely(dev!=NULL))
    {
        G39_DEV_LOCK;

        g39_set_power(dev,0,0);

        dev->state.open=0;

        dev->func_close(dev);

        wake_up_all(&dev->power_wait_queue);

        G39_DEV_UNLOCK;

        del_timer_sync(&dev->wd_timer);

        if(likely(dev->workqueue!=NULL))
        {
            flush_workqueue(dev->workqueue);
            destroy_workqueue(dev->workqueue);
            dev->workqueue=NULL;
        }

        put_device(&dev->dev);

        file->private_data=NULL;
    }

    return 0;
}



static int __init g39_init(void)
{
 int status;

    g39_mutex_init(&g39_reg_mutex);

    G39_INFO("init " G39_DESCRIPTION"\n");

    status=alloc_chrdev_region(&g39_dev0,0,G39_MAX_DEVICE_COUNT,G39_DEVICE_NAME);

    if(status<0)
    {
        G39_ERROR("unable to allocate chrdev region!\n");
        goto error_mutex_destroy;
    }

    status=class_register(&g39_class);

    if(status<0)
    {
        G39_ERROR("failed to register class!\n");
        goto error_unregister_chrdev_region;
    }

#ifdef __G39_USB__

    status=usb_register(&g39_usb_driver);

    if(unlikely(status<0))
    {
        G39_ERROR("failed to register usb driver!\n");
        goto error_class_unregister;
    }

#endif //__G39_USB__

#ifdef __G39_PCI__
    status=pci_register_driver(&g39_pci_driver);

    if(unlikely(status<0))
    {
        G39_ERROR("failed to register pci driver!\n");
        goto error_usb_deregister;
    }
#endif //__G39_PCI__

    return 0;

    error_usb_deregister:
#ifdef __G39_USB__
        usb_deregister(&g39_usb_driver);

    error_class_unregister:
#endif
        class_unregister(&g39_class);

    error_unregister_chrdev_region:
        unregister_chrdev_region(g39_dev0,G39_MAX_DEVICE_COUNT);

    error_mutex_destroy:
        g39_mutex_destroy(&g39_reg_mutex);

    return status;
}

static void __exit g39_exit(void)
{
#ifdef __G39_PCI__
    pci_unregister_driver(&g39_pci_driver);
#endif //__G39_PCI__

#ifdef __G39_USB__
    usb_deregister(&g39_usb_driver);
#endif //__G39_USB__

    class_unregister(&g39_class);
    unregister_chrdev_region(g39_dev0,G39_MAX_DEVICE_COUNT);

    g39_mutex_destroy(&g39_reg_mutex);

    G39_INFO("exit " G39_DESCRIPTION "\n");
}


static int g39_fo_mmap(struct file *file,struct vm_area_struct *vma)
{
 struct g39ddc *dev;

    dev=file->private_data;

    if(unlikely(dev==NULL || !dev->state.connected))
    {
        return -ENODEV;
    }

    return dev->func_mmap(dev,vma);
}

#ifdef HAVE_UNLOCKED_IOCTL
static long g39_fo_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
#else
static int g39_fo_ioctl(struct inode *node,struct file *file, unsigned int cmd, unsigned long arg)
#endif //HAVE_UNLOCKED_IOCTL
{
 int status;
 struct g39ddc *dev;
 uint8_t byte;
 uint16_t word;
 uint32_t dword;
 struct g39_spi2_registers spi2;
 struct g39_spi_registers spi;
 struct g39_usb_control1 usb_control1;
 struct g39_usb_control2 usb_control2;
 struct g39_fpga_data fpga;
 struct g39_ddc1_start_params ddc1_start_params;
 struct g39_ddc1_playback_params ddc1_playback_params;
 struct g39_dac_params dac_params;

    dev=file->private_data;


    if(unlikely(dev==NULL))
    {
        return -ENODEV;
    }

    switch(cmd)
    {
        case G39_IOCTL_DDC1_0_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_DDC1_0,(void __user*)arg,1,0);
            break;

        case G39_IOCTL_DDC1_1_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_DDC1_1,(void __user*)arg,1,0);
            break;

        case G39_IOCTL_DDC2_0_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_DDC2_0,(void __user*)arg,1,0);
            break;

        case G39_IOCTL_DDC2_1_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_DDC2_1,(void __user*)arg,1,0);
            break;

        case G39_IOCTL_IF_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_IF,(void __user*)arg,1,0);
            break;

        case G39_IOCTL_DAC_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_OUT_DAC,(void __user*)arg,0,0);
            break;

        case G39_IOCTL_DDC1_0_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_IN_DDC1_0,arg,0);
            break;

        case G39_IOCTL_DDC1_1_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_IN_DDC1_1,arg,0);
            break;

        case G39_IOCTL_DDC2_0_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_IN_DDC2_0,arg,0);
            break;

        case G39_IOCTL_DDC2_1_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_IN_DDC2_1,arg,0);
            break;

        case G39_IOCTL_IF_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_IN_IF,arg,0);
            break;

        case G39_IOCTL_DAC_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_OUT_DAC,arg,0);
            break;

        case G39_IOCTL_DDC1_0_PLAYBACK_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_OUT_DDC1_0,(void __user*)arg,0,0);
            break;

        case G39_IOCTL_DDC1_0_PLAYBACK_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_OUT_DDC1_0,arg,0);
            break;

        case G39_IOCTL_DDC1_1_PLAYBACK_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_OUT_DDC1_1,(void __user*)arg,0,0);
            break;

        case G39_IOCTL_DDC1_1_PLAYBACK_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_OUT_DDC1_1,arg,0);
            break;

        case G39_IOCTL_GET_ADC_LEVEL:
            status=g39_get_adc_level(dev,&word);

            if(likely(status==0))
            {
                status=put_user(word,(uint16_t __user*)arg);
            }
            break;

        case G39_IOCTL_SWP_GET_BUFFER:
            status=g39_get_complete_buffer(dev,G39_IO_STREAM_IN_IF,arg,1);
            break;

        case G39_IOCTL_SWP_PUT_BUFFER:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_IF,(void __user*)arg,1,1);
            break;

        case G39_IOCTL_SWP_SNAPSHOT:
            status=g39_swp_snapshot(dev);
            break;

        case G39_IOCTL_DDC1_0_SET_FREQUENCY:
            if(unlikely(get_user(dword,(uint32_t __user*)arg)<0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_set_frequency(dev,0,dword);
            break;

        case G39_IOCTL_DDC1_1_SET_FREQUENCY:
            if(unlikely(get_user(dword,(uint32_t __user*)arg)<0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_set_frequency(dev,1,dword);
            break;

        case G39_IOCTL_DDC2_0_SET_FREQUENCY:
            if(unlikely(get_user(dword,(uint32_t __user*)arg)<0))
            {
                return -EFAULT;
            }

            status=g39_ddc2_set_frequency(dev,0,dword);
            break;

        case G39_IOCTL_DDC2_1_SET_FREQUENCY:
            if(unlikely(get_user(dword,(uint32_t __user*)arg)<0))
            {
                return -EFAULT;
            }

            status=g39_ddc2_set_frequency(dev,1,dword);
            break;

        case G39_IOCTL_SET_SPI2_REGS:
            if(unlikely(copy_from_user(&spi2,(struct g39_spi2_registers __user*)arg,sizeof(spi2))!=0))
            {
                return -EFAULT;
            }

            status=g39_set_spi2_regs(dev,(struct g39_spi2_register __user*)spi2.regs,spi2.count,spi2.id);
            break;

        case G39_IOCTL_SET_SPI_REGS:
            if(unlikely(copy_from_user(&spi,(struct g39_spi_registers __user*)arg,sizeof(spi))!=0))
            {
                return -EFAULT;
            }

            status=g39_set_spi_regs(dev,(struct g39_spi_register __user*)spi.regs,spi.count,spi.id);
            break;

        case G39_IOCTL_DDC1_0_START:
            if(unlikely(copy_from_user(&ddc1_start_params,(void __user*)arg,sizeof(ddc1_start_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_start(dev,0,&ddc1_start_params.params,(void __user*)ddc1_start_params.coeffs,
                    ddc1_start_params.buffer_size,ddc1_start_params.bytes_per_sec);
            break;

        case G39_IOCTL_DDC1_0_STOP:
            g39_ddc1_stop(dev,0);
            status=0;
            break;

        case G39_IOCTL_DDC1_1_START:
            if(unlikely(copy_from_user(&ddc1_start_params,(void __user*)arg,sizeof(ddc1_start_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_start(dev,1,&ddc1_start_params.params,(void __user*)ddc1_start_params.coeffs,
                    ddc1_start_params.buffer_size,ddc1_start_params.bytes_per_sec);
            break;

        case G39_IOCTL_DDC1_1_STOP:
            g39_ddc1_stop(dev,1);
            status=0;
            break;

        case G39_IOCTL_DAC_START:
            if(unlikely(copy_from_user(&dac_params,(void __user*)arg,sizeof(dac_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_dac_start(dev,&dac_params);
            break;

        case G39_IOCTL_DAC_STOP:
            g39_dac_stop(dev);
            status=0;
            break;

        case G39_IOCTL_DDC1_0_PLAYBACK_START:
            if(unlikely(copy_from_user(&ddc1_playback_params,(void __user*)arg,sizeof(ddc1_playback_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_playback_start(dev,0,&ddc1_playback_params);
            break;

        case G39_IOCTL_DDC1_0_PLAYBACK_STOP:
            g39_ddc1_playback_stop(dev,0);
            status=0;
            break;

        case G39_IOCTL_DDC1_1_PLAYBACK_START:
            if(unlikely(copy_from_user(&ddc1_playback_params,(void __user*)arg,sizeof(ddc1_playback_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_playback_start(dev,1,&ddc1_playback_params);
            break;

        case G39_IOCTL_DDC1_1_PLAYBACK_STOP:
            g39_ddc1_playback_stop(dev,1);
            status=0;
            break;

        case G39_IOCTL_DDC2_0_START:
            status=g39_ddc2_start(dev,0,(void __user*)arg);
            break;

        case G39_IOCTL_DDC2_0_STOP:
            g39_ddc2_stop(dev,0);
            status=0;
            break;

        case G39_IOCTL_DDC2_1_START:
            status=g39_ddc2_start(dev,1,(void __user*)arg);
            break;

        case G39_IOCTL_DDC2_1_STOP:
            g39_ddc2_stop(dev,1);
            status=0;
            break;

        case G39_IOCTL_IF_START:
            if(unlikely(get_user(word,(uint16_t __user*)arg)<0))
            {
                return -EFAULT;
            }

            status=g39_if_start(dev,word);
            break;

        case G39_IOCTL_IF_STOP:
            g39_if_stop(dev);

            status=0;
            break;

        case G39_IOCTL_SWP_START:
            if(unlikely(get_user(word,(uint16_t __user*)arg)<0))
            {
                return -EFAULT;
            }

            status=g39_swp_start(dev,word);
            break;

        case G39_IOCTL_SWP_STOP:
            g39_swp_stop(dev);

            status=0;
            break;

        case G39_IOCTL_GET_STATE:
            G39_DEV_LOCK;

            status=put_user(dev->state,(struct g39_state __user*)arg);

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_SET_NOISE_BLANKER:
            if(unlikely(get_user(word,(uint16_t __user*)arg)<0))
            {
                return -EFAULT;
            }

            status=g39_set_noise_blanker(dev,word);
            break;

        case G39_IOCTL_IF_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_IF);
            status=0;
            break;

        case G39_IOCTL_DDC1_0_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_DDC1_0);
            status=0;
            break;

        case G39_IOCTL_DDC1_1_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_DDC1_1);
            status=0;
            break;

        case G39_IOCTL_DDC2_0_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_DDC2_0);
            status=0;
            break;

        case G39_IOCTL_DDC2_1_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_DDC2_1);
            status=0;
            break;

        case G39_IOCTL_USB_CONTROL1_READ:
            if(unlikely(copy_from_user(&usb_control1,(struct g39_usb_control1 __user*)arg,sizeof(usb_control1))!=0))
            {
                return -EFAULT;
            }

            G39_DEV_LOCK;

            status=dev->func_usb_control1_read(dev,usb_control1.params,(void __user*)usb_control1.buffer,usb_control1.size,usb_control1.transferred);

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_USB_CONTROL1_WRITE:
            if(unlikely(copy_from_user(&usb_control1,(struct g39_usb_control1 __user*)arg,sizeof(usb_control1))!=0))
            {
                return -EFAULT;
            }

            G39_DEV_LOCK;

            status=dev->func_usb_control1_write(dev,usb_control1.params,(void __user*)usb_control1.buffer,usb_control1.size,usb_control1.transferred);

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_USB_CONTROL2_READ:
            if(unlikely(copy_from_user(&usb_control2,(struct g39_usb_control2 __user*)arg,sizeof(usb_control2))!=0))
            {
                return -EFAULT;
            }

            G39_DEV_LOCK;

            status=dev->func_usb_control2_read(dev,(void __user*)usb_control2.buffer,usb_control2.size,usb_control2.transferred);

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_USB_CONTROL2_WRITE:
            if(unlikely(copy_from_user(&usb_control2,(struct g39_usb_control2 __user*)arg,sizeof(usb_control2))!=0))
            {
                return -EFAULT;
            }

            G39_DEV_LOCK;

            status=dev->func_usb_control2_write(dev,(void __user*)usb_control2.buffer,usb_control2.size,usb_control2.transferred);

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_SET_POWER:
            if(unlikely(get_user(byte,(uint8_t __user*)arg)<0))
            {
                return -EFAULT;
            }

            status=g39_set_power(dev,byte,1);
            break;

        case G39_IOCTL_CONFIGURE_FPGA:
            if(unlikely(copy_from_user(&fpga,(struct g39_fpga_data __user*)arg,sizeof(fpga))!=0))
            {
                return -EFAULT;
            }

            status=g39_configure_fpga(dev,(void __user*)fpga.buffer,fpga.size);
            break;

        case G39_IOCTL_GET_CONFIG:
            status=copy_to_user((void __user*)arg,dev->config,sizeof(dev->config))?-EFAULT:0;
            break;

        case G39_IOCTL_WAIT_FOR_POWER_OFF:
            status=g39_wait_for_power_off(dev);
            break;

        default:
            status=-EINVAL;
            break;
    }

    return status;
}

#if defined(CONFIG_COMPAT) && defined(HAVE_COMPAT_IOCTL)

static long g39_fo_ioctl_compat(struct file *file, unsigned int cmd, unsigned long arg)
{
 int status;
 struct g39ddc *dev;
 uint8_t byte;
 uint16_t word;
 uint32_t dword;
 struct g39_spi2_registers_compat spi2;
 struct g39_spi_registers_compat spi;
 struct g39_usb_control1_compat usb_control1;
 struct g39_usb_control2_compat usb_control2;
 struct g39_fpga_data_compat fpga;
 struct g39_ddc1_start_params_compat ddc1_start_params;
 struct g39_ddc1_playback_params ddc1_playback_params;
 struct g39_dac_params dac_params;

    dev=file->private_data;

    if(unlikely(dev==NULL))
    {
        return -ENODEV;
    }

    switch(cmd)
    {
        case G39_IOCTL_DDC1_0_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_DDC1_0,compat_ptr(arg),1,0);
            break;

        case G39_IOCTL_DDC1_1_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_DDC1_1,compat_ptr(arg),1,0);
            break;

        case G39_IOCTL_DDC2_0_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_DDC2_0,compat_ptr(arg),1,0);
            break;

        case G39_IOCTL_DDC2_1_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_DDC2_1,compat_ptr(arg),1,0);
            break;

        case G39_IOCTL_IF_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_IF,compat_ptr(arg),1,0);
            break;

        case G39_IOCTL_DAC_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_OUT_DAC,compat_ptr(arg),0,0);
            break;

        case G39_IOCTL_DDC1_0_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_IN_DDC1_0,arg,0);
            break;

        case G39_IOCTL_DDC1_1_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_IN_DDC1_1,arg,0);
            break;

        case G39_IOCTL_DDC2_0_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_IN_DDC2_0,arg,0);
            break;

        case G39_IOCTL_DDC2_1_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_IN_DDC2_1,arg,0);
            break;

        case G39_IOCTL_IF_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_IN_IF,arg,0);
            break;

        case G39_IOCTL_DAC_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_OUT_DAC,arg,0);
            break;

        case G39_IOCTL_DDC1_0_PLAYBACK_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_OUT_DDC1_0,compat_ptr(arg),0,0);
            break;

        case G39_IOCTL_DDC1_0_PLAYBACK_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_OUT_DDC1_0,arg,0);
            break;

        case G39_IOCTL_DDC1_1_PLAYBACK_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_OUT_DDC1_1,compat_ptr(arg),0,0);
            break;

        case G39_IOCTL_DDC1_1_PLAYBACK_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_OUT_DDC1_1,arg,0);
            break;

        case G39_IOCTL_GET_ADC_LEVEL:
            status=g39_get_adc_level(dev,&word);

            if(likely(status==0))
            {
                status=put_user(word,(uint16_t __user*)compat_ptr(arg));
            }
            break;

        case G39_IOCTL_SWP_GET_BUFFER_COMPAT:
            status=g39_get_complete_buffer_compat(dev,G39_IO_STREAM_IN_IF,arg,1);
            break;

        case G39_IOCTL_SWP_PUT_BUFFER_COMPAT:
            status=g39_put_buffer(dev,G39_IO_STREAM_IN_IF,compat_ptr(arg),1,1);
            break;

        case G39_IOCTL_SWP_SNAPSHOT:
            status=g39_swp_snapshot(dev);
            break;

        case G39_IOCTL_DDC1_0_SET_FREQUENCY:
            if(unlikely(get_user(dword,(uint32_t __user*)compat_ptr(arg))<0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_set_frequency(dev,0,dword);
            break;

        case G39_IOCTL_DDC1_1_SET_FREQUENCY:
            if(unlikely(get_user(dword,(uint32_t __user*)compat_ptr(arg))<0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_set_frequency(dev,1,dword);
            break;

        case G39_IOCTL_DDC2_0_SET_FREQUENCY:
            if(unlikely(get_user(dword,(uint32_t __user*)compat_ptr(arg))<0))
            {
                return -EFAULT;
            }

            status=g39_ddc2_set_frequency(dev,0,dword);
            break;

        case G39_IOCTL_DDC2_1_SET_FREQUENCY:
            if(unlikely(get_user(dword,(uint32_t __user*)compat_ptr(arg))<0))
            {
                return -EFAULT;
            }

            status=g39_ddc2_set_frequency(dev,1,dword);
            break;

        case G39_IOCTL_SET_SPI2_REGS_COMPAT:
            if(unlikely(copy_from_user(&spi2,(struct g39_spi2_registers __user*)compat_ptr(arg),sizeof(spi2))!=0))
            {
                return -EFAULT;
            }

            status=g39_set_spi2_regs(dev,(struct g39_spi2_register __user*)compat_ptr(spi2.regs),spi2.count,spi2.id);
            break;

        case G39_IOCTL_SET_SPI_REGS_COMPAT:
            if(unlikely(copy_from_user(&spi,(struct g39_spi_registers __user*)compat_ptr(arg),sizeof(spi))!=0))
            {
                return -EFAULT;
            }

            status=g39_set_spi_regs(dev,(struct g39_spi_register __user*)compat_ptr(spi.regs),spi.count,spi.id);
            break;

        case G39_IOCTL_DDC1_0_START_COMPAT:
            if(unlikely(copy_from_user(&ddc1_start_params,(void __user*)compat_ptr(arg),sizeof(ddc1_start_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_start(dev,0,&ddc1_start_params.params,(void __user*)compat_ptr(ddc1_start_params.coeffs),
                    ddc1_start_params.buffer_size,ddc1_start_params.bytes_per_sec);
            break;

        case G39_IOCTL_DDC1_0_STOP:
            g39_ddc1_stop(dev,0);
            status=0;
            break;

        case G39_IOCTL_DDC1_1_START_COMPAT:
            if(unlikely(copy_from_user(&ddc1_start_params,(void __user*)compat_ptr(arg),sizeof(ddc1_start_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_start(dev,1,&ddc1_start_params.params,(void __user*)compat_ptr(ddc1_start_params.coeffs),
                    ddc1_start_params.buffer_size,ddc1_start_params.bytes_per_sec);
            break;

        case G39_IOCTL_DDC1_1_STOP:
            g39_ddc1_stop(dev,1);
            status=0;
            break;

        case G39_IOCTL_DAC_START:
            if(unlikely(copy_from_user(&dac_params,(void __user*)compat_ptr(arg),sizeof(dac_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_dac_start(dev,&dac_params);
            break;

        case G39_IOCTL_DAC_STOP:
            g39_dac_stop(dev);
            status=0;
            break;

        case G39_IOCTL_DDC1_0_PLAYBACK_START:
            if(unlikely(copy_from_user(&ddc1_playback_params,(void __user*)compat_ptr(arg),sizeof(ddc1_playback_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_playback_start(dev,0,&ddc1_playback_params);
            break;

        case G39_IOCTL_DDC1_0_PLAYBACK_STOP:
            g39_ddc1_playback_stop(dev,0);
            status=0;
            break;

        case G39_IOCTL_DDC1_1_PLAYBACK_START:
            if(unlikely(copy_from_user(&ddc1_playback_params,(void __user*)compat_ptr(arg),sizeof(ddc1_playback_params))!=0))
            {
                return -EFAULT;
            }

            status=g39_ddc1_playback_start(dev,1,&ddc1_playback_params);
            break;

        case G39_IOCTL_DDC1_1_PLAYBACK_STOP:
            g39_ddc1_playback_stop(dev,1);
            status=0;
            break;

        case G39_IOCTL_DDC2_0_START:
            status=g39_ddc2_start(dev,0,(void __user*)compat_ptr(arg));
            break;

        case G39_IOCTL_DDC2_0_STOP:
            g39_ddc2_stop(dev,0);
            status=0;
            break;

        case G39_IOCTL_DDC2_1_START:
            status=g39_ddc2_start(dev,1,(void __user*)compat_ptr(arg));
            break;

        case G39_IOCTL_DDC2_1_STOP:
            g39_ddc2_stop(dev,1);
            status=0;
            break;

        case G39_IOCTL_IF_START:
            if(unlikely(get_user(word,(uint16_t __user*)compat_ptr(arg))<0))
            {
                return -EFAULT;
            }

            status=g39_if_start(dev,word);
            break;

        case G39_IOCTL_IF_STOP:
            g39_if_stop(dev);

            status=0;
            break;

        case G39_IOCTL_SWP_START:
            if(unlikely(get_user(word,(uint16_t __user*)compat_ptr(arg))<0))
            {
                return -EFAULT;
            }

            status=g39_swp_start(dev,word);
            break;

        case G39_IOCTL_SWP_STOP:
            g39_swp_stop(dev);

            status=0;
            break;

        case G39_IOCTL_GET_STATE:
            G39_DEV_LOCK;

            status=put_user(dev->state,(struct g39_state __user*)compat_ptr(arg));

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_SET_NOISE_BLANKER:
            if(unlikely(get_user(word,(uint16_t __user*)compat_ptr(arg))<0))
            {
                return -EFAULT;
            }

            status=g39_set_noise_blanker(dev,word);
            break;

        case G39_IOCTL_DDC1_0_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_DDC1_0);
            status=0;
            break;

        case G39_IOCTL_DDC1_1_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_DDC1_1);
            status=0;
            break;

        case G39_IOCTL_DDC2_0_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_DDC2_0);
            status=0;
            break;

        case G39_IOCTL_DDC2_1_CANCEL_BUFFERS:
            g39_cancel_buffers(dev,G39_IO_STREAM_IN_DDC2_1);
            status=0;
            break;

        case G39_IOCTL_USB_CONTROL1_READ_COMPAT:
            if(unlikely(copy_from_user(&usb_control1,(struct g39_usb_control1 __user*)compat_ptr(arg),sizeof(usb_control1))!=0))
            {
                return -EFAULT;
            }

            G39_DEV_LOCK;

            status=dev->func_usb_control1_read(dev,usb_control1.params,(void __user*)compat_ptr(usb_control1.buffer),usb_control1.size,compat_ptr(usb_control1.transferred));

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_USB_CONTROL1_WRITE_COMPAT:
            if(unlikely(copy_from_user(&usb_control1,(struct g39_usb_control1 __user*)compat_ptr(arg),sizeof(usb_control1))!=0))
            {
                return -EFAULT;
            }

            G39_DEV_LOCK;

            status=dev->func_usb_control1_write(dev,usb_control1.params,(void __user*)compat_ptr(usb_control1.buffer),usb_control1.size,compat_ptr(usb_control1.transferred));

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_USB_CONTROL2_READ_COMPAT:
            if(unlikely(copy_from_user(&usb_control2,(struct g39_usb_control2 __user*)compat_ptr(arg),sizeof(usb_control2))!=0))
            {
                return -EFAULT;
            }

            G39_DEV_LOCK;

            status=dev->func_usb_control2_read(dev,(void __user*)compat_ptr(usb_control2.buffer),usb_control2.size,compat_ptr(usb_control2.transferred));

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_USB_CONTROL2_WRITE_COMPAT:
            if(unlikely(copy_from_user(&usb_control2,(struct g39_usb_control2 __user*)compat_ptr(arg),sizeof(usb_control2))!=0))
            {
                return -EFAULT;
            }

            G39_DEV_LOCK;

            status=dev->func_usb_control2_write(dev,(void __user*)compat_ptr(usb_control2.buffer),usb_control2.size,compat_ptr(usb_control2.transferred));

            G39_DEV_UNLOCK;
            break;

        case G39_IOCTL_SET_POWER:
            if(unlikely(get_user(byte,(uint8_t __user*)compat_ptr(arg))<0))
            {
                return -EFAULT;
            }

            status=g39_set_power(dev,byte,1);
            break;

        case G39_IOCTL_CONFIGURE_FPGA_COMPAT:
            if(unlikely(copy_from_user(&fpga,(struct g39_fpga_data __user*)compat_ptr(arg),sizeof(fpga))!=0))
            {
                return -EFAULT;
            }

            status=g39_configure_fpga(dev,(void __user*)compat_ptr(fpga.buffer),fpga.size);
            break;

        case G39_IOCTL_GET_CONFIG:
            status=copy_to_user((void __user*)compat_ptr(arg),dev->config,sizeof(dev->config))?-EFAULT:0;
            break;

        case G39_IOCTL_WAIT_FOR_POWER_OFF:
            g39_wait_for_power_off(dev);
            status=0;
            break;

        default:
            status=-EINVAL;
            break;
    }

    return status;
}
#endif

module_init(g39_init);
module_exit(g39_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("RADIXON");
MODULE_DESCRIPTION(G39_DESCRIPTION);
